//------------------------------------------------------------------------
// UNIT           : E950Drv1.pas
// CONTENTS       : TE950Driver component
// VERSION        : 1.0
// TARGET         : (Inprise's) Borland Delphi 4.0
// AUTHORS         : Konstantin Grishenko, Andrey Ivanov, Dmitry Myarikianov, Marco Cocco
// STATUS         : Freeware
//------------------------------------------------------------------------
//
// Copyright (c) 1996-98 by Konstantin Grishenko, Andrey Ivanov, Dmitry Myarikianov. All rights reseved.
// Copyright (c) 1996-98 by Marco Cocco. All rights reseved.
// Copyright (c) 1996-98 by d3k Software Company. All rights reseved.
//
// NO REPRESENTATIONS ARE MADE ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
//    PURPOSE. IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
//   NEITHER MARCO COCCO OR D3K SHALL BE LIABLE FOR ANY DAMAGES SUFFERED BY
//                          THE USE OF THIS SOFTWARE.
//

unit E950Drv1;

interface

uses
  // Delphi units
  Windows, Messages, SysUtils, Classes, Forms, Controls, E950Const,
  Graphics, Dialogs, ExtCtrls, stdctrls, Led,
  E950DriverXC_TLB;

  // ComDrv32 units

const
  BByte = Chr(2);
  EByte = Chr(3);
  DByte = Chr($1C);
const
  RELEASE_NOCLOSE_PORT = HFILE(INVALID_HANDLE_VALUE - 1);

//------------------------------------------------------------------------
// Property types
//------------------------------------------------------------------------


type
  TE950Driver = class;
  TE950NotifyEvent = procedure(E950: TE950Driver) of object;
  TE950ErrorEvent = procedure(Error: string) of object;
  // Baud Rates (custom or 110...256k bauds)
  TBaudRate = (brCustom,
    br110, br300, br600, br1200, br2400, br4800,
    br9600, br14400, br19200, br38400, br56000,
    br57600, br115200, br128000, br256000);
  // Port Numbers ( custom or COM1..COM16 )
  TPortNumber = (pnCustom,
    pnCOM1, pnCOM2, pnCOM3, pnCOM4, pnCOM5, pnCOM6, pnCOM7,
    pnCOM8, pnCOM9, pnCOM10, pnCOM11, pnCOM12, pnCOM13,
    pnCOM14, pnCOM15, pnCOM16);
  // Data bits ( 5, 6, 7, 8 )
  TDataBits = (db5BITS, db6BITS, db7BITS, db8BITS);
  // Stop bits ( 1, 1.5, 2 )
  TStopBits = (sb1BITS, sb1HALFBITS, sb2BITS);
  // Parity ( None, odd, even, mark, space )
  TParity = (ptNONE, ptODD, ptEVEN, ptMARK, ptSPACE);
  // Hardware Flow Control ( None, None + RTS always on, RTS/CTS )
  THwFlowControl = (hfNONE, hfNONERTSON, hfRTSCTS);
  // Software Flow Control ( None, XON/XOFF )
  TSwFlowControl = (sfNONE, sfXONXOFF);
  // What to do with incomplete (incoming) packets ( Discard, Pass )
  TPacketMode = (pmDiscard, pmPass);

//------------------------------------------------------------------------
// Event types
//------------------------------------------------------------------------


  // RX event ( packet mode disabled )
  TReceiveDataEvent = procedure(Sender: TObject; DataPtr: pointer; DataSize: DWORD) of object;
  // RX event ( packed mode enabled )
  TReceivePacketEvent = procedure(Sender: TObject; Packet: pointer; DataSize: DWORD) of object;

//------------------------------------------------------------------------
// Other types
//------------------------------------------------------------------------


  // Line status ( Clear To Send, Data Set Ready, Ring, Carrier Detect )
  TLineStatus = (lsCTS, lsDSR, lsRING, lsCD);
  // Set of line status
  TLineStatusSet = set of TLineStatus;

//------------------------------------------------------------------------
// Constants
//------------------------------------------------------------------------

  TPrnStates = record
    psError,
      psOff,
      psLogLine,
      psCheckLine,
      psBuffFull,
      psBuffEmpty,
      psBeginDok,
      psEndDok,
      psDokSmall,
      psLineNo,
      psAllError: Boolean;
  end;
  PPrnStates = ^TPrnStates;

  TFiskalStates = record
    fsErrorFMem,
      fsErrorWMem,
      fsBattaryLow,
      fsUnknowCommand,
      fsErrorField,
      fsErrorCommand,
      fsOverTotal,
      fsFMemFull,
      fsFMemSoonFull,
      fsCertificated,
      fsFiskalited,
      fsErrorPassDate,
      fsDokCheckOpened,
      fsCheckOpened,
      fsDokOpened,
      fsAllError: Boolean;
  end;
  PFiskalStates = ^TFiskalStates;

  TStateRegistrator = record
    Printer: TPrnStates;
    Fiskal: TFiskalStates;
  end;


 //  
 // TTypeCheck = (Service, ServiceBack, Payment, PaymentBack, Cashe, CasheBack);

 //
  TFontPrn = (Bold, DblH, DblW, UndLine);

 //  
 // TTypeDok = (ServiceDok, ServiceBackDok, PaymentDok, PaymentBackDok, CasheDok, CasheBackDok);
 //
 //  TTypeStrFDok = (Sale, ASale);//($47,$67,$53,$73,$44,$64,$52,$72,50,$70,$57,$74);

//------------------------------------------------------------------------
// TCommPortDriver component
//------------------------------------------------------------------------
  TE950Driver = class(TCustomPanel)
  private
    FOverlapped: TOVERLAPPED;
    FLst: TStrings;
    // Device Handle ( File Handle )
    FHandle: HFILE;
    // # of the COM port to use, or pnCustom to use custom port name
    FPort: TPortNumber;
    // Custom port name ( usually '\\.\COMn', with n = 1..x )
    FPortName: string;
    // COM Port speed (brXXX)
    FBaudRate: TBaudRate;
    // Baud rate ( actual numeric value )
    FBaudRateValue: DWORD;
    // Data bits size (dbXXX)
    FDataBits: TDataBits;
    // How many stop bits to use (sbXXX)
    FStopBits: TStopBits;
    // Type of parity to use (ptXXX)
    FParity: TParity;
    // Type of hw handshaking (hw flow control) to use (hfXXX)
    FHwFlow: THwFlowControl;
    // Type of sw handshaking (sw flow control) to use (sFXXX)
    FSwFlow: TSwFlowControl;
    // Size of the input buffer
    FInBufSize: DWORD;
    // Size of the output buffer
    FOutBufSize: DWORD;
    // Event to raise on data reception (asynchronous)
    FOnReceiveData: TReceiveDataEvent;
    // Event to raise on packet reception (asynchronous)
    FOnReceivePacket: TReceivePacketEvent;
    // ms of delay between COM port pollings
    FPollingDelay: word;
    // Specifies if the DTR line must be enabled/disabled on connect
    FEnableDTROnOpen: boolean;
    // Output timeout - milliseconds
    FOutputTimeout: word;
    // Timeout for ReadData
    FInputTimeout: DWORD;
    // Set to TRUE to prevent hangs when no device connected or
    // device is OFF
    FCkLineStatus: boolean;
    // This is used for the timer
    FNotifyWnd: HWND;
    // Temporary buffer (RX) - used internally
    FTempInBuffer: pointer;
    // Time of the first byte of current RX packet
//    FFirstByteOfPacketTime     : DWORD;
    // Number of RX polling timer pauses
    FRXPollingPauses: integer;
    FEnumComm: Byte;
    FPassword: WideString;
    FInspPassword: WideString;
    FStateRegistrator: TStateRegistrator;
    FCritSect: TRTLCriticalSection;
    FErrorStr: WideString;
   { Events }
    FBeforeOpenCheck: TE950NotifyEvent;
    FAfterOpenCheck: TE950NotifyEvent;
    FBeforePrnTxtCheck: TE950NotifyEvent;
    FAfterPrnTxtCheck: TE950NotifyEvent;
    FBeforePrnStrCheck: TE950NotifyEvent;
    FAfterPrnStrCheck: TE950NotifyEvent;
    FBeforePrnSubTotalCheck: TE950NotifyEvent;
    FAfterPrnSubTotalCheck: TE950NotifyEvent;
    FBeforePrnTotalCheck: TE950NotifyEvent;
    FAfterPrnTotalCheck: TE950NotifyEvent;
    FBeforeCloseCheck: TE950NotifyEvent;
    FAfterCloseCheck: TE950NotifyEvent;
    FBeforeOpenNCheck: TE950NotifyEvent;
    FAfterOpenNCheck: TE950NotifyEvent;
    FBeforePrnTxtNCheck: TE950NotifyEvent;
    FAfterPrnTxtNCheck: TE950NotifyEvent;
    FBeforeCloseNCheck: TE950NotifyEvent;
    FAfterCloseNCheck: TE950NotifyEvent;
    FBeforeOpenDok: TE950NotifyEvent;
    FAfterOpenDok: TE950NotifyEvent;
    FBeforePrnTxtDok: TE950NotifyEvent;
    FAfterPrnTxtDok: TE950NotifyEvent;
    FBeforePrnStrDok: TE950NotifyEvent;
    FAfterPrnStrDok: TE950NotifyEvent;
    FBeforePrnSubTotalDok: TE950NotifyEvent;
    FAfterPrnSubTotalDok: TE950NotifyEvent;
    FBeforePrnTotalDok: TE950NotifyEvent;
    FAfterPrnTotalDok: TE950NotifyEvent;
    FBeforeCloseDok: TE950NotifyEvent;
    FAfterCloseDok: TE950NotifyEvent;
    FBeforeOpenNDok: TE950NotifyEvent;
    FAfterOpenNDok: TE950NotifyEvent;
    FBeforePrnTxtNDok: TE950NotifyEvent;
    FAfterPrnTxtNDok: TE950NotifyEvent;
    FBeforeCloseNDok: TE950NotifyEvent;
    FAfterCloseNDok: TE950NotifyEvent;
    FOnError: TE950ErrorEvent;
    FLed: TLed;
    XLed: TLed;
    RLed: TLed;
    ELed: TLed;
    FLabel: TLabel;
    XLabel: TLabel;
    RLabel: TLabel;
    ELabel: TLabel;
    // default- return values uses system currency&date format
    // XXX0 - return values uses internal representation (if applicable)
    function GetLastResult(AParameter: WideString): WideString;
    procedure FLEDOnClick(Sender: TObject);
    procedure SetLedOn(Index: Integer; Value: Boolean);
    function GetLedOn(Index: Integer): Boolean;
    // Sets the COM port handle
    procedure SetHandle(Value: HFILE);
    // Selects the COM port to use
    procedure SetPort(Value: TPortNumber);
    // Sets the port name
    procedure SetPortName(Value: string);
    // Selects the baud rate
    procedure SetBaudRate(Value: TBaudRate);
    // Selects the baud rate ( actual baud rate value )
    procedure SetBaudRateValue(Value: DWORD);
    // Selects the number of data bits
    procedure SetDataBits(Value: TDataBits);
    // Selects the number of stop bits
    procedure SetStopBits(Value: TStopBits);
    // Selects the kind of parity
    procedure SetParity(Value: TParity);
    // Selects the kind of hardware flow control
    procedure SetHwFlowControl(Value: THwFlowControl);
    // Selects the kind of software flow control
    procedure SetSwFlowControl(Value: TSwFlowControl);
    // Sets the RX buffer size
    procedure SetInBufSize(Value: DWORD);
    // Sets the TX buffer size
    procedure SetOutBufSize(Value: DWORD);
    // Sets the delay between polling checks
    procedure SetPollingDelay(Value: word);
    // Applies current settings to open COM port
    function ApplyCOMSettings: boolean;
    // Polling proc
    procedure TimerWndProc(var msg: TMessage);
    // Sends binary data
    //function SendData( DataPtr: pointer; DataSize: DWORD ): DWORD;
    // Sends binary data. Returns number of bytes sent. Timeout overrides
    // the value specifiend in the OutputTimeout property
    function SendDataEx(DataPtr: pchar; DataSize, Timeout: DWORD): DWORD;
    function SendString(S: string): Integer;
    function SendWideString(S: WideString): Integer;
    // Sends a byte. Returns true if the byte has been sent
    // Reads binary data. Returns number of bytes read
    function ReadData(DataPtr: pchar; MaxDataSize: DWORD): DWORD;
        // Reads a byte. Returns true if the byte has been read
    function ReadByte(var Value: byte): Boolean;
    procedure SetEnumComm;
    //  $3e,  . /.
    function GetTrainingState: WordBool;
    procedure SetTrainingState(State: WordBool);
    // $5a,   .
    procedure SetCashireName(CashireName: WideString);
    // $5b,   .
    function GetCashireName: WideString;
    // $5D,   .
    procedure SetHeaderCheck(Index: Integer; Header: WideString);
    // $5E,    .
    function GetHeaderCheck(Index: Integer): WideString;
    function ReadAnswer: WideString;
    function CheckAnswer(ASend: string; var AAnswer: string; LastCommand: Integer): WordBool;
   { ,    .}
    // $58,    .
    procedure SetDateTime(ADateTime: TDateTime);
    // $59,        .
    function GetDateTime: TDateTime;
   {  . }
    // $6e,  .
    function GetDefineDesc(NumTbl, NumDesc: Integer): WideString;
    // $6f,     .
    procedure SetDefineDesc(NumTbl, NumDesc: Integer; DefineDesc: WideString);
    {  .}
    // $7b,$7c  /  .
    function GetCashBoxOpened(ANo: Integer): WordBool;
    procedure SetCashBoxOpened(ANo: Integer; AOpen: WordBool);
  protected
    procedure Paint; override;
    procedure DoBeforeOpenCheck; virtual;
    procedure DoAfterOpenCheck; virtual;
    procedure DoBeforePrnTxtCheck; virtual;
    procedure DoAfterPrnTxtCheck; virtual;
    procedure DoBeforePrnStrCheck; virtual;
    procedure DoAfterPrnStrCheck; virtual;
    procedure DoBeforePrnSubTotalCheck; virtual;
    procedure DoAfterPrnSubTotalCheck; virtual;
    procedure DoBeforePrnTotalCheck; virtual;
    procedure DoAfterPrnTotalCheck; virtual;
    procedure DoBeforeCloseCheck; virtual;
    procedure DoAfterCloseCheck; virtual;
    procedure DoBeforeOpenNCheck; virtual;
    procedure DoAfterOpenNCheck; virtual;
    procedure DoBeforePrnTxtNCheck; virtual;
    procedure DoAfterPrnTxtNCheck; virtual;
    procedure DoBeforeCloseNCheck; virtual;
    procedure DoAfterCloseNCheck; virtual;
    procedure DoBeforeOpenDok; virtual;
    procedure DoAfterOpenDok; virtual;
    procedure DoBeforePrnTxtDok; virtual;
    procedure DoAfterPrnTxtDok; virtual;
    procedure DoBeforePrnStrDok; virtual;
    procedure DoAfterPrnStrDok; virtual;
    procedure DoBeforePrnSubTotalDok; virtual;
    procedure DoAfterPrnSubTotalDok; virtual;
    procedure DoBeforePrnTotalDok; virtual;
    procedure DoAfterPrnTotalDok; virtual;
    procedure DoBeforeCloseDok; virtual;
    procedure DoAfterCloseDok; virtual;
    procedure DoBeforeOpenNDok; virtual;
    procedure DoAfterOpenNDok; virtual;
    procedure DoBeforePrnTxtNDok; virtual;
    procedure DoAfterPrnTxtNDok; virtual;
    procedure DoBeforeCloseNDok; virtual;
    procedure DoAfterCloseNDok; virtual;
    procedure DoOnError(Error: string); virtual;
  public
    FIOerr: Integer;
    // Constructor
    constructor Create(AOwner: TComponent); override;
    // Destructor
    destructor Destroy; override;
     // Opens the COM port and takes of it. Returns false if something
    // goes wrong.
    function Connect: boolean;
    // Closes the COM port and releases control of it
    procedure Disconnect;
    // Returns true if COM port has been opened
    function Connected: boolean;
    // Returns the current state of CTS, DSR, RING and RLSD (CD) lines.
    // The function fails if the hardware does not support the control-register
    // values (that is, returned set is always empty).
    function GetLineStatus: TLineStatusSet;
    // Returns true if polling has not been paused
    function IsPolling: boolean;
    // Pauses polling
    procedure PausePolling;
    // Re-starts polling (after pause)
    procedure ContinuePolling;
    // Flushes the rx/tx buffers
    function FlushBuffers(inBuf, outBuf: boolean): boolean;
    // Returns number of received bytes in the RX buffer
    function CountRX: Integer;
    // Returns the output buffer free space or 65535 if not connected
    function OutFreeSpace: word;
    procedure ToggleDTR(onOff: boolean);
    // Set RTS line high (onOff=TRUE) or low (onOff=FALSE).
    // You must not use HW handshaking.
    procedure ToggleRTS(onOff: boolean);
        // Sends binary data
//    function SendData( DataPtr: pointer; DataSize: DWORD ): DWORD;
{  }
    // $20, $21    
    //      
    procedure TestFBlock(A2Printer: WordBool; var CountBlock, CountSuccBlock, CountErrorBlock, CountFreeBlock: Integer);
    // $22,  .
    procedure ReportDiag;
    // $2c,      .
    procedure SetBlockWorkMem(SetOf: Integer; InfoBlock: WideString);
    // $2d,      .
    function GetBlockWorkMem(SetOf, LenBlock: Integer): WideString;
    // $2e,    .
    procedure PrintTestPrinter;
    // $2f,   .
    procedure ResetRegistrator;
    // $2a,    .
    procedure GetStatusRegistrator(var LastFDok: Integer;
      var FirstDokDateTime: TDateTime; var LastClose, CountFiskals: Integer;
      var VersionROM: WideString; var Lastdate: TDateTime; var SumMoney: Currency);

    {    .}
     {  .}
    // $30,   .
    procedure GetDmpSertificate(SendToPrint: WordBool;
      var SN: WideString;
      var PosPoint, Devaluations, CurrDef, TypePrn: Integer;
      var PasswordFiskal, PasswordUser: WideString;
      var DateSer: TDateTime);

    // $31,   .
    procedure LoadSertificate(SN: WideString; PosPoint, Devaluations, CurrDef, TypePrn: Integer;
      PasswordFiskal, PasswordUser: WideString; DateSer: TDateTime);
    // $32, .
    procedure Certification;
  {  .}
    // $33,   .
    procedure GetDmpFiskal(SendToPrint: WordBool;
      var RN, UserFR, PasswordFiskal, PasswordUser: WideString; var DateReg: TDateTime);
    // $34,   .
    procedure LoadFiskal(RN, UserFR, PasswordFiskal, PasswordUser: WideString; DateReg: TDateTime);
    // $35, .
    procedure Fiskalization;
    // $37,   
    procedure GetFMemSize(var AllMem, UsesMem: Integer);
    // $39,  .
    procedure CloseSmena(Fix: Boolean;
      var NumClose, CountFCheckComplect, CountFCheckNotComplect, CountFDokComplect,
      CountFDokNotComplect, CountFDokAnulComplect, CountCheckDokService: Integer;
      var SumSale, SumMargin, SumDiscount, SumContainer: Currency;
      var CountCheckDokBack: Integer;
      var SumSaleBack, SumMarginBack, SumDiscountBack, SumContainerBack: Currency;
      var CountCheckDokPayment: Integer;
      var SumPension, SumTranslation, SumComission: Currency;
      var CountCheckDokPaymentBack: Integer;
      var SumPensionBack, SumTranslationBack, SumComissionBack: Currency;
      var CountCheckDokCashe: Integer;
      var SumCashe: Currency;
      var CountCheckDokCasheBack: Integer;
      var SumCasheBack: Currency);
    // $3a,    .
    procedure ReportOfDates(BeginDate, EndDate: TDateTime; ItogOnly: WordBool);
    // $3b,      .
    procedure ReportOfSmena(BeginSmena, EndSmena: Integer; ItogOnly: WordBool);
    //  $3f,   .
    function ChangePassword(PasswordOld, PasswordNew: WideString): WideString;
{   .}
    // $40,   .
    procedure OpenCheck(TypeCheck: TxTypeCheck; IndxCurr: Integer);
    // $41,   .
    procedure PrintFiskalTxt(IndxDesc: Integer; FiskalTxt: WideString);
    // $42,    .
    procedure PrintFiskalStr(IndxDescStr: Integer; DefineDescStr: WideString;
      Count: Double; IndxDescUM: Integer; Price: Currency; IndxDescCurr: Integer; TypeStr: TTypeStrFDok; ProcMargine, ProcDiscount: Double);
    procedure SubTotal(PrintSubTotal, PrintCheck: WordBool; IndxDescSubTotal: Integer;
      DefineSubTotal: WideString; IndDescCurr: Integer;
      ProcMargine, ProcDiscount: Double;
      var TypeCheck: TxTypeCheck; var CountLine: Integer;
      var TotalSale, TotalMargine, TotalDiscount, TotalContainer, InCashe: Currency);
    // $44,   .
    function CheckTotal(InSum: Currency; IndxCurr, IndxOpl: Integer;
      DefOpl: WideString; Tender: WordBool): Currency;
    // $45,   .
    function CloseCheck: Integer;
{   . }
    // $48,   .
    procedure OpenNFCheck(IndxNCheck: Integer; DefineDescNCheck: WideString);
    // $49,   .
    procedure PrintNFText(Str: WideString);
    // $4a,   .
    procedure CloseNFCheck;
    // $4b,   .
    procedure CutNFCheck;
  {  .}
    // $50,   .
    procedure FeedLineCheck(CountLine: Integer);
    // $51,   .
    procedure FeedLineLog(CountLine: Integer);
    // $52,     .
    procedure FeedLineBoth(CountLine: Integer);
    // $53,   .
    procedure FeedLineDok(CountLine: Integer);
    // $54,   .
    procedure EjectDok;
    // $55,  Slip- .
    function ConditionSLIP(CountFreeLine, Pos1, Pos2: Integer): Integer;
    // $57,   .
    procedure FontPrn(Font: Integer);
{   Slip-. }
    // $60,    .
    // $61,    .
    procedure OpenFDok(TDok: TxTypeDok;
      IndxDescCurr, CountFreeLine, PosHeader1, PosDesc, LenDesc, PosDefineDesc,
      LenDefineDesc, PosDescStr, LenDescStr, PosDefineStr, LenDefgineDescStr,
      PosSum, LenSum, PosDescTotal, LenDescTotal, PosDefineTotal, LenDefineToTal,
      PosSumTotal, LenSumTotal, PosHeader2: Integer);
    procedure PrintDefineFDok(indxdesc: Integer; DefineFDok: WideString);
    // $62,    .
    procedure PrintStrFDok(IndxDescStr: Integer; DefineDescStr: WideString;
      Count: Double; IndxDescUM: Integer; Price: Currency; IndxDescCurr: Integer; TypeStr: TTypeStrFDok;
      ProcMargine, ProcDiscount: Double);
    // $63,    
    procedure SubTotalFDok(PrintDok: WordBool; indxdescSubTotal: Integer; DefineSubTotal: WideString;
      IndDescCurr: Integer; ProcMargine, ProcDiscount: Double; var TDok: TxTypeDok; var CountLine: Integer;
      var TotalSale, TotalMargine, TotalDiscount, TotalContainer, InCashe: Currency);
    // $64,    .
    function TotalFDok(InSum: Currency; IndxCurr, IndxOpl: Integer; DefOpl: WideString; Tender: Boolean): Currency;
    // $65,    .
    function CloseFDok: Integer;
{   .}
    // $68,    .
    procedure OpenNDok(IndxDesc: Integer;
      DefIndxDesc: WideString;
      EmptyStr, PosHeader1, PosHeader2: Integer);
    // $69,     .
    procedure PrintNDokStr(Str: WideString);
    // $6a,    .
    procedure CloseNDok;
{  .}
 // $70,    COM 1.
 // $71,   COM 1.
 // $72,     COM 1.
 // $73,     COM 1.
 // $74,    COM 2.
 // $75,   COM 2.
 // $76,     COM 2.
 // $77,     COM 2.
    // Make the Handle of the COM port public (for TAPI...) [read/write]
    property Handle: HFILE read FHandle write SetHandle;
    // $3e
    property TrainingState: WordBool read GetTrainingState write SetTrainingState;
    // $5a/$5b
    property CashireName: WideString read GetCashireName write SetCashireName;
    // $5d/$5e
    property HeaderCheck[i: Integer]: WideString read GetHeaderCheck write SetHeaderCheck;
    // $58/$59
    property DateTimeFR: TDateTime read GetDateTime write SetDateTime;
    //  
    property Password: WideString read FPassword write FPassword;
    //  
    property InspPassword: WideString read FInspPassword write FInspPassword;
    // property StateRegistrator: TStateRegistrator read FStateRegistrator;
    // $6f,     .
    property DefineDesc[NumTbl, NumDesc: Integer]: WideString read GetDefineDesc write SetDefineDesc;
    property CashBoxOpened[ACashBoxNo: Integer]: WordBool read GetCashBoxOpened write SetCashBoxOpened;
    property ErrorStr: WideString read FErrorStr;
    //     ( var )
    property LastResult[parameter: WideString]: WideString read GetLastResult;
  published
   { property FLedOn: Boolean index 0 read GetLedOn write SetLedOn;
    property XLedOn: Boolean index 1 read GetLedOn write SetLedOn;
    property RLedOn: Boolean index 2 read GetLedOn write SetLedOn;}
    property ELedOn: Boolean index 3 read GetLedOn write SetLedOn;
    // # of the COM Port to use ( or pnCustom for port by name )
    property Port: TPortNumber read FPort write SetPort default pnCOM2;
    // Name of COM port
    property PortName: string read FPortName write SetPortName;
    // Speed ( Baud Rate )
    property BaudRate: TBaudRate read FBaudRate write SetBaudRate default br9600;
    // Speed ( Actual Baud Rate value )
    property BaudRateValue: DWORD read FBaudRateValue write SetBaudRateValue default 9600;
    // Data bits to use (5..8, for the 8250 the use of 5 data bits with 2 stop
    // bits is an invalid combination, as is 6, 7, or 8 data bits with 1.5 stop
    // bits)
    property DataBits: TDataBits read FDataBits write SetDataBits default db8BITS;
    // Stop bits to use (1, 1.5, 2)
    property StopBits: TStopBits read FStopBits write SetStopBits default sb1BITS;
    // Kind of Parity to use (none,odd,even,mark,space)
    property Parity: TParity read FParity write SetParity default ptNONE;
    // Kind of Hardware Flow Control to use:
    //   hfNONE          none
    //   hfNONERTSON     no flow control but keep RTS line on
    //   hfRTSCTS        Request-To-Send/Clear-To-Send
    property HwFlow: THwFlowControl read FHwFlow write SetHwFlowControl default hfNONERTSON;
    // Kind of Software Flow Control to use:
    //   sfNONE          none
    //   sfXONXOFF       XON/XOFF
    property SwFlow: TSwFlowControl read FSwFlow write SetSwFlowControl default sfNONE;
    // Input Buffer size ( suggested - driver might ignore this setting ! )
    property InBufSize: DWORD read FInBufSize write SetInBufSize default 2048;
    // Output Buffer size ( suggested - driver usually ignores this setting ! )
    property OutBufSize: DWORD read FOutBufSize write SetOutBufSize default 2048;
    // ms of delay between COM port pollings
    property PollingDelay: word read FPollingDelay write SetPollingDelay default 50;
    // Set to TRUE to enable DTR line on connect and to leave it on until disconnect.
    // Set to FALSE to disable DTR line on connect.
    property EnableDTROnOpen: boolean read FEnableDTROnOpen write FEnableDTROnOpen default true;
    // Output timeout (milliseconds)
    property OutputTimeout: word read FOutputTimeOut write FOutputTimeout default 500;
    // Input timeout (milliseconds)
    property InputTimeout: DWORD read FInputTimeOut write FInputTimeout default 200;
    // Set to TRUE to prevent hangs when no device connected or device is OFF
    property CheckLineStatus: Boolean read FCkLineStatus write FCkLineStatus default false;
    // Event to raise when there is data available (input buffer has data)
    // (called only if PacketSize <= 0)
    property OnReceiveData: TReceiveDataEvent read FOnReceiveData write FOnReceiveData;
    // Event to raise when there is data packet available (called only if PacketSize > 0)
    property OnReceivePacket: TReceivePacketEvent read FOnReceivePacket write FOnReceivePacket;
    property BeforeOpenCheck: TE950NotifyEvent read FBeforeOpenCheck write FBeforeOpenCheck;
    property AfterOpenCheck: TE950NotifyEvent read FAfterOpenCheck write FAfterOpenCheck;
    property BeforePrnTxtCheck: TE950NotifyEvent read FBeforePrnTxtCheck write FBeforePrnTxtCheck;
    property AfterPrnTxtCheck: TE950NotifyEvent read FAfterPrnTxtCheck write FAfterPrnTxtCheck;
    property BeforePrnStrCheck: TE950NotifyEvent read FBeforePrnStrCheck write FBeforePrnStrCheck;
    property AfterPrnStrCheck: TE950NotifyEvent read FAfterPrnStrCheck write FAfterPrnStrCheck;
    property BeforePrnSubTotalCheck: TE950NotifyEvent read FBeforePrnSubTotalCheck write FBeforePrnSubTotalCheck;
    property AfterPrnSubTotalCheck: TE950NotifyEvent read FAfterPrnSubTotalCheck write FAfterPrnSubTotalCheck;
    property BeforePrnTotalCheck: TE950NotifyEvent read FBeforePrnTotalCheck write FBeforePrnTotalCheck;
    property AfterPrnTotalCheck: TE950NotifyEvent read FAfterPrnTotalCheck write FAfterPrnTotalCheck;
    property BeforeCloseCheck: TE950NotifyEvent read FBeforeCloseCheck write FBeforeCloseCheck;
    property AfterCloseCheck: TE950NotifyEvent read FAfterCloseCheck write FAfterCloseCheck;
    property BeforeOpenNCheck: TE950NotifyEvent read FBeforeOpenNCheck write FBeforeOpenNCheck;
    property AfterOpenNCheck: TE950NotifyEvent read FAfterOpenNCheck write FAfterOpenNCheck;
    property BeforePrnTxtNCheck: TE950NotifyEvent read FBeforePrnTxtNCheck write FBeforePrnTxtNCheck;
    property AfterPrnTxtNCheck: TE950NotifyEvent read FAfterPrnTxtNCheck write FAfterPrnTxtNCheck;
    property BeforeCloseNCheck: TE950NotifyEvent read FBeforeCloseNCheck write FBeforeCloseNCheck;
    property AfterCloseNCheck: TE950NotifyEvent read FAfterCloseNCheck write FAfterCloseNCheck;
    property BeforeOpenDok: TE950NotifyEvent read FBeforeOpenDok write FBeforeOpenDok;
    property AfterOpenDok: TE950NotifyEvent read FAfterOpenDok write FAfterOpenDok;
    property BeforePrnTxtDok: TE950NotifyEvent read FBeforePrnTxtDok write FBeforePrnTxtDok;
    property AfterPrnTxtDok: TE950NotifyEvent read FAfterPrnTxtDok write FAfterPrnTxtDok;
    property BeforePrnStrDok: TE950NotifyEvent read FBeforePrnStrDok write FBeforePrnStrDok;
    property AfterPrnStrDok: TE950NotifyEvent read FAfterPrnStrDok write FAfterPrnStrDok;
    property BeforePrnSubTotalDok: TE950NotifyEvent read FBeforePrnSubTotalDok write FBeforePrnSubTotalDok;
    property AfterPrnSubTotalDok: TE950NotifyEvent read FAfterPrnSubTotalDok write FAfterPrnSubTotalDok;
    property BeforePrnTotalDok: TE950NotifyEvent read FBeforePrnTotalDok write FBeforePrnTotalDok;
    property AfterPrnTotalDok: TE950NotifyEvent read FAfterPrnTotalDok write FAfterPrnTotalDok;
    property BeforeCloseDok: TE950NotifyEvent read FBeforeCloseDok write FBeforeCloseDok;
    property AfterCloseDok: TE950NotifyEvent read FAfterCloseDok write FAfterCloseDok;
    property BeforeOpenNDok: TE950NotifyEvent read FBeforeOpenNDok write FBeforeOpenNDok;
    property AfterOpenNDok: TE950NotifyEvent read FAfterOpenNDok write FAfterOpenNDok;
    property BeforePrnTxtNDok: TE950NotifyEvent read FBeforePrnTxtNDok write FBeforePrnTxtNDok;
    property AfterPrnTxtNDok: TE950NotifyEvent read FAfterPrnTxtNDok write FAfterPrnTxtNDok;
    property BeforeCloseNDok: TE950NotifyEvent read FBeforeCloseNDok write FBeforeCloseNDok;
    property AfterCloseNDok: TE950NotifyEvent read FAfterCloseNDok write FAfterCloseNDok;
    property OnError: TE950ErrorEvent read FOnError write FOnError;
  end;

function BaudRateOf(bRate: TBaudRate): DWORD;
function DelayForRX(bRate: TBaudRate; DataSize: DWORD): DWORD;

implementation

const
  Win32BaudRates: array[br110..br256000] of DWORD =
    (CBR_110, CBR_300, CBR_600, CBR_1200, CBR_2400, CBR_4800, CBR_9600,
    CBR_14400, CBR_19200, CBR_38400, CBR_56000, CBR_57600, CBR_115200,
    CBR_128000, CBR_256000);

const
  dcb_Binary = $00000001;
  dcb_ParityCheck = $00000002;
  dcb_OutxCtsFlow = $00000004;
  dcb_OutxDsrFlow = $00000008;
  dcb_DtrControlMask = $00000030;
  dcb_DtrControlDisable = $00000000;
  dcb_DtrControlEnable = $00000010;
  dcb_DtrControlHandshake = $00000020;
  dcb_DsrSensivity = $00000040;
  dcb_TXContinueOnXoff = $00000080;
  dcb_OutX = $00000100;
  dcb_InX = $00000200;
  dcb_ErrorChar = $00000400;
  dcb_NullStrip = $00000800;
  dcb_RtsControlMask = $00003000;
  dcb_RtsControlDisable = $00000000;
  dcb_RtsControlEnable = $00001000;
  dcb_RtsControlHandshake = $00002000;
  dcb_RtsControlToggle = $00003000;
  dcb_AbortOnError = $00004000;
  dcb_Reserveds = $FFFF8000;

function GetWinPlatform: string;
var ov: TOSVERSIONINFO;
begin
  ov.dwOSVersionInfoSize := sizeof(ov);
  if GetVersionEx(ov) then
  begin
    case ov.dwPlatformId of
      VER_PLATFORM_WIN32s: // Win32s on Windows 3.1
        Result := 'W32S';
      VER_PLATFORM_WIN32_WINDOWS: // Win32 on Windows 95/98
        Result := 'W95';
      VER_PLATFORM_WIN32_NT: //	Windows NT
        Result := 'WNT';
    end;
  end
  else
    Result := '??';
end;

function GetWinVersion: DWORD;
var ov: TOSVERSIONINFO;
begin
  ov.dwOSVersionInfoSize := sizeof(ov);
  if GetVersionEx(ov) then
    Result := MAKELONG(ov.dwMinorVersion, ov.dwMajorVersion)
  else
    Result := $00000000;
end;

function BaudRateOf(bRate: TBaudRate): DWORD;
begin
  if bRate = brCustom then
    Result := 0
  else
    Result := Win32BaudRates[bRate];
end;

function DelayForRX(bRate: TBaudRate; DataSize: DWORD): DWORD;
begin
  Result := round(DataSize / (BaudRateOf(bRate) / 10) * 1000);
end;

procedure AddCheckSum(var AQueryStr: string);
var
  i, AQuerySum: Integer;
  S: string;
begin
  AQuerySum := 0;
  for i := 1 to Length(AQueryStr)
    do AQuerySum := AQuerySum + Ord(AQueryStr[i]);
  S := IntToHex(AQuerySum, 4);
  AQueryStr := AQueryStr + S;
end;

function WWinToDos(WinStr: WideString): WideString;
var
  S, SS: string;
begin
  S := WinStr;
  SetLength(SS, Length(S) + 1);
  CharToOemBuff(PChar(S), PChar(SS), Length(S));
  Result := SS;
end;

function WDosToWin(DosStr: WideString): WideString;
var
  S, SS: string;
begin
  S := DosStr;
  SetLength(SS, Length(S) + 1);
  OemToCharBuff(PChar(S), PChar(SS), Length(S));
  Result := SS;
end;

function WinToDos(WinStr: string): string;
begin
  SetLength(Result, Length(WinStr));
  CharToOemBuff(PChar(WinStr), PChar(Result), Length(WinStr));
end;

function DosToWin(DosStr: string): string;
begin
  SetLength(Result, Length(DosStr));
  OemToCharBuff(PChar(DosStr), PChar(Result), Length(DosStr));
end;

constructor TE950Driver.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLst := TStringList.Create;
  SetBounds(Left, Top, 213, 51);
  Self.Caption := '';

  InitializeCriticalSection(FCritSect);
  // Initialize to default values -----------------------
  // Not connected
  FHandle := INVALID_HANDLE_VALUE;
  // COM 2
  FPort := pnCOM2;
  FPortName := '\\.\COM2';
  // 9600 bauds
  FBaudRate := br9600;
  FBaudRateValue := BaudRateOf(br9600);
  // 8 data bits
  FDataBits := db8BITS;
  // 1 stop bit
  FStopBits := sb1BITS;
  // no parity
  FParity := ptNONE;
  // No hardware flow control but RTS on
  FHwFlow := hfNONERTSON;
  // No software flow control
  FSwFlow := sfNONE;
  // Input buffer of 2048 bytes
  FInBufSize := 2048;
  // Output buffer of 2048 bytes
  FOutBufSize := 2048;
  // Poll COM port every 50ms
  FPollingDelay := 50;
  // Output timeout of 500ms
  FOutputTimeout := 500;
  // Timeout for ReadData(), 500ms
  FInputTimeout := 500;
  // DTR high on connect
  FEnableDTROnOpen := true;
  // Time not valid ( used by the packing routines )
  //FFirstByteOfPacketTime     := DWORD(-1);
  // Don't check of off-line devices
  FCkLineStatus := false;
  // Init number of RX polling timer pauses - not paused
  FRXPollingPauses := 0;
  // Temporary buffer for received data
  FTempInBuffer := AllocMem(FInBufSize);
  // Allocate a window handle to catch timer's notification messages
  if not (csDesigning in ComponentState) then
    FNotifyWnd := AllocateHWnd(TimerWndProc);
  FErrorStr := '';
  FLed := TLed.Create(Self);
  FLed.Parent := Self;
  with FLed do
  begin
    Left := 24;
    Top := 24;
    Width := 10;
    Height := 10;
    LEDOnCol := clLime;
    LEDOffCol := clGreen;
    OnClick := FLEDOnClick;
  end;
  XLed := TLed.Create(Self);
  XLed.Parent := Self;
  with XLed do
  begin
    Left := 72;
    Top := 24;
    Width := 10;
    Height := 10;
    LEDOnCol := clLime;
    LEDOffCol := clGreen;
  end;

  RLed := TLed.Create(Self);
  RLed.Parent := Self;
  with RLed do
  begin
    Left := 120;
    Top := 24;
    Width := 10;
    Height := 10;
    LEDOnCol := clLime;
    LEDOffCol := clGreen;
  end;

  ELed := TLed.Create(Self);
  ELed.Parent := Self;
  with ELed do
  begin
    Left := 168;
    Top := 24;
    Width := 10;
    Height := 10;
    LEDOnCol := clRed;
    LEDOffCol := clMaroon;
  end;

  FLabel := TLabel.Create(Self);
  FLabel.Parent := Self;
  with FLabel do
  begin
    ParentFont := False;
    Font.Charset := DEFAULT_CHARSET;
    Font.Color := clWindowText;
    Font.Height := -11;
    Font.Name := 'MS Sans Serif';
    Font.Style := [];
    Left := 8;
    Top := 8;
    Width := 41;
    Height := 13;
    Caption := '';
  end;

  XLabel := TLabel.Create(Self);
  XLabel.Parent := Self;
  with XLabel do
  begin
    ParentFont := False;
    Font.Charset := DEFAULT_CHARSET;
    Font.Color := clWindowText;
    Font.Height := -11;
    Font.Name := 'MS Sans Serif';
    Font.Style := [];
    Left := 56;
    Top := 8;
    Width := 49;
    Height := 13;
    Caption := '';
  end;

  RLabel := TLabel.Create(Self);
  RLabel.Parent := Self;
  with RLabel do
  begin
    ParentFont := False;
    Font.Charset := DEFAULT_CHARSET;
    Font.Color := clWindowText;
    Font.Height := -11;
    Font.Name := 'MS Sans Serif';
    Font.Style := [];
    Left := 112;
    Top := 8;
    Width := 34;
    Height := 13;
    Caption := '';
  end;

  ELabel := TLabel.Create(Self);
  ELabel.Parent := Self;
  with ELabel do
  begin
    ParentFont := False;
    Font.Charset := DEFAULT_CHARSET;
    Font.Color := clWindowText;
    Font.Height := -11;
    Font.Name := 'MS Sans Serif';
    Font.Style := [];
    Left := 152;
    Top := 8;
    Width := 40;
    Height := 13;
    Caption := '';
  end;

end;

destructor TE950Driver.Destroy;
begin
  // Be sure to release the COM port
  FLed.Free;
  XLed.Free;
  RLed.Free;
  ELed.Free;
  FLed := nil;
  XLed := nil;
  RLed := nil;
  ELed := nil;

  FLabel.Free;
  XLabel.Free;
  RLabel.Free;
  ELabel.Free;
  Disconnect;
  // Free the temporary buffer
  FreeMem(FTempInBuffer, FInBufSize);
  // Destroy the timer's window
  if not (csDesigning in ComponentState) then
    DeallocateHWnd(FNotifyWnd);
  DeleteCriticalSection(FCritSect);
  FLst.Free;
  // Call inherited destructor
  inherited Destroy;
end;

procedure TE950Driver.Paint;
begin
  inherited Paint;
  SetBounds(Left, Top, 213, 51);
end;

// The COM port handle made public and writeable.
// This lets you connect to external opened com port.
// Setting ComPortHandle to INVALID_PORT_HANDLE acts as Disconnect.

procedure TE950Driver.SetHandle(Value: HFILE);
begin
  FErrorStr := '';
  // If same COM port then do nothing
  if FHandle = Value then
    exit;
  // If value is RELEASE_NOCLOSE_PORT then stop controlling the COM port
  // without closing in
  if Value = RELEASE_NOCLOSE_PORT then begin
    // Stop the timer
    if Connected
      then KillTimer(FNotifyWnd, 1);
    // No more connected
    FHandle := INVALID_HANDLE_VALUE;
  end else begin
    Disconnect;
    // If Value is INVALID_HANDLE_VALUE then exit now
    if Value = INVALID_HANDLE_VALUE then
      exit;
    // Set COM port handle
    FHandle := Value;
    // Start the timer ( used for polling )
    SetTimer(FNotifyWnd, 1, FPollingDelay, nil);
  end;
end;

// Selects the COM port to use

procedure TE950Driver.SetPort(Value: TPortNumber);
begin
  FErrorStr := '';
  // Be sure we are not using any COM port
  if Connected
    then exit;
  // Change COM port
  FPort := Value;
  // Update the port name
  if FPort <> pnCustom
    then FPortName := Format('\\.\COM%d', [ord(FPort)]);
end;

// Sets the port name

procedure TE950Driver.SetPortName(Value: string);
begin
  FErrorStr := '';
  // Be sure we are not using any COM port
  if Connected
    then Exit;
  // Change COM port
  FPort := pnCustom;
  // Update the port name
  FPortName := Value;
end;

// Selects the baud rate

procedure TE950Driver.SetBaudRate(Value: TBaudRate);
begin
  FErrorStr := '';
  // Set new COM speed
  FBaudRate := Value;
  if FBaudRate <> brCustom then
    FBaudRateValue := BaudRateOf(FBaudRate);
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Selects the baud rate ( actual baud rate value )

procedure TE950Driver.SetBaudRateValue(Value: DWORD);
begin
  FErrorStr := '';
  // Set new COM speed
  FBaudRate := brCustom;
  FBaudRateValue := Value;
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Selects the number of data bits

procedure TE950Driver.SetDataBits(Value: TDataBits);
begin
  FErrorStr := '';
  // Set new data bits
  FDataBits := Value;
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Selects the number of stop bits

procedure TE950Driver.SetStopBits(Value: TStopBits);
begin
  FErrorStr := '';
  // Set new stop bits
  FStopBits := Value;
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Selects the kind of parity

procedure TE950Driver.SetParity(Value: TParity);
begin
  FErrorStr := '';
  // Set new parity
  FParity := Value;
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Selects the kind of hardware flow control

procedure TE950Driver.SetHwFlowControl(Value: THwFlowControl);
begin
  FErrorStr := '';
  // Set new hardware flow control
  FHwFlow := Value;
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Selects the kind of software flow control

procedure TE950Driver.SetSwFlowControl(Value: TSwFlowControl);
begin
  FErrorStr := '';
  // Set new software flow control
  FSwFlow := Value;
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Sets the RX buffer size

procedure TE950Driver.SetInBufSize(Value: DWORD);
begin
  FErrorStr := '';
  // Do nothing if connected
  if Connected then
    exit;
  // Free the temporary input buffer
  FreeMem(FTempInBuffer, FInBufSize);
  // Set new input buffer size
  if Value > 8192 then
    Value := 8192
  else if Value < 128 then
    Value := 128;
  FInBufSize := Value;
  // Allocate the temporary input buffer
  FTempInBuffer := AllocMem(FInBufSize);
  // Adjust the RX packet size
end;

// Sets the TX buffer size

procedure TE950Driver.SetOutBufSize(Value: DWORD);
begin
  FErrorStr := '';
  // Do nothing if connected
  if Connected then
    exit;
  // Set new output buffer size
  if Value > 8192 then
    Value := 8192
  else if Value < 128 then
    Value := 128;
  FOutBufSize := Value;
end;

// Sets the size of incoming packets

// Sets the delay between polling checks

procedure TE950Driver.SetPollingDelay(Value: word);
begin
  FErrorStr := '';
  // Make it greater than 4 ms
  if Value < 5 then
    Value := 5;
  // If new delay is not equal to previous value...
  if Value <> FPollingDelay then
  begin
    // Stop the timer
    if Connected then
      KillTimer(FNotifyWnd, 1);
    // Store new delay value
    FPollingDelay := Value;
    // Restart the timer
    if Connected then
      SetTimer(FNotifyWnd, 1, FPollingDelay, nil);
  end;
end;

// Apply COM settings

function TE950Driver.ApplyCOMSettings: boolean;
var dcb: TDCB;
begin
  FErrorStr := '';
  // Do nothing if not connected
  Result := false;
  if not Connected then
    exit;

  // ** Setup DCB (Device Control Block) fields ******************************

  // Clear all
  fillchar(dcb, sizeof(dcb), 0);
  // DCB structure size
  dcb.DCBLength := sizeof(dcb);
  // Baud rate
  dcb.BaudRate := FBaudRateValue;
  // Set fBinary: Win32 does not support non binary mode transfers
  // (also disable EOF check)
  dcb.Flags := dcb_Binary;
  // Enables the DTR line when the device is opened and leaves it on
  if EnableDTROnOpen then
    dcb.Flags := dcb.Flags or dcb_DtrControlEnable;
  // Kind of hw flow control to use
  case FHwFlow of
    // No hw flow control
    hfNONE: ;
    // No hw flow control but set RTS high and leave it high
    hfNONERTSON:
      dcb.Flags := dcb.Flags or dcb_RtsControlEnable;
    // RTS/CTS (request-to-send/clear-to-send) flow control
    hfRTSCTS:
      dcb.Flags := dcb.Flags or dcb_OutxCtsFlow or dcb_RtsControlHandshake;
  end;
  // Kind of sw flow control to use
  case FSwFlow of
    // No sw flow control
    sfNONE: ;
    // XON/XOFF sw flow control
    sfXONXOFF:
      dcb.Flags := dcb.Flags or dcb_OutX or dcb_InX;
  end;
  // Set XONLim: specifies the minimum number of bytes allowed in the input
  // buffer before the XON character is sent (or CTS is set).
  if (GetWinPlatform = 'WNT') and (GetWinVersion >= $00040000) then
  begin
    // WinNT 4.0 + Service Pack 3 needs XONLim to be less than or
    // equal to 4096 bytes. Win95/98 doesn't have such limit.
    if FInBufSize div 4 > 4096 then
      dcb.XONLim := 4096
    else
      dcb.XONLim := FInBufSize div 4;
  end
  else
    dcb.XONLim := FInBufSize div 4;
  // Specifies the maximum number of bytes allowed in the input buffer before
  // the XOFF character is sent (or CTS is set low). The maximum number of bytes
  // allowed is calculated by subtracting this value from the size, in bytes, of
  // the input buffer.
  dcb.XOFFLim := dcb.XONLim;
  // How many data bits to use
  dcb.ByteSize := 5 + ord(FDataBits);
  // Kind of parity to use
  dcb.Parity := ord(FParity);
  // How many stop bits to use
  dcb.StopBits := ord(FStopbits);
  // XON ASCII char - DC1, Ctrl-Q, ASCII 17
  dcb.XONChar := #17;
  // XOFF ASCII char - DC3, Ctrl-S, ASCII 19
  dcb.XOFFChar := #19;

  // Apply new settings
  Result := SetCommState(FHandle, dcb);
  if not Result then
    exit;
  // Flush buffers
  Result := FlushBuffers(true, true);
  if not Result then
    exit;
  // Setup buffers size
  Result := SetupComm(FHandle, FInBufSize, FOutBufSize);
end;

function TE950Driver.Connect: boolean;
var tms: TCOMMTIMEOUTS;
begin
  FErrorStr := '';
  // Do nothing if already connected
  Result := Connected;
  if Result then
    exit;
  // Open the COM port
  FHandle := CreateFile(pchar(FPortName),
    GENERIC_READ or GENERIC_WRITE,
    0, // Not shared
    nil, // No security attributes
    OPEN_EXISTING,
    FILE_ATTRIBUTE_NORMAL { or FILE_FLAG_OVERLAPPED},
    0 // No template
    );
  Result := Connected;
  if not Result then
    DoOnError(Format('   : %s', [SysErrorMessage(GetLastError)]));

  // Apply settings
  Result := ApplyCOMSettings;
  if not Result then begin
    Disconnect;
    //exit;
    DoOnError(Format('   : %s', [SysErrorMessage(GetLastError)]));
  end;
  // Set ReadIntervalTimeout: Specifies the maximum time, in milliseconds,
  // allowed to elapse between the arrival of two characters on the
  // communications line.
  // We disable timeouts because we are polling the com port!
  tms.ReadIntervalTimeout := 1;
  // Set ReadTotalTimeoutMultiplier: Specifies the multiplier, in milliseconds,
  // used to calculate the total time-out period for read operations.
  tms.ReadTotalTimeoutMultiplier := 0;
  // Set ReadTotalTimeoutConstant: Specifies the constant, in milliseconds,
  // used to calculate the total time-out period for read operations.
  tms.ReadTotalTimeoutConstant := 1;
  // Set WriteTotalTimeoutMultiplier: Specifies the multiplier, in milliseconds,
  // used to calculate the total time-out period for write operations.
  tms.WriteTotalTimeoutMultiplier := 0;
  // Set WriteTotalTimeoutConstant: Specifies the constant, in milliseconds,
  // used to calculate the total time-out period for write operations.
  tms.WriteTotalTimeoutConstant := 1;
  // Apply timeouts
  SetCommTimeOuts(FHandle, tms);
  // Start the timer (used for polling)
  SetTimer(FNotifyWnd, 1, FPollingDelay, nil);
  FLed.LEDOn := True;
//  FLed.Update;
end;

procedure TE950Driver.Disconnect;
begin
  if FLed <> nil
    then
  begin
    FLed.LEDOn := False;
//    FLed.Update;
  end;
  FErrorStr := '';
  if Connected then
  begin
    // Stop the timer (used for polling)
    KillTimer(FNotifyWnd, 1);
    // Release the COM port
    CloseHandle(FHandle);
    // No more connected
    FHandle := INVALID_HANDLE_VALUE;
  end;
end;

// Returns true if connected

function TE950Driver.Connected: boolean;
begin
  FErrorStr := '';
  Result := FHandle <> INVALID_HANDLE_VALUE;
end;

// Returns CTS, DSR, RING and RLSD (CD) signals status

function TE950Driver.GetLineStatus: TLineStatusSet;
var dwS: DWORD;
begin
  FErrorStr := '';
  Result := [];
  if not Connected then
    exit;
  // Retrieves modem control-register values.
  // The function fails if the hardware does not support the control-register
  // values.
  if not GetCommModemStatus(FHandle, dwS) then
    exit;
  if dwS and MS_CTS_ON <> 0 then Result := Result + [lsCTS];
  if dwS and MS_DSR_ON <> 0 then Result := Result + [lsDSR];
  if dwS and MS_RING_ON <> 0 then Result := Result + [lsRING];
  if dwS and MS_RLSD_ON <> 0 then Result := Result + [lsCD];
end;

// Returns true if polling has not been paused

function TE950Driver.IsPolling: boolean;
begin
  FErrorStr := '';
  Result := FRXPollingPauses <= 0;
end;

// Pauses polling

procedure TE950Driver.PausePolling;
begin
  FErrorStr := '';
  // Inc. RX polling pauses counter
  inc(FRXPollingPauses);
end;

// Re-starts polling (after pause)

procedure TE950Driver.ContinuePolling;
begin
  FErrorStr := '';
  // Dec. RX polling pauses counter
  dec(FRXPollingPauses);
end;

// Flush rx/tx buffers

function TE950Driver.FlushBuffers(inBuf, outBuf: boolean): boolean;
var dwAction: DWORD;
begin
  FErrorStr := '';
  // Do nothing if not connected
  Result := false;
  if not Connected then
    exit;
  // Flush the RX data buffer
  dwAction := 0;
  if outBuf then
    dwAction := dwAction or PURGE_TXABORT or PURGE_TXCLEAR;
  // Flush the TX data buffer
  if inBuf then
    dwAction := dwAction or PURGE_RXABORT or PURGE_RXCLEAR;
  Result := PurgeComm(FHandle, dwAction);
  // Used by the RX packet mechanism
//  if Result then
//    FFirstByteOfPacketTime := DWORD(-1);
end;

// Returns number of received bytes in the RX buffer

function TE950Driver.CountRX: integer;
var stat: TCOMSTAT;
  errs: DWORD;
begin
  FErrorStr := '';
  // Do nothing if port has not been opened
  Result := 65535;
  if not Connected then
    exit;
  // Get count
  ClearCommError(FHandle, errs, @stat);
  Result := stat.cbInQue;
end;

// Returns the output buffer free space or 65535 if not connected

function TE950Driver.OutFreeSpace: word;
var stat: TCOMSTAT;
  errs: DWORD;
begin
  FErrorStr := '';
  if not Connected then
    Result := 65535
  else
  begin
    ClearCommError(FHandle, errs, @stat);
    Result := FOutBufSize - stat.cbOutQue;
  end;
end;

// Sends binary data. Returns number of bytes sent. Timeout overrides
// the value specifiend in the OutputTimeout property

function TE950Driver.SendDataEx(DataPtr: pchar; DataSize, Timeout: DWORD): DWORD;
var
  nToSend, nSent, t1: DWORD;
begin
  FErrorStr := '';
  // Do nothing if port has not been opened
  Result := 0;
  if not Connected
    then exit;
  // Current time
  t1 := GetTickCount;
  // Loop until all data sent or timeout occurred
  XLed.LEDOn := True;
//  XLed.Update;
  while DataSize > 0 do begin
    // Get TX buffer free space
    nToSend := OutFreeSpace;
    // If output buffer has some free space...
    if nToSend > 0 then begin
      // Check signals
      if FCkLineStatus and (GetLineStatus = [])
        then exit;
      // Don't send more bytes than we actually have to send
      if nToSend > DataSize then
        nToSend := DataSize;
      // Send
      with FOverLapped do begin
        Offset := 0;
        OffsetHigh := 0;
        hEvent := THandle(nil);
      end;
      WriteFile(FHandle, DataPtr^, nToSend, nSent, nil);
      FIOerr := GetLastError;
      nSent := abs(nSent);
      if nSent > 0 then begin
        // Update number of bytes sent
        Result := Result + nSent;
        // Decrease the count of bytes to send
        DataSize := DataSize - nSent;
        // Inc. data pointer
        DataPtr := DataPtr + nSent;
        // Get current time
        t1 := GetTickCount;
        // Continue. This skips the time check below (don't stop
        // trasmitting if the Timeout is set too low)
        continue;
      end;
    end;
    // Buffer is full. If we are waiting too long then exit
    if DWORD(GetTickCount - t1) > Timeout
      then Break;
  end;
  XLed.LEDOn := False;
//  XLed.Update;
end;

function TE950Driver.SendString(S: string): Integer;
begin
  Result := SendDataEx(PChar(S), Length(S), FOutputTimeout);
end;

function TE950Driver.SendWideString(S: WideString): Integer;
var
  SS: string;
begin
  SS := S;
  Result := SendDataEx(PChar(SS), Length(SS), FOutputTimeout);
end;

// Send data (breaks the data in small packets if it doesn't fit in the output
// buffer)
//function TE950Driver.SendData( DataPtr: pointer; DataSize: DWORD ): DWORD;
//begin
//  Result := SendString(DataPtr);
//end;

// Sends a byte. Returns true if the byte has been sent

// Reads binary data. Returns number of bytes read

function TE950Driver.ReadData(DataPtr: pchar; MaxDataSize: DWORD): DWORD;
var nToRead, nRead, t1: DWORD;
begin
  FErrorStr := '';
  // Do nothing if port has not been opened
  Result := 0;
  if not Connected then
    exit;
  // Pause polling
  PausePolling;
  // Current time
  t1 := GetTickCount;
  // Loop until all requested data read or timeout occurred
  while MaxDataSize > 0 do
  begin
    // Get data bytes count in RX buffer
    nToRead := CountRX;
    // If input buffer has some data...
    if nToRead > 0 then
    begin
      // Don't read more bytes than we actually have to read
      if nToRead > MaxDataSize then
        nToRead := MaxDataSize;
      // Read
      with FOverLapped do begin
        Offset := 0;
        OffsetHigh := 0;
        hEvent := THandle(nil);
      end;
      ReadFile(FHandle, DataPtr^, nToRead, nRead, nil);
      // Update number of bytes read
      Result := Result + nRead;
      // Decrease the count of bytes to read
      MaxDataSize := MaxDataSize - nRead;
      // Inc. data pointer
      DataPtr := DataPtr + nRead;
      // Get current time
      t1 := GetTickCount;
      // Continue. This skips the time check below (don't stop
      // reading if the FInputTimeout is set too low)
      continue;
    end;
    // Buffer is empty. If we are waiting too long then exit
    if (GetTickCount - t1) > FInputTimeout then
      break;
  end;
  // Continue polling
  ContinuePolling;
end;

// Reads a byte. Returns true if the byte has been read

function TE950Driver.ReadByte(var Value: byte): boolean;
begin
  FErrorStr := '';
  Result := ReadData(@Value, 1) = 1;
end;

// Set DTR line high (onOff=TRUE) or low (onOff=FALSE).
// You must not use HW handshaking.

procedure TE950Driver.ToggleDTR(onOff: boolean);
const funcs: array[boolean] of integer = (CLRDTR, SETDTR);
begin
  FErrorStr := '';
  if Connected then
    EscapeCommFunction(FHandle, funcs[onOff]);
end;

// Set RTS line high (onOff=TRUE) or low (onOff=FALSE).
// You must not use HW handshaking.

procedure TE950Driver.ToggleRTS(onOff: boolean);
const funcs: array[boolean] of integer = (CLRRTS, SETRTS);
begin
  FErrorStr := '';
  if Connected then
    EscapeCommFunction(FHandle, funcs[onOff]);
end;

// COM port polling proc

procedure TE950Driver.TimerWndProc(var msg: TMessage);
var nRead, nToRead, dummy: DWORD;
  comStat: TCOMSTAT;
begin
  FErrorStr := '';
  if (msg.Msg = WM_TIMER) and Connected then
  begin
    // Do nothing if RX polling has been paused
    if FRXPollingPauses > 0 then
      exit;
    // If PacketSize is > 0 then raise the OnReceiveData event only if the RX
    // buffer has at least PacketSize bytes in it.
    ClearCommError(FHandle, dummy, @comStat);
    // Standard data handling
    nRead := 0;
    nToRead := comStat.cbInQue;
    with FOverLapped do begin
      Offset := 0;
      OffsetHigh := 0;
      hEvent := THandle(nil);
    end;
    if (nToRead > 0) and ReadFile(FHandle, FTempInBuffer^, nToRead, nRead, nil) then
      if (nRead <> 0) and Assigned(FOnReceiveData) then
        FOnReceiveData(Self, FTempInBuffer, nRead);
  end
  // Let Windows handle other messages
  else
    Msg.Result := DefWindowProc(FNotifyWnd, Msg.Msg, Msg.wParam, Msg.lParam);
end;

function TE950Driver.CheckAnswer(ASend: string; var AAnswer: string; LastCommand: Integer): WordBool;
  function CheckSumAnswer(const Str: string): Boolean;
  var
    i, AnswerSum: Integer;
    TestSum: Integer;
  begin
    FErrorStr := '';
    AnswerSum := 0;
    for i := 1 to Length(AAnswer) - 4 do
      AnswerSum := AnswerSum + Ord(AAnswer[i]);
    TestSum := StrToIntDef('$' + Copy(AAnswer, (Length(AAnswer) - 3), 4), 0);
    Result := TestSum = AnswerSum;
  end;

  function CheckPassword(const Str: string): Boolean;
  begin
    Result := DosToWin(Str) = Format('%-*s', [8, FPassword]);
  end;

  function CheckEnumComm(const Str: Char): Boolean;
  begin
    Result := Ord(Str) = FEnumComm;
  end;

  function CheckCommand(const Str: Char; const LCommand: Byte): Boolean;
  begin
    Result := Ord(Str) = LCommand;
  end;

  procedure SetFState(const AFState: string);
  var
    iState: Word;
  begin
    FErrorStr := '';
    iState := StrToIntDef('$' + AFState, -1);
    with FStateRegistrator.Fiskal do
    begin
      fsErrorFMem := iState and 1 > 0;
      fsErrorWMem := iState and 2 > 0;
      fsBattaryLow := iState and 4 > 0;
      fsUnknowCommand := iState and 8 > 0;
      fsErrorField := iState and 16 > 0;
      fsErrorCommand := iState and 32 > 0;
      fsOverTotal := iState and 64 > 0;
      fsFMemFull := iState and 128 > 0;
      fsFMemSoonFull := iState and 256 > 0;
      fsCertificated := iState and 512 > 0;
      fsFiskalited := iState and 1024 > 0;
      fsErrorPassDate := iState and 2048 > 0;
      fsDokCheckOpened := iState and 4096 > 0;
      fsCheckOpened := iState and 8192 > 0;
      fsDokOpened := iState and 16384 > 0;
      fsAllError := iState and 32768 > 0;
    end;
  end;

  procedure SetPState(const APState: string);
  var
    iState: Word;
  begin
    FErrorStr := '';
    iState := StrToIntDef('$' + APState, -1);
    with FStateRegistrator.Printer do
    begin
      psError := iState and 4 > 0;
      psOff := iState and 8 > 0;
      psLogLine := iState and 16 > 0;
      psCheckLine := iState and 32 > 0;
      psBuffFull := iState and 64 > 0;
      psBuffEmpty := iState and 128 > 0;
      psBeginDok := iState and 512 > 0;
      psEndDok := iState and 2048 > 0;
      psDokSmall := iState and 8192 > 0;
      psLineNo := iState and 16384 > 0;
      psAllError := iState and 32768 > 0;
    end;
  end;
var
  ErrStr: string;
  i, i1: Integer;
  SendStr1: string;
begin
  FErrorStr := '';
  i := 0;
  i1 := 0;
  Result := False;
  if CheckSumAnswer(AAnswer)
    then
    Result := True
  else
  begin
    try
      EnterCriticalSection(FCritSect);
      try
        repeat
          repeat
            SendString(ASend);
            SendStr1 := ReadAnswer();
            if SendStr1 = ''
              then
              DoOnError(SErrorNotAnswer);
            if (SendStr1[1] <> Chr($15))
              then
            begin
              if CheckSumAnswer(SendStr1)
                then
              begin
                i := 3;
                i1 := 3;
                AAnswer := Copy(SendStr1, 1, Length(SendStr1));
              end
              else
              begin
                Inc(i);
                i1 := 0;
              end;
            end
            else
            begin
              Inc(i1);
              i := 0;
            end;
          until i > 1;
        until i1 > 1;
        if i1 = 2
          then
          ErrStr := SErrorNak;
        if i = 2
          then ErrStr := '  ';
      finally
        LeaveCriticalSection(FCritSect);
      end;
    except
      DoOnError(ErrStr);
    end;
  end;
  if CheckPassword(Copy(AAnswer, 4, 8))
    then
    Result := Result = True
  else
    if ErrStr = ''
      then
      ErrStr := ' '
    else
      ErrStr := ErrStr + #10#13 + ' ';

  if CheckEnumComm(AAnswer[2])
    then
    Result := Result = True
  else
    if ErrStr = ''
      then
      ErrStr := '   '
    else
      ErrStr := ErrStr + #10#13 + '   ';
  if CheckCommand(AAnswer[3], LastCommand)
    then
    Result := Result = True
  else
    if ErrStr = ''
      then
      ErrStr := '    '
    else
      ErrStr := ErrStr + #10#13 + '    ';

  SetPState(Copy(AAnswer, 13, 4));
  if FStateRegistrator.Printer.psAllError
    then
  begin
    if FStateRegistrator.Printer.psError
      then
      if ErrStr = ''
        then
        ErrStr := SErrorP
      else
        ErrStr := ErrStr + #10#13 + SErrorP;
    if FStateRegistrator.Printer.psOff
      then
      if ErrStr = ''
        then
        ErrStr := SErrorPOff
      else
        ErrStr := ErrStr + #10#13 + SErrorPOff;

    if FStateRegistrator.Printer.psLogLine
      then
      if ErrStr = ''
        then
        ErrStr := SErrorPEndGL
      else
        ErrStr := ErrStr + #10#13 + SErrorPEndGL;

    if FStateRegistrator.Printer.psCheckLine
      then
      if ErrStr = ''
        then
        ErrStr := SErrorPEndCL
      else
        ErrStr := ErrStr + #10#13 + SErrorPEndCL;

    if FStateRegistrator.Printer.psDokSmall
      then
      if ErrStr = ''
        then
        ErrStr := SErrorPDokSmall
      else
        ErrStr := ErrStr + #10#13 + SErrorPDokSmall;

    if FStateRegistrator.Printer.psLineNo
      then
      if ErrStr = ''
        then
        ErrStr := SErrorPNoCG
      else
        ErrStr := ErrStr + #10#13 + SErrorPNoCG;
  end;

  SetFState(Copy(AAnswer, 18, 4));
  if FStateRegistrator.Fiskal.fsAllError
    then
  begin
    if FStateRegistrator.Fiskal.fsErrorFMem
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFSumFMem
      else
        ErrStr := ErrStr + #10#13 + SErrorFSumFMem;

    if FStateRegistrator.Fiskal.fsErrorWMem
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFSumWorkMeme
      else
        ErrStr := ErrStr + #10#13 + SErrorFSumWorkMeme;

    if FStateRegistrator.Fiskal.fsBattaryLow
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFLowBat
      else
        ErrStr := ErrStr + #10#13 + SErrorFLowBat;

    if FStateRegistrator.Fiskal.fsUnknowCommand
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFUnknown
      else
        ErrStr := ErrStr + #10#13 + SErrorFUnknown;

    if FStateRegistrator.Fiskal.fsErrorField
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFField
      else
        ErrStr := ErrStr + #10#13 + SErrorFField;

    if FStateRegistrator.Fiskal.fsErrorCommand
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFStat
      else
        ErrStr := ErrStr + #10#13 + SErrorFStat;

    if FStateRegistrator.Fiskal.fsOverTotal
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFOver
      else
        ErrStr := ErrStr + #10#13 + SErrorFOver;

    if FStateRegistrator.Fiskal.fsFMemFull
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFFullMem
      else
        ErrStr := ErrStr + #10#13 + SErrorFFullMem;

    if FStateRegistrator.Fiskal.fsFMemSoonFull
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFFullMem1
      else
        ErrStr := ErrStr + #10#13 + SErrorFSumFMem;

  end;

  if ErrStr <> ''
    then DoOnError(ErrStr);
end;

//    

procedure TE950Driver.SetEnumComm;
begin
  FErrorStr := '';
  if (FEnumComm < $7F) and (FEnumComm > $19)
    then Inc(FEnumComm)
  else FEnumComm := $20;
end;

function TE950Driver.ReadAnswer: WideString;
var
  InByte: Byte;
  blnR: Boolean;
  fiscalstatus, printerstatus: Integer;
begin
  FErrorStr := '';
  InByte := 0;
  fiscalstatus := 0;
  printerstatus := 0;

  Result := Chr($15);
  RLed.LEDOn := True;
  repeat
    blnR := ReadByte(InByte);
  until (InByte in [$2, $12, $14, $15]) or (not blnR);
  if (InByte <> $15) and BlnR then begin
    if InByte = $02
      then Result := Chr(InByte)
    else Result := '';
    repeat
      ReadByte(InByte);
      if not (InByte in [$12, $14, $15])
        then Result := Result + Chr(InByte);
    until InByte = $03;
    // Result := Result + Chr(InByte);
    ReadByte(InByte);
    Result := Result + Chr(InByte);
    ReadByte(InByte);
    Result := Result + Chr(InByte);
    ReadByte(InByte);
    Result := Result + Chr(InByte);
    ReadByte(InByte);
    Result := Result + Chr(InByte);
    fiscalstatus := StrToIntDef('$' + Copy(Result, 18, 4), 0);
    printerstatus := StrToIntDef('$' + Copy(Result, 13, 4), 0);
  end else begin
    if InByte = $0
      then Result := '';
  end;
  FLst.Values['statusprinter'] := IntToStr(printerstatus);
  FLst.Values['statusfiscal'] := IntToStr(fiscalstatus);
  RLed.LEDOn := False;
end;
  {  }

//$20    
//$21      

procedure TE950Driver.TestFBlock(A2Printer: WordBool; var CountBlock, CountSuccBlock, CountErrorBlock, CountFreeBlock: Integer);
var
  SendStr,
    SendStr1: string;
  i: Integer;
  cmd: Byte;
begin
  FErrorStr := '';
  FLst.Values['CountBlock'] := '';
  FLst.Values['CountSuccBlock'] := '';
  FLst.Values['CountErrorBlock'] := '';
  FLst.Values['CountFreeBlock'] := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        if A2Printer
          then cmd := $21
        else cmd := $20;
        SendStr := BByte + Chr(FEnumComm) + Chr(cmd) + Format('%-*s', [8, WinToDos(FPassword)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          if CheckAnswer(SendStr, SendStr1, cmd) then begin
            Delete(SendStr1, 1, 22);
            CountBlock := StrToIntDef(Copy(SendStr1, 1, Pos(DByte, SendStr1) - 1), 0);
            Delete(SendStr1, 1, Pos(DByte, SendStr1));
            CountSuccBlock := StrToIntDef(Copy(SendStr1, 1, Pos(DByte, SendStr1) - 1), 0);
            Delete(SendStr1, 1, Pos(DByte, SendStr1));
            CountErrorBlock := StrToIntDef(Copy(SendStr1, 1, Pos(DByte, SendStr1) - 1), 0);
            Delete(SendStr1, 1, Pos(DByte, SendStr1));
            CountFreeBlock := StrToIntDef(Copy(SendStr1, 1, Pos(DByte, SendStr1) - 1), 0);
            FLst.Values['CountBlock'] := IntToStr(CountBlock);
            FLst.Values['CountSuccBlock'] := IntToStr(CountSuccBlock);
            FLst.Values['CountErrorBlock'] := IntToStr(CountErrorBlock);
            FLst.Values['CountFreeBlock'] := IntToStr(CountFreeBlock);
          end;
          i := 3;
        end else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;


    // $22,  .

procedure TE950Driver.ReportDiag;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($22) +
          Format('%-*s', [8, WinToDos(FPassword)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if SendStr1[1] <> Chr($15)
          then
        begin
          CheckAnswer(SendStr, SendStr1, $22);
          i := 3;
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $2c,      .

procedure TE950Driver.SetBlockWorkMem(SetOf: Integer; InfoBlock: WideString);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($2C) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        IntToStr(SetOf) + DByte +
          InfoBlock + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if SendStr1[1] <> Chr($15) then begin
          CheckAnswer(SendStr, SendStr1, $2C);
          i := 3;
        end else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $2d,      .

function TE950Driver.GetBlockWorkMem(SetOf, LenBlock: Integer): WideString;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FLst.Values['GetBlockWorkMem'] := '';
  FErrorStr := '';
  Result := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($2D) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        IntToStr(SetOf) + DByte +
          IntToSTr(LenBlock) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if SendStr1[1] <> Chr($15) then begin
          CheckAnswer(SendStr, SendStr1, $2D);
          i := 3;
        end else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
      Delete(SendStr1, 1, 22);
      SetLength(Result, Pos(EByte, SendStr1) - 1);
      Result := Copy(SendStr1, 1, Pos(EByte, SendStr1) - 1);
      FLst.Values['GetBlockWorkMem'] := Result;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $2e,    .

procedure TE950Driver.PrintTestPrinter;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($2E) +
          Format('%-*s', [8, WinToDos(FPassword)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          CheckAnswer(SendStr, SendStr1, $2E);
          i := 3;
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $2f,   .

procedure TE950Driver.ResetRegistrator;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($2F) +
          Format('%-*s', [8, WinToDos(FPassword)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $2F);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $2a,    .

procedure TE950Driver.GetStatusRegistrator(var LastFDok: Integer; var FirstDokDateTime: TDateTime;
  var LastClose, CountFiskals: Integer; var VersionROM: WideString; var Lastdate: TDateTime; var SumMoney: Currency);
var
  SendStr,
    SendStr1: string;
  Month, Day, Year: Word;
  Hour, Min, Sec: Word;
  i: Integer;
begin
  FErrorStr := '';
  FLst.Values['LastFDok'] := '';
  FLst.Values['FirstDokDate'] := '';
  FLst.Values['LastClose'] := '';
  FLst.Values['CountFiskals'] := '';
  FLst.Values['VersionROM'] := '';
  FLst.Values['Lastdate'] := '';
  FLst.Values['SumMoney'] := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($2A) +
          Format('%-*s', [8, WinToDos(FPassword)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $2A)
            then
          begin
            Delete(SendStr1, 1, 22);
            LastFDok := StrToIntDef(Copy(SendStr1, 1, Pos(DByte, SendStr1) - 1), 0);
            Delete(SendStr1, 1, Pos(DByte, SendStr1));
            Year := StrToInt(Copy(SendStr1, 1, 2));
            if Year < 99
              then
              Year := 2000 + Year
            else
              Year := 1900 + Year;
            Delete(SendStr1, 1, 2);
            Month := StrToInt(Copy(SendStr1, 1, 2));
            Delete(SendStr1, 1, 2);
            Day := StrToInt(Copy(SendStr1, 1, 2));
            FirstDokDateTime := EncodeDate(Year, Month, Day);
            Delete(SendStr1, 1, Pos(DByte, SendStr1));
            Hour := StrToInt(Copy(SendStr1, 1, 2));
            Delete(SendStr1, 1, 2);
            Min := StrToInt(Copy(SendStr1, 1, 2));
            Delete(SendStr1, 1, 2);
            Sec := StrToInt(Copy(SendStr1, 1, 2));
            FirstDokDateTime := FirstDokDateTime + EncodeTime(Hour, Min, Sec, 0);
            Delete(SendStr1, 1, Pos(DByte, SendStr1));
            LastClose := StrToIntDef(Copy(SendStr1, 1, Pos(DByte, SendStr1) - 1), 0);
            Delete(SendStr1, 1, Pos(DByte, SendStr1));
            CountFiskals := StrToIntDef(Copy(SendStr1, 1, Pos(DByte, SendStr1) - 1), 0);
            Delete(SendStr1, 1, Pos(DByte, SendStr1));
            VersionROM := Copy(SendStr1, 1, Pos(DByte, SendStr1) - 1);
            Delete(SendStr1, 1, Pos(DByte, SendStr1));
            Year := StrToInt(Copy(SendStr1, 1, 2));
            if Year <= 96
              then
              Year := 2000 + Year
            else
              Year := 1900 + Year;
            Delete(SendStr1, 1, 2);
            Month := StrToInt(Copy(SendStr1, 1, 2));
            Delete(SendStr1, 1, 2);
            Day := StrToInt(Copy(SendStr1, 1, 2));
            Lastdate := EncodeDate(Year, Month, Day);
            Delete(SendStr1, 1, Pos(DByte, SendStr1));
            SumMoney := StrToIntDef(Copy(SendStr1, 1, Pos(DByte, SendStr1) - 1), 0);
            Delete(SendStr1, 1, Pos(DByte, SendStr1));
              {}
            FLst.Values['LastFDok'] := IntToStr(LastFDok);
            FLst.Values['FirstDokDateTime'] := DateTimeToStr(FirstDokDateTime);
            FLst.Values['LastClose'] := IntToStr(LastClose);
            FLst.Values['CountFiskals'] := IntToStr(CountFiskals);
            FLst.Values['VersionROM'] := VersionROM;
            FLst.Values['Lastdate'] := DateToStr(LastDate);
            SumMoney := SumMoney / 100;
            FLst.Values['SumMoney'] := Format('%.2f', [SumMoney]);
          end;
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

{    .}
{  .}
// $30,   .

procedure TE950Driver.GetDmpSertificate(SendToPrint: WordBool;
  var SN: WideString; var PosPoint, Devaluations, CurrDef, TypePrn: Integer;
  var PasswordFiskal, PasswordUser: WideString; var DateSer: TDateTime);
var
  Str,
    SendStr,
    SendStr1: string;
  Month, Day, Year: word;
  i: Integer;
begin
  FErrorStr := '';
  FLst.Values['SN'] := '';
  FLst.Values['PosPoint'] := '';
  FLst.Values['Devaluations'] := '';
  FLst.Values['CurrDef'] := '';
  FLst.Values['TypePrn'] := '';
  FLst.Values['PasswordFiskal'] := '';
  FLst.Values['PasswordUser'] := '';
  FLst.Values['DateSer'] := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($30) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        if SendToPrint
          then SendStr := SendStr + Chr($50)
        else SendStr := SendStr + Chr($20);
        SendStr := SendStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $30) then begin
            Str := Copy(SendStr1, 23, Pos(#$03, SendStr1) - 23);
            SN := Copy(Str, 1, Pos(DByte, SendStr1) - 1);
            Delete(Str, 1, Pos(DByte, Str));
            PosPoint := Byte(StrToInt(Copy(Str, 1, Pos(DByte, Str) - 1)));
            Delete(Str, 1, Pos(DByte, Str));
            Devaluations := Byte(StrToInt(Copy(Str, 1, Pos(DByte, Str) - 1)));
            Delete(Str, 1, Pos(DByte, Str));
            CurrDef := Byte(StrToInt(Copy(Str, 1, Pos(DByte, Str) - 1)));
            Delete(Str, 1, Pos(DByte, Str));
            TypePrn := Byte(StrToInt(Copy(Str, 1, Pos(DByte, Str) - 1)));
            Delete(Str, 1, Pos(DByte, Str));


            PasswordFiskal := Copy(Str, 1, Pos(DByte, Str) - 1);
            Delete(Str, 1, Pos(DByte, Str));
            PasswordUser := Copy(Str, 1, Pos(DByte, Str) - 1);
            Delete(Str, 1, Pos(DByte, Str));

            Year := StrToInt(Copy(Str, 1, 2));
            if Year <= 96
              then Year := 2000 + Year
            else Year := 1900 + Year;
            Month := StrToInt(Copy(Str, 3, 2));
            Day := StrToInt(Copy(Str, 5, 2));
            DateSer := EncodeDate(Year, Month, Day);
           {}
            FLst.Values['SN'] := SN;
            FLst.Values['PosPoint'] := IntToStr(PosPoint);
            FLst.Values['Devaluations'] := IntToStr(Devaluations);
            FLst.Values['CurrDef'] := IntToStr(CurrDef);
            FLst.Values['TypePrn'] := IntToStr(TypePrn);
            FLst.Values['PasswordFiskal'] := PasswordFiskal;
            FLst.Values['PasswordUser'] := PasswordUser;
            FLst.Values['DateSer'] := DateToStr(DateSer);
          end;
        end else Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $31,   .

procedure TE950Driver.LoadSertificate(SN: WideString; PosPoint, Devaluations, CurrDef, TypePrn: Integer;
  PasswordFiskal, PasswordUser: WideString; DateSer: TDateTime);
var
  SendStr,
    SendStr1: string;
  DateStr: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        DateTimeToString(DateStr, 'yymmdd', DateSer);
        SendStr := BByte + Chr(FEnumComm) + Chr($31) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        SendStr := SendStr + SN + DByte;
        SendStr := SendStr + IntToStr(PosPoint) + DByte;
        SendStr := SendStr + IntToSTr(Devaluations) + DByte;
        SendStr := SendStr + IntToStr(CurrDef) + DByte;
        SendStr := SendStr + IntToStr(TypePrn) + DByte;
        SendStr := SendStr + WinToDos(PasswordFiskal) + DByte;
        SendStr := SendStr + WinToDos(PasswordUser) + DByte;
        SendStr := SendStr + DateStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if SendStr1[1] <> Chr($15)
          then
        begin
          CheckAnswer(SendStr, SendStr1, $31);
          i := 3;
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $32, .

procedure TE950Driver.Certification;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($32) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        Format('%-*s', [8, WinToDos(FInspPassword)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $32);
        end
        else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

{  .}
// $33,   .

procedure TE950Driver.GetDmpFiskal(SendToPrint: WordBool;
  var RN, UserFR, PasswordFiskal, PasswordUser: WideString; var DateReg: TDateTime);
var
  Str,
    SendStr,
    SendStr1: string;
  Month, Day, Year: word;
  i: Integer;
begin
  FErrorStr := '';
  FLst.Values['RN'] := '';
  FLst.Values['UserFR'] := '';
  FLst.Values['PasswordFiskal'] := '';
  FLst.Values['PasswordUser'] := '';
  FLst.Values['DateReg'] := '';

  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($33) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        if SendToPrint
          then SendStr := SendStr + Chr($50)
        else SendStr := SendStr + Chr($20);
        SendStr := SendStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $33) then begin
            Str := Copy(SendStr1, 23, Pos(#$03, SendStr1) - 23);
            RN := Copy(Str, 1, Pos(DByte, Str) - 1);
            Delete(Str, 1, Pos(DByte, Str));
            UserFR := Copy(Str, 1, Pos(DByte, Str) - 1);
            Delete(Str, 1, Pos(DByte, Str));
            PasswordFiskal := Copy(Str, 1, Pos(DByte, Str) - 1);
            Delete(Str, 1, Pos(DByte, Str));
            PasswordUser := Copy(Str, 1, Pos(DByte, Str) - 1);
            Delete(Str, 1, Pos(DByte, Str));
            Year := StrToInt(Copy(Str, 1, 2));
            if Year <= 96
              then Year := 2000 + Year
            else Year := 1900 + Year;
            Month := StrToInt(Copy(Str, 3, 2));
            Day := StrToInt(Copy(Str, 5, 2));
            DateReg := EncodeDate(Year, Month, Day);

            FLst.Values['RN'] := RN;
            FLst.Values['UserFR'] := UserFR;
            FLst.Values['PasswordFiskal'] := PasswordFiskal;
            FLst.Values['PasswordUser'] := PasswordUser;
            FLst.Values['DateReg'] := DateToStr(DateReg);
          end;
        end else Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $34,   .

procedure TE950Driver.LoadFiskal(RN, UserFR, PasswordFiskal, PasswordUser: WideString; DateReg: TDateTime);
var
  SendStr,
    SendStr1: string;
  DateStr: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        DateTimeToString(DateStr, 'yymmdd', DateReg);
        SendStr := BByte + Chr(FEnumComm) + Chr($34) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        SendStr := SendStr + RN + DByte +
          WinToDos(UserFR) + DByte +
          WinToDos(PasswordFiskal) + DByte +
          WinToDos(PasswordUser) + DByte +
          DateStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if SendStr1[1] <> Chr($15) then begin
          CheckAnswer(SendStr, SendStr1, $34);
          i := 3;
        end else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $35, .

procedure TE950Driver.Fiskalization;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($35) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        Format('%-*s', [8, WinToDos(FInspPassword)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $35);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $37,   

procedure TE950Driver.GetFMemSize(var AllMem, UsesMem: Integer);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  FLst.Values['AllMem'] := '';
  FLst.Values['UsesMem'] := '';

  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($37) +
          Format('%-*s', [8, WinToDos(FPassword)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $37);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      Delete(SendStr1, 1, 22);
      AllMem := StrToIntDef(Copy(SendStr1, 1, Pos(DByte, SendStr1) - 1), 0);
      Delete(SendStr1, 1, Pos(DByte, SendStr1));
      UsesMem := StrToIntDef(Copy(SendStr1, 1, Pos(EByte, SendStr1) - 1), 0);
      FLst.Values['AllMem'] := IntToStr(AllMem);
      FLst.Values['UsesMem'] := IntToStr(UsesMem);

    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $39,  .

procedure TE950Driver.CloseSmena(Fix: Boolean;
  var NumClose, CountFCheckComplect, CountFCheckNotComplect, CountFDokComplect,
  CountFDokNotComplect, CountFDokAnulComplect, CountCheckDokService: Integer;
  var SumSale, SumMargin, SumDiscount, SumContainer: Currency;
  var CountCheckDokBack: Integer;
  var SumSaleBack, SumMarginBack, SumDiscountBack, SumContainerBack: Currency;
  var CountCheckDokPayment: Integer;
  var SumPension, SumTranslation, SumComission: Currency;

  var CountCheckDokPaymentBack: Integer;
  var SumPensionBack, SumTranslationBack, SumComissionBack: Currency;
  var CountCheckDokCashe: Integer;
  var SumCashe: Currency;
  var CountCheckDokCasheBack: Integer;
  var SumCasheBack: Currency);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  FLst.Values['NumClose'] := '';
  FLst.Values['CountFCheckComplect'] := '';
  FLst.Values['CountFCheckNotComplect'] := '';
  FLst.Values['CountFDokComplect'] := '';
  FLst.Values['CountFDokNotComplect'] := '';
  FLst.Values['CountFDokAnulComplect'] := '';
  FLst.Values['CountCheckDokService'] := '';
  FLst.Values['SumSale'] := '';
  FLst.Values['SumMargin'] := '';
  FLst.Values['SumDiscount'] := '';
  FLst.Values['SumContainer'] := '';
  FLst.Values['CountCheckDokBack'] := '';
  FLst.Values['SumSaleBack'] := '';
  FLst.Values['SumMarginBack'] := '';
  FLst.Values['SumDiscountBack'] := '';
  FLst.Values['SumContainerBack'] := '';
  FLst.Values['CountCheckDokPayment'] := '';
  FLst.Values['SumPension'] := '';
  FLst.Values['SumTranslation'] := '';
  FLst.Values['SumComission'] := '';
  FLst.Values['CountCheckDokPaymentBack'] := '';
  FLst.Values['SumPensionBack'] := '';
  FLst.Values['SumTranslationBack'] := '';
  FLst.Values['SumComissionBack'] := '';
  FLst.Values['CountCheckDokCashe'] := '';
  FLst.Values['SumCashe'] := '';
  FLst.Values['CountCheckDokCasheBack'] := '';
  FLst.Values['SumCasheBack'] := '';

  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($39) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        if Fix
          then
          SendStr := SendStr + Chr($5A)
        else
          SendStr := SendStr + Chr($50);
        SendStr := SendStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $39);
          Delete(SendStr1, 1, 22);
          Delete(SendStr1, Pos(#$03, SendStr1), 5);
          NumClose := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          CountFCheckComplect := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          CountFCheckNotComplect := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          CountFDokComplect := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          CountFDokNotComplect := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          CountFDokAnulComplect := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          CountCheckDokService := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumSale := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumMargin := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumDiscount := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumContainer := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          CountCheckDokBack := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumSaleBack := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumMarginBack := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumDiscountBack := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumContainerBack := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          CountCheckDokPayment := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumPension := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumTranslation := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumComission := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          CountCheckDokPaymentBack := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumPensionBack := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumTranslationBack := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumComissionBack := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          CountCheckDokCashe := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumCashe := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          CountCheckDokCasheBack := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          SumCasheBack := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);

          FLst.Values['NumClose'] := IntToStr(NumClose);
          FLst.Values['CountFCheckComplect'] := IntToStr(CountFCheckComplect);
          FLst.Values['CountFCheckNotComplect'] := IntToStr(CountFCheckNotComplect);
          FLst.Values['CountFDokComplect'] := IntToStr(CountFDokComplect);
          FLst.Values['CountFDokNotComplect'] := IntToStr(CountFDokNotComplect);
          FLst.Values['CountFDokAnulComplect'] := IntToStr(CountFDokAnulComplect);
          FLst.Values['CountCheckDokService'] := IntToStr(CountCheckDokService);
          SumSale := SumSale / 100;
          FLst.Values['SumSale'] := Format('%.2f', [SumSale]);
          SumMargin := SumMargin / 100;
          FLst.Values['SumMargin'] := Format('%.2f', [SumMargin]);
          SumDiscount := SumDiscount / 100;
          FLst.Values['SumDiscount'] := Format('%.2f', [SumDiscount]);
          SumContainer := SumContainer / 100;
          FLst.Values['SumContainer'] := Format('%.2f', [SumContainer]);
          FLst.Values['CountCheckDokBack'] := IntToStr(CountCheckDokBack);
          SumSaleBack := SumSaleBack / 100;
          FLst.Values['SumSaleBack'] := Format('%.2f', [SumSaleBack]);
          SumMarginBack := SumMarginBack / 100;
          FLst.Values['SumMarginBack'] := Format('%.2f', [SumMarginBack]);
          SumDiscountBack := SumDiscountBack / 100;
          FLst.Values['SumDiscountBack'] := Format('%.2f', [SumDiscountBack]);
          SumContainerBack := SumContainerBack / 100;
          FLst.Values['SumContainerBack'] := Format('%.2f', [SumContainerBack]);
          FLst.Values['CountCheckDokPayment'] := IntToStr(CountCheckDokPayment);
          SumPension := SumPension / 100;
          FLst.Values['SumPension'] := Format('%.2f', [SumPension]);
          SumTranslation := SumTranslation / 100;
          FLst.Values['SumTranslation'] := Format('%.2f', [SumTranslation]);
          SumComission := SumComission / 100;
          FLst.Values['SumComission'] := Format('%.2f', [SumComission]);
          FLst.Values['CountCheckDokPaymentBack'] := IntToStr(CountCheckDokPaymentBack);
          SumPensionBack := SumPensionBack / 100;
          FLst.Values['SumPensionBack'] := Format('%.2f', [SumPensionBack]);
          SumTranslationBack := SumTranslationBack / 100;
          FLst.Values['SumTranslationBack'] := Format('%.2f', [SumTranslationBack]);
          SumComissionBack := SumComissionBack / 100;
          FLst.Values['SumComissionBack'] := Format('%.2f', [SumComissionBack]);
          FLst.Values['CountCheckDokCashe'] := IntToStr(CountCheckDokCashe);
          SumCashe := SumCashe / 100;
          FLst.Values['SumCashe'] := Format('%.2f', [SumCashe]);
          FLst.Values['CountCheckDokCasheBack'] := IntToStr(CountCheckDokCasheBack);
          SumCasheBack := SumCasheBack / 100;
          FLst.Values['SumCasheBack'] := Format('%.2f', [SumCasheBack]);
        end else Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $3a,    .

procedure TE950Driver.ReportOfDates(BeginDate, EndDate: TDateTime; ItogOnly: WordBool);
var
  SendStr,
    SendStr1: string;
  DateStr: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($3A) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        Format('%-*s', [8, WinToDos(FInspPassword)]) + DByte;
        DateTimeToString(DateStr, 'yymmdd', BeginDate);
        SendStr := SendStr + DateStr + DByte;
        DateTimeToString(DateStr, 'yymmdd', EndDate);
        SendStr := SendStr + DateStr + DByte;
        if ItogOnly
          then
          SendStr := SendStr + Chr($54) + EByte
        else
          SendStr := SendStr + Chr($55) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $3A);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $3b,      .

procedure TE950Driver.ReportOfSmena(BeginSmena, EndSmena: Integer; ItogOnly: WordBool);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($3B) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        Format('%-*s', [8, WinToDos(FInspPassword)]) + DByte;
        SendStr := SendStr + IntToStr(BeginSmena) + DByte;
        SendStr := SendStr + IntToStr(EndSmena) + DByte;
        if ItogOnly
          then
          SendStr := SendStr + Chr($54) + EByte
        else
          SendStr := SendStr + Chr($55) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $3B);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

//  $3f,   .

function TE950Driver.ChangePassword(PasswordOld, PasswordNew: WideString): WideString;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  FLst.Values['ChangePassword'] := '';
  Result := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($3F) +
          Format('%-*s', [8, WinToDos(PasswordOld)]) + DByte +
        Format('%-*s', [8, WinToDos(PasswordNew)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $3F);
          SetLength(Result, Length(PasswordNew));
          Result := PasswordNew;
          FLst.Values['ChangePassword'] := PasswordNew;
        end
        else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

 {   .}
// $40,   .

procedure TE950Driver.OpenCheck(TypeCheck: TxTypeCheck; IndxCurr: Integer);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  DoBeforeOpenCheck;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($40) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        case TypeCheck of
          Service: SendStr := SendStr + Chr($53);
          ServiceBack: SendStr := SendStr + Chr($52);
          Payment: SendStr := SendStr + Chr($42);
          PaymentBack: SendStr := SendStr + Chr($43);
          Cashe: SendStr := SendStr + Chr($50);
          CasheBack: SendStr := SendStr + Chr($54);
        end;
        SendStr := SendStr + DByte + '1';
        SendStr := SendStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $40);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterOpenCheck;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $41,   .

procedure TE950Driver.PrintFiskalTxt(IndxDesc: Integer; FiskalTxt: WideString);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  DoBeforePrnTxtCheck;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($41) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        SendStr := SendStr + IntToStr(IndxDesc) + DByte;
        SendStr := SendStr + Format('%-*s', [20, WinToDos(FiskalTxt)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $41);
        end
        else
          inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterPrnTxtCheck;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $42,    .

procedure TE950Driver.PrintFiskalStr(IndxDescStr: Integer; DefineDescStr: WideString;
  Count: Double; IndxDescUM: Integer; Price: Currency; IndxDescCurr: Integer; TypeStr: TTypeStrFDok; ProcMargine, ProcDiscount: Double);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  DoBeforePrnStrCheck;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($42) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        SendStr := SendStr + IntToStr(IndxDescStr) + DByte;
        SendStr := SendStr + WinToDos(DefineDescStr + DByte);
        SendStr := SendStr + IntToStr(Round(Count * 1000)) + DByte;
        SendStr := SendStr + IntToStr(IndxDescUM) + DByte;
        SendStr := SendStr + IntToStr(Round(Price * 100)) + DByte;
        SendStr := SendStr + IntToStr(IndxDescCurr) + DByte;
        SendStr := SendStr + TypeStr + DByte;
        SendStr := SendStr + IntToStr(Round(ProcMargine * 100)) + DByte;
        SendStr := SendStr + IntToStr(Round(ProcDiscount * 100));
        SendStr := SendStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $42);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterPrnStrCheck;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $43,   .

procedure TE950Driver.SubTotal(PrintSubTotal, PrintCheck: WordBool; IndxDescSubTotal: Integer;
  DefineSubTotal: WideString; IndDescCurr: Integer;
  ProcMargine, ProcDiscount: Double;
  var TypeCheck: TxTypeCheck; var CountLine: Integer;
  var TotalSale, TotalMargine, TotalDiscount, TotalContainer, InCashe: Currency);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  FLst.Values['TypeCheck'] := '';
  FLst.Values['CountLine'] := '';
  FLst.Values['TotalSale'] := '';
  FLst.Values['TotalMargine'] := '';
  FLst.Values['TotalDiscount'] := '';
  FLst.Values['TotalContainer'] := '';
  FLst.Values['InCashe'] := '';

  DoBeforePrnSubTotalCheck;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($43) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        if PrintCheck
          then
          SendStr := SendStr + Chr($50) + DByte
        else
          SendStr := SendStr + Chr($49) + DByte;
        SendStr := SendStr + IntToStr(IndxDescSubTotal) + DByte;
        SendStr := SendStr + WinToDos(DefineSubTotal) + DByte;
        SendStr := SendStr + IntToStr(IndDescCurr) + DByte;
        SendStr := SendStr + IntToStr(Round(ProcMargine * 100)) + DByte;
        SendStr := SendStr + IntToStr(Round(ProcDiscount * 100)) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $43)
            then
            Delete(SendStr1, 1, 22);
          case Ord(SendStr1[1]) of
            $42: TypeCheck := Payment;
            $43: TypeCheck := PaymentBack;
            $52: TypeCheck := ServiceBack;
            $53: TypeCheck := Service;
          end;
          Delete(SendStr1, 1, 2);
          CountLine := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          TotalSale := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          TotalMargine := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          TotalDiscount := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          TotalContainer := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          InCashe := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($03), SendStr1) - 1), 0);

          FLst.Values['TypeCheck'] := IntToStr(Ord(TypeCheck));
          FLst.Values['CountLine'] := IntToStr(CountLine);
          TotalSale := TotalSale / 100;
          FLst.Values['TotalSale'] := Format('%.2f', [TotalSale]);
          TotalMargine := TotalMargine / 100;
          FLst.Values['TotalMargine'] := Format('%.2f', [TotalMargine]);
          TotalDiscount := TotalDiscount / 100;
          FLst.Values['TotalDiscount'] := Format('%.2f', [TotalDiscount]);
          TotalContainer := TotalContainer / 100;
          FLst.Values['TotalContainer'] := Format('%.2f', [TotalContainer]);
          InCashe := InCashe / 100;
          FLst.Values['InCashe'] := Format('%.2f', [InCashe]);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterPrnSubTotalCheck;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $44,   .

function TE950Driver.CheckTotal(InSum: Currency; IndxCurr, IndxOpl: Integer;
  DefOpl: WideString; Tender: WordBool): Currency;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  FLst.Values['CheckTotal'] := '';

  Result := 0;
  DoBeforePrnTotalCheck;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($44) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        SendStr := SendStr + IntToStr(IndxOpl) + DByte;
        SendStr := SendStr + WinToDos(Format('%-*s', [15, DefOpl])) + DByte;
        SendStr := SendStr + IntToStr(Round(InSum * 100)) + DByte;
        SendStr := SendStr + IntToStr(IndxCurr) + DByte;
        if Tender
          then
          SendStr := SendStr + Chr($54)
        else
          SendStr := SendStr + Chr($43);
        SendStr := SendStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $44)
            then
          begin
            Delete(SendStr1, 1, 22);
            Delete(SendStr1, Pos(Chr($03), SendStr1), 6);
            Result := StrToIntDef(SendStr1, 0);
            Result := Result / 100;
            FLst.Values['CheckTotal'] := Format('%.2f', [Result]);
          end;
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterPrnTotalCheck;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $45,   .

function TE950Driver.CloseCheck: Integer;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  Result := 0;
  FLst.Values['CloseCheck'] := '';

  DoBeforeCloseCheck;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($45) +
          Format('%-*s', [8, WinToDos(FPassword)]);
        SendStr := SendStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $45) then begin
            Result := StrToIntDef(Copy(SendStr1, 23, Pos(EByte, SendStr) - 23), 0);
            FLst.Values['CloseCheck'] := IntToStr(Result);
          end;
        end else Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterCloseCheck;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

{   . ----------------------------------------------}
// $48,   .

procedure TE950Driver.OpenNFCheck(IndxNCheck: Integer; DefineDescNCheck: WideString);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  DoBeforeOpenNCheck;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($48) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        IntToStr(IndxNCheck) + DByte +
          Format('%-*s', [15, WinToDos(DefineDescNCheck)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $48)
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterOpenNCheck;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $49,   .

procedure TE950Driver.PrintNFText(Str: WideString);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  DoBeforePrnTxtNCheck;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($49) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        SendStr := SendStr + Format('%-*s', [40, WinToDos(Str)]);
        SendStr := SendStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $49)
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterPrnTxtNCheck;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $4a,   .

procedure TE950Driver.CloseNFCheck;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  DoBeforeCloseNCheck;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($4A) +
          Format('%-*s', [8, WinToDos(FPassword)]);
        SendStr := SendStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $4A)
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterCloseNCheck;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $4b,   .

procedure TE950Driver.CutNFCheck;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($4B) +
          Format('%-*s', [8, WinToDos(FPassword)]);
        SendStr := SendStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $4B)
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

{   . }
// $50,   .

procedure TE950Driver.FeedLineCheck(CountLine: Integer);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($50) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        SendStr := SendStr + IntToStr(CountLine) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $50)
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $51,   .

procedure TE950Driver.FeedLineLog(CountLine: Integer);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($51) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        IntToStr(CountLine) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $51)
        end else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $52,     .

procedure TE950Driver.FeedLineBoth(CountLine: Integer);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($52) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        IntToStr(CountLine) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $52)
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $53,   .

procedure TE950Driver.FeedLineDok(CountLine: Integer);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($53) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        SendStr := SendStr + IntToStr(CountLine) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $53)
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $54,   .

procedure TE950Driver.EjectDok;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($54) +
          Format('%-*s', [8, WinToDos(FPassword)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $54)
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $55,  Slip- .

function TE950Driver.ConditionSLIP(CountFreeLine, Pos1, Pos2: Integer): Integer;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  Result := 0;
  FLst.Values['ConditionSLIP'] := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($55) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        SendStr := SendStr + IntToSTr(CountFreeLine) + DByte;
        SendStr := SendStr + format('%2.2d', [Pos1]) + DByte;
        SendStr := SendStr + format('%2.2d', [Pos2]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $55) then begin
            Result := StrToIntDef(Copy(SendStr1, 23, Pos(EByte, SendStr) - 23), 0);
            FLst.Values['ConditionSLIP'] := IntToStr(Result);
          end;
        end
        else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $57,   

procedure TE950Driver.FontPrn(Font: Integer);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($57) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        Chr((Font and $F) + $30) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $57)
        end else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

{  ,    ----------------------------------- }
// $58,    .

procedure TE950Driver.SetDateTime(ADateTime: TDateTime);
var
  SendStr,
    SendStr1: string;
  TimeStr, DateStr: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        DateTimeToString(TimeStr, 'hhmmss', ADateTime);
        DateTimeToString(DateStr, 'yymmdd', ADateTime);
        SendStr := BByte + Chr(FEnumComm) + Chr($58) +
          Format('%-*s', [8, WinToDos(FPassword)]) +
        DByte + DateStr + DByte + TimeStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $58);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $59,        .

function TE950Driver.GetDateTime: TDateTime;
var
  SendStr,
    SendStr1: string;
  Month, Day, Year: Word;
  Hour, Min, Sec: Word;
  i: Integer;
begin
  FErrorStr := '';
 // FLst.Values['GetDateTime']:= '';
  FLst.Values['DateTimeFR'] := '';
  Result := 0;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($59) +
          Format('%-*s', [8, WinToDos(FPassword)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $59) then begin
            Hour := StrToInt(Copy(SendStr1, 30, 2));
            Min := StrToInt(Copy(SendStr1, 32, 2));
            Sec := StrToInt(Copy(SendStr1, 34, 2));
            Year := StrToInt(Copy(SendStr1, 23, 2));
            if Year <= 96
              then Year := 2000 + Year
            else Year := 1900 + Year;
            Month := StrToInt(Copy(SendStr1, 25, 2));
            Day := StrToInt(Copy(SendStr1, 27, 2));
            Result := EncodeDate(Year, Month, Day) + EncodeTime(Hour, Min, Sec, 0);
           // FLst.Values['GetDateTime']:= DateToStr(Result);
            FLst.Values['DateTimeFR'] := DateTimeToStr(Result);
          end;
        end
        else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

{   Slip-. --------------------------------------------}
// $60,    .

procedure TE950Driver.OpenFDok(TDok: TxTypeDok;
  IndxDescCurr, CountFreeLine, PosHeader1, PosDesc, LenDesc, PosDefineDesc,
  LenDefineDesc, PosDescStr, LenDescStr, PosDefineStr, LenDefgineDescStr,
  PosSum, LenSum, PosDescTotal, LenDescTotal, PosDefineTotal, LenDefineToTal,
  PosSumTotal, LenSumTotal, PosHeader2: Integer);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  DoBeforeOpenDok;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($60) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        case TDok of
          ServiceDok: SendStr := SendStr + Chr($53);
          ServiceBackDok: SendStr := SendStr + Chr($52);
          PaymentDok: SendStr := SendStr + Chr($42);
          PaymentBackDok: SendStr := SendStr + Chr($43);
          CasheDok: SendStr := SendStr + Chr($50);
          CasheBackDok: SendStr := SendStr + Chr($54);
        end;
        SendStr := SendStr + DByte + format('%5.5d', [IndxDescCurr]) + DByte;
        SendStr := SendStr + format('%2.2d', [CountFreeLine]) + DByte;
        SendStr := SendStr + format('%2.2d', [PosHeader1]) + DByte;
        SendStr := SendStr + format('%2.2d', [PosDesc]) + DByte;
        SendStr := SendStr + format('%2.2d', [LenDesc]) + DByte;
        SendStr := SendStr + format('%2.2d', [PosDefineDesc]) + DByte;
        SendStr := SendStr + format('%2.2d', [LenDefineDesc]) + DByte;
        SendStr := SendStr + format('%2.2d', [PosDescStr]) + DByte;
        SendStr := SendStr + format('%2.2d', [LenDescStr]) + DByte;
        SendStr := SendStr + format('%2.2d', [PosDefineStr]) + DByte;
        SendStr := SendStr + format('%2.2d', [LenDefgineDescStr]) + DByte;
        SendStr := SendStr + format('%2.2d', [PosSum]) + DByte;
        SendStr := SendStr + format('%2.2d', [LenSum]) + DByte;
        SendStr := SendStr + format('%2.2d', [PosDescTotal]) + DByte;
        SendStr := SendStr + format('%2.2d', [LenDescTotal]) + DByte;
        SendStr := SendStr + format('%2.2d', [PosDefineTotal]) + DByte;
        SendStr := SendStr + format('%2.2d', [LenDefineToTal]) + DByte;
        SendStr := SendStr + format('%2.2d', [PosSumTotal]) + DByte;
        SendStr := SendStr + format('%2.2d', [LenSumTotal]) + DByte;
        SendStr := SendStr + format('%2.2d', [PosHeader2]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $60);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterOpenDok;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $61,    .

procedure TE950Driver.PrintDefineFDok(IndxDesc: Integer; DefineFDok: WideString);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  DoBeforePrnTxtDok;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($61) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        SendStr := SendStr + IntToStr(IndxDesc) + DByte;
        SendStr := SendStr + WinToDos(DefineFDok) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $61);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterPrnTxtDok;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $62,    .

procedure TE950Driver.PrintStrFDok(IndxDescStr: Integer; DefineDescStr: WideString;
  Count: Double; IndxDescUM: Integer; Price: Currency; IndxDescCurr: Integer; TypeStr: TTypeStrFDok;
  ProcMargine, ProcDiscount: Double);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  DoBeforePrnStrDok;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($62) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        IntToStr(indxdescStr) + DByte +
          WinToDos(DefineDescStr + DByte) +
          IntToStr(Round(Count * 1000)) + DByte +
          IntToStr(indxdescUM) + DByte +
          IntToStr(Round(Price * 100)) + DByte +
          IntToStr(indxdescCurr) + DByte +
          TypeStr + DByte +
          IntToStr(Round(ProcMargine * 100)) + DByte +
          IntToStr(Round(ProcDiscount * 100)) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $62);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterPrnStrDok;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $63,    

procedure TE950Driver.SubTotalFDok(PrintDok: WordBool; indxdescSubTotal: Integer; DefineSubTotal: WideString;
  IndDescCurr: Integer;
  ProcMargine, ProcDiscount: Double;
  var TDok: TxTypeDok; var CountLine: Integer;
  var TotalSale, TotalMargine, TotalDiscount, TotalContainer, InCashe: Currency);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  FLst.Values['TDok'] := '';
  FLst.Values['CountLine'] := '';
  FLst.Values['TotalSale'] := '';
  FLst.Values['TotalMargine'] := '';
  FLst.Values['TotalDiscount'] := '';
  FLst.Values['TotalContainer'] := '';
  FLst.Values['InCashe'] := '';
  DoBeforePrnSubTotalDok;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($63) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        if PrintDok
          then
          SendStr := SendStr + Chr($50) + DByte
        else
          SendStr := SendStr + Chr($49) + DByte;
        SendStr := SendStr + IntToStr(indxdescSubTotal) + DByte;
        SendStr := SendStr + WinToDos(DefineSubTotal) + DByte;
        SendStr := SendStr + IntToStr(IndDescCurr) + DByte;
        SendStr := SendStr + IntToStr(Round(ProcMargine * 100)) + DByte;
        SendStr := SendStr + IntToStr(Round(ProcDiscount * 100)) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $63)
            then
            Delete(SendStr1, 1, 22);
          case Ord(SendStr1[1]) of
            $42: TDok := PaymentDok;
            $43: TDok := PaymentBackDok;
            $52: TDok := ServiceBackDok;
            $53: TDok := ServiceDok;
          end;
          Delete(SendStr1, 1, 2);
          CountLine := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          TotalSale := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          TotalMargine := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          TotalDiscount := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          TotalContainer := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($1C), SendStr1) - 1), 0);
          Delete(SendStr1, 1, Pos(Chr($1C), SendStr1));
          InCashe := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($03), SendStr1) - 1), 0);

          FLst.Values['TDok'] := IntToStr(Ord(TDok));
          FLst.Values['CountLine'] := IntToStr(CountLine);
          TotalSale := TotalSale / 100;
          FLst.Values['TotalSale'] := Format('%.2f', [TotalSale]);
          TotalMargine := TotalMargine / 100;
          FLst.Values['TotalMargine'] := Format('%.2f', [TotalMargine]);
          TotalDiscount := TotalDiscount / 100;
          FLst.Values['TotalDiscount'] := Format('%.2f', [TotalDiscount]);
          TotalContainer := TotalContainer / 100;
          FLst.Values['TotalContainer'] := Format('%.2f', [TotalContainer]);
          InCashe := InCashe / 100;
          FLst.Values['InCashe'] := Format('%.2f', [InCashe]);
        end
        else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
      DoAfterPrnSubTotalDok;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $64,    .

function TE950Driver.TotalFDok(InSum: Currency; IndxCurr, IndxOpl: Integer; DefOpl: WideString; Tender: Boolean): Currency;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  Result := 0;
  FLst.Values['TotalFDok'] := '';
  DoBeforePrnTotalDok;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($64) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        IntToStr(IndxOpl) + DByte +
          WinToDos(Format('%-*s', [15, DefOpl])) + DByte +
          IntToStr(Round(InSum * 100)) + DByte +
          IntToStr(IndxCurr) + DByte;
        if Tender
          then SendStr := SendStr + Chr($54) { Terminate }
        else SendStr := SendStr + Chr($43); { Continue  }
        SendStr := SendStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $64) then begin
            Delete(SendStr1, 1, 22);
            Delete(SendStr1, Pos(Chr($03), SendStr1), 6);
            Result := StrToIntDef(SendStr1, -1);
            Result := Result / 100;
            FLst.Values['TotalFDok'] := Format('%.2f', [Result]);
          end;
        end else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
      DoAfterPrnTotalDok;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $65,    .

function TE950Driver.CloseFDok: Integer;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  Result := 0;
  FLst.Values['CloseFDok'] := '';
  DoBeforeCloseDok;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($65) +
          Format('%-*s', [8, WinToDos(FPassword)]);
        SendStr := SendStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $65) then begin
            Delete(SendStr1, 1, 22);
            Result := StrToIntDef(Copy(SendStr1, 1, Pos(Chr($03), SendStr1) - 1), 0);
         // Result:= StrToIntDef(Copy(SendStr1,23,Pos(EByte,SendStr)-23), 0);
            FLst.Values['CloseFDok'] := IntToStr(Result);
          end;
        end else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
      DoAfterCloseDok;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

  {   .}
    // $68,    .

procedure TE950Driver.OpenNDok(indxdesc: Integer;
  Defindxdesc: WideString; EmptyStr, PosHeader1, PosHeader2: Integer);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  DoBeforeOpenNDok;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($68) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        SendStr := SendStr + IntToStr(indxdesc) + DByte;
        SendStr := SendStr + WinToDos(Defindxdesc) + DByte;
        SendStr := SendStr + IntToStr(EmptyStr) + DByte;
        SendStr := SendStr + IntToStr(PosHeader1) + DByte;
        SendStr := SendStr + IntToStr(PosHeader2) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $68);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterOpenNDok;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $69,     .

procedure TE950Driver.PrintNDokStr(Str: WideString);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  DoBeforePrnTxtNDok;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($69) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        SendStr := SendStr + WinToDos(Str) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $69);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterPrnTxtNDok;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $6a,    .

procedure TE950Driver.CloseNDok;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  DoBeforeCloseNDok;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($6A) +
          Format('%-*s', [8, WinToDos(FPassword)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $6A);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
      DoAfterCloseNDok;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

{  .}

 // $70,    COM 1.
 // $71,   COM 1.
 // $72,     COM 1.
 // $73,     COM 1.
 // $74,    COM 2.
 // $75,   COM 2.
 // $76,     COM 2.
 // $77,     COM 2.

{  .}

 // $7b,    . ANo = 0
 // $7c,    . ANo = 1

function TE950Driver.GetCashBoxOpened(ANo: Integer): WordBool;
begin
  case ANo of
    0: begin
        Result := FLst.Values['CashBox1'] = 'true';
      end;
    1: begin
        Result := FLst.Values['CashBox2'] = 'true';
      end;
  else begin
      Result := False;
      DoOnError(SErrorInvalidCashBox);
    end;
  end;
end;

procedure TE950Driver.SetCashBoxOpened(ANo: Integer; AOpen: WordBool);
var
  SendStr,
    SendStr1: string;
  i: Integer;
  cmd: Byte;
begin
  FErrorStr := '';
  case ANo of
    0: begin
        cmd := $7B;
        FLst.Values['CashBox1'] := '';
      end;
    1: begin
        cmd := $7C;
        FLst.Values['CashBox2'] := '';
      end;
  else begin
      DoOnError(SErrorInvalidCashBox);
    end;
  end;

  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr(cmd) +
          Format('%-*s', [8, WinToDos(FPassword)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, cmd) then begin
            case ANo of
              0: begin
                  FLst.Values['CashBox1'] := 'true';
                end;
              1: begin
                  FLst.Values['CashBox2'] := 'true';
                end;
            end;
          end else begin
            case ANo of
              0: begin
                  FLst.Values['CashBox1'] := 'false';
                end;
              1: begin
                  FLst.Values['CashBox2'] := 'false';
                end;
            end;
          end;
        end else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

{   -----------------------------------------------------}
// $6e,  .

function TE950Driver.GetDefineDesc(NumTbl, NumDesc: Integer): WideString;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  FLst.Values['GetDefineDesc'] := '';
  Result := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($6E) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        IntToStr(NumTbl) + DByte + IntToStr(NumDesc) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $6E) then begin
            Delete(SendStr1, 1, 22);
            Result := Copy(SendStr1, 1, Pos(EByte, SendStr1) - 1);
            Result := DosToWin(Result);
            FLst.Values['GetDefineDesc'] := Result;
          end;
        end
        else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $6f,     .

procedure TE950Driver.SetDefineDesc(NumTbl, NumDesc: Integer; DefineDesc: WideString);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($6F) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        IntToStr(NumTbl) + DByte + IntToStr(NumDesc) + DByte +
          WinToDos(DefineDesc) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $6F);
        end else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;


function TE950Driver.GetTrainingState: WordBool;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  FLst.Values['TrainingState'] := '';
  Result := False;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($3E) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte + Chr($51) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $3E) then begin
            if SendStr1[23] = Chr($53) then begin
              Result := True;
              FLst.Values['TrainingState'] := 'true';
            end else begin
              Result := False;
              FLst.Values['TrainingState'] := 'false';
            end;
          end;
        end else Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

procedure TE950Driver.SetTrainingState(State: WordBool);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($3E) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        if State
          then SendStr := SendStr + Chr($53)
        else SendStr := SendStr + Chr($52);
        SendStr := SendStr + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $3E);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $5a,   .

procedure TE950Driver.SetCashireName(CashireName: WideString);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($5A) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte +
        WinToDos(CashireName) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15))
          then
        begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $5A);
        end
        else
          Inc(i);
      until i > 1;
      if i = 2
        then
        DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $5b,   .

function TE950Driver.GetCashireName: WideString;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  Result := '';
  FLst.Values['CashireName'] := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($5B) +
          Format('%-*s', [8, WinToDos(FPassword)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          if CheckAnswer(SendStr, SendStr1, $5B) then begin
            Delete(SendStr1, 1, 22);
            Result := DosToWin(Copy(SendStr1, 1, Pos(#$03, SendStr1) - 1));
            FLst.Values['CashireName'] := Result;
            i := 3;
          end;
        end
        else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $5D,   .

procedure TE950Driver.SetHeaderCheck(Index: Integer; Header: WideString);
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($5D) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte;
        SendStr := SendStr + IntToStr(Index) + DByte;
        SendStr := SendStr + Format('%-*s', [40, WinToDos(Header)]) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then
          DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          CheckAnswer(SendStr, SendStr1, $5D);
        end else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

// $5E,    .

function TE950Driver.GetHeaderCheck(Index: Integer): WideString;
var
  SendStr,
    SendStr1: string;
  i: Integer;
begin
  FErrorStr := '';
  Result := '';
  FLst.Values['HeaderCheck'] := '';

  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      i := 0;
      repeat
        SendStr := BByte + Chr(FEnumComm) + Chr($5E) +
          Format('%-*s', [8, WinToDos(FPassword)]) + DByte + IntToStr(Index) + EByte;
        AddCheckSum(SendStr);
        SendString(SendStr);
        SendStr1 := ReadAnswer();
        if SendStr1 = ''
          then DoOnError(SErrorNotAnswer);
        if (SendStr1[1] <> Chr($15)) then begin
          i := 3;
          if CheckAnswer(SendStr, SendStr1, $5E) then begin
            Delete(SendStr1, 1, 22);
            Delete(SendStr1, 1, Pos(DByte, SendStr1));
            Result := DosToWin(Copy(SendStr1, 1, Pos(EByte, SendStr1) - 1));
            FLst.Values['HeaderCheck'] := Result;
          end;
        end else Inc(i);
      until i > 1;
      if i = 2
        then DoOnError(SErrorNak);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

procedure TE950Driver.DoBeforeOpenCheck;
begin
  if Assigned(FBeforeOpenCheck) then FBeforeOpenCheck(Self);
end;

procedure TE950Driver.DoAfterOpenCheck;
begin
  if Assigned(FAfterOpenCheck) then FAfterOpenCheck(Self);
end;

procedure TE950Driver.DoBeforePrnTxtCheck;
begin
  if Assigned(FBeforePrnTxtCheck) then FBeforePrnTxtCheck(Self);
end;

procedure TE950Driver.DoAfterPrnTxtCheck;
begin
  if Assigned(FAfterPrnTxtCheck) then FAfterPrnTxtCheck(Self);
end;

procedure TE950Driver.DoBeforePrnStrCheck;
begin
  if Assigned(FBeforePrnStrCheck) then FBeforePrnStrCheck(Self);
end;

procedure TE950Driver.DoAfterPrnStrCheck;
begin
  if Assigned(FAfterPrnStrCheck) then FAfterPrnStrCheck(Self);
end;

procedure TE950Driver.DoBeforePrnSubTotalCheck;
begin
  if Assigned(FBeforePrnSubTotalCheck) then FBeforePrnSubTotalCheck(Self);
end;

procedure TE950Driver.DoAfterPrnSubTotalCheck;
begin
  if Assigned(FAfterPrnSubTotalCheck) then FAfterPrnSubTotalCheck(Self);
end;

procedure TE950Driver.DoBeforePrnTotalCheck;
begin
  if Assigned(FBeforePrnTotalCheck) then FBeforePrnTotalCheck(Self);
end;

procedure TE950Driver.DoAfterPrnTotalCheck;
begin
  if Assigned(FAfterPrnTotalCheck) then FAfterPrnTotalCheck(Self);
end;

procedure TE950Driver.DoBeforeCloseCheck;
begin
  if Assigned(FBeforeCloseCheck) then FBeforeCloseCheck(Self);
end;

procedure TE950Driver.DoAfterCloseCheck;
begin
  if Assigned(FAfterCloseCheck) then FAfterCloseCheck(Self);
end;

procedure TE950Driver.DoBeforeOpenNCheck;
begin
  if Assigned(FBeforeOpenNCheck) then FBeforeOpenNCheck(Self);
end;

procedure TE950Driver.DoAfterOpenNCheck;
begin
  if Assigned(FAfterOpenNCheck) then FAfterOpenNCheck(Self);
end;

procedure TE950Driver.DoBeforePrnTxtNCheck;
begin
  if Assigned(FBeforePrnTxtNCheck) then FBeforePrnTxtNCheck(Self);
end;

procedure TE950Driver.DoAfterPrnTxtNCheck;
begin
  if Assigned(FAfterPrnTxtNCheck) then FAfterPrnTxtNCheck(Self);
end;

procedure TE950Driver.DoBeforeCloseNCheck;
begin
  if Assigned(FBeforeCloseNCheck) then FBeforeCloseNCheck(Self);
end;

procedure TE950Driver.DoAfterCloseNCheck;
begin
  if Assigned(FAfterCloseNCheck) then FAfterCloseNCheck(Self);
end;

procedure TE950Driver.DoBeforeOpenDok;
begin
  if Assigned(FBeforeOpenDok) then FBeforeOpenDok(Self);
end;

procedure TE950Driver.DoAfterOpenDok;
begin
  if Assigned(FAfterOpenDok) then FAfterOpenDok(Self);
end;

procedure TE950Driver.DoBeforePrnTxtDok;
begin
  if Assigned(FBeforePrnTxtDok) then FBeforePrnTxtDok(Self);
end;

procedure TE950Driver.DoAfterPrnTxtDok;
begin
  if Assigned(FAfterPrnTxtDok) then FAfterPrnTxtDok(Self);
end;

procedure TE950Driver.DoBeforePrnStrDok;
begin
  if Assigned(FBeforePrnStrDok) then FBeforePrnStrDok(Self);
end;

procedure TE950Driver.DoAfterPrnStrDok;
begin
  if Assigned(FAfterPrnStrDok) then FAfterPrnStrDok(Self);
end;

procedure TE950Driver.DoBeforePrnSubTotalDok;
begin
  if Assigned(FBeforePrnSubTotalDok) then FBeforePrnSubTotalDok(Self);
end;

procedure TE950Driver.DoAfterPrnSubTotalDok;
begin
  if Assigned(FAfterPrnSubTotalDok) then FAfterPrnSubTotalDok(Self);
end;

procedure TE950Driver.DoBeforePrnTotalDok;
begin
  if Assigned(FBeforePrnTotalDok) then FBeforePrnTotalDok(Self);
end;

procedure TE950Driver.DoAfterPrnTotalDok;
begin
  if Assigned(FAfterPrnTotalDok) then FAfterPrnTotalDok(Self);
end;

procedure TE950Driver.DoBeforeCloseDok;
begin
  if Assigned(FBeforeCloseDok) then FBeforeCloseDok(Self);
end;

procedure TE950Driver.DoAfterCloseDok;
begin
  if Assigned(FAfterCloseDok) then FAfterCloseDok(Self);
end;

procedure TE950Driver.DoBeforeOpenNDok;
begin
  if Assigned(FBeforeOpenNDok) then FBeforeOpenNDok(Self);
end;

procedure TE950Driver.DoAfterOpenNDok;
begin
  if Assigned(FAfterOpenNDok) then FAfterOpenNDok(Self);
end;

procedure TE950Driver.DoBeforePrnTxtNDok;
begin
  if Assigned(FBeforePrnTxtNDok) then FBeforePrnTxtNDok(Self);
end;

procedure TE950Driver.DoAfterPrnTxtNDok;
begin
  if Assigned(FAfterPrnTxtNDok) then FAfterPrnTxtNDok(Self);
end;

procedure TE950Driver.DoBeforeCloseNDok;
begin
  if Assigned(FBeforeCloseNDok) then FBeforeCloseNDok(Self);
end;

procedure TE950Driver.DoAfterCloseNDok;
begin
  if Assigned(FAfterCloseNDok) then FAfterCloseNDok(Self);
end;

procedure TE950Driver.DoOnError(Error: string);
begin
  FErrorStr := Error;
  ELed.LEDOn := True;
  if Assigned(FOnError)
    then
  begin
    FOnError(Error);
    ELed.LEDOn := False;
    Abort;
  end
  else
  begin
    MessageDlg(Format('%s.', [Error]), mtError, [mbOk], 0);
    ELed.LEDOn := False;
    Abort;
  end;
end;

procedure TE950Driver.FLEDOnClick(Sender: TObject);
begin
  if Connected
    then Disconnect
  else Connect;
end;

procedure TE950Driver.SetLedOn(Index: Integer; Value: Boolean);
begin
  case Index of
    3: ELed.LEDOn := Value;
  end;
end;

function TE950Driver.GetLedOn(Index: Integer): Boolean;
begin
  Result := False;
  case Index of
    3: Result := ELed.LEDOn;
  end;
end;

// default- return values uses system currency&date format
// XXX0 - return values uses internal representation (if applicable)

function TE950Driver.GetLastResult(AParameter: WideString): WideString;
begin
  Result := FLst.Values[AParameter];
end;

end.

