//------------------------------------------------------------------------
// UNIT           : E950Drv20Id.pas
// CONTENTS       : TE950Driver component intterface
// VERSION        : 1.0
// TARGET         : (Inprise's) Borland Delphi 4.0
// AUTHORS         : Konstantin Grishenko, Andrey Ivanov, Dmitry Myarikianov, Marco Cocco
// STATUS         : Freeware
//------------------------------------------------------------------------
//
// Copyright (c) 1996-98 by Konstantin Grishenko, Andrey Ivanov, Dmitry Myarikianov. All rights reseved.
// Copyright (c) 1996-98 by Marco Cocco. All rights reseved.
// Copyright (c) 1996-98 by d3k Software Company. All rights reseved.
//
// NO REPRESENTATIONS ARE MADE ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
//    PURPOSE. IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
//   NEITHER MARCO COCCO OR D3K SHALL BE LIABLE FOR ANY DAMAGES SUFFERED BY
//                          THE USE OF THIS SOFTWARE.
//

unit E950Drv;

interface

uses
  // Delphi units
  Windows, Messages, SysUtils, Classes, Forms, Controls, E950Const;
  // ComDrv32 units


//------------------------------------------------------------------------
// Property types
//------------------------------------------------------------------------

type
  // Baud Rates (custom or 110...256k bauds)
  TBaudRate = (brCustom,
    br110, br300, br600, br1200, br2400, br4800,
    br9600, br14400, br19200, br38400, br56000,
    br57600, br115200, br128000, br256000);
  // Port Numbers ( custom or COM1..COM16 )
  TPortNumber = (pnCustom,
    pnCOM1, pnCOM2, pnCOM3, pnCOM4, pnCOM5, pnCOM6, pnCOM7,
    pnCOM8, pnCOM9, pnCOM10, pnCOM11, pnCOM12, pnCOM13,
    pnCOM14, pnCOM15, pnCOM16);
  // Data bits ( 5, 6, 7, 8 )
  TDataBits = (db5BITS, db6BITS, db7BITS, db8BITS);
  // Stop bits ( 1, 1.5, 2 )
  TStopBits = (sb1BITS, sb1HALFBITS, sb2BITS);
  // Parity ( None, odd, even, mark, space )
  TParity = (ptNONE, ptODD, ptEVEN, ptMARK, ptSPACE);
  // Hardware Flow Control ( None, None + RTS always on, RTS/CTS )
  THwFlowControl = (hfNONE, hfNONERTSON, hfRTSCTS);
  // Software Flow Control ( None, XON/XOFF )
  TSwFlowControl = (sfNONE, sfXONXOFF);
  // What to do with incomplete (incoming) packets ( Discard, Pass )
  TPacketMode = (pmDiscard, pmPass);

//------------------------------------------------------------------------
// Event types
//------------------------------------------------------------------------

type
  // RX event ( packet mode disabled )
  TReceiveDataEvent = procedure(Sender: TObject; DataPtr: pointer; DataSize: DWORD) of object;
  // RX event ( packed mode enabled )
  TReceivePacketEvent = procedure(Sender: TObject; Packet: pointer; DataSize: DWORD) of object;

//------------------------------------------------------------------------
// Other types
//------------------------------------------------------------------------

type
  // Line status ( Clear To Send, Data Set Ready, Ring, Carrier Detect )
  TLineStatus = (lsCTS, lsDSR, lsRING, lsCD);
  // Set of line status
  TLineStatusSet = set of TLineStatus;

//------------------------------------------------------------------------
// Constants
//------------------------------------------------------------------------

type
 //  
  TStatusRegistr = record
    LastFDok: Integer;
    FirstDokDate: TDate;
    FirstDokTime: TTime;
    LastClose: Integer;
    CountFiskals: Integer;
    VersionROM: string;
    Lastdate: TDate;
    SumMoney: Currency;
  end;
  PStatusRegistr = ^TStatusRegistr;

 //  
  TSertificate = record
    SN: string[16];
    PosPoint: Byte;
    Devaluations: Byte;
    CurrDef: Byte;
    TypePrn: Byte;
    PasswordFiskal: string[8];
    PasswordUser: string[8];
    DateSer: TDate;
  end;
  PSertificate = ^TSertificate;

 //
  TFiskal = record
    RN: string[16];
    UserFR: string[16];
    PasswordFiskal: string[8];
    PasswordUser: string[8];
    DateReg: TDate;
  end;
  PFiskal = ^TFiskal;

 // 
  TFMemSize = record
    AllMem: Word;
    UsesMem: Word;
  end;
  PFMemSize = ^TFMemSize;

  TSmenaClose = record
    NumClose: Word;
    CountFCheckComplect: Word;
    CountFCheckNotComplect: Word;
    CountFDokComplect: Word;
    CountFDokNotComplect: Word;
    CountFDokAnulComplect: Word;
    CountCheckDokService: Word;
    SumSale: Cardinal;
    SumMargin: Cardinal;
    SumDiscount: Cardinal;
    SumContainer: Cardinal;
    CountCheckDokBack: Word;
    SumSaleBack: Cardinal;
    SumMarginBack: Cardinal;
    SumDiscountBack: Cardinal;
    SumContainerBack: Cardinal;
    CountCheckDokPayment: Word;
    SumPension: Cardinal;
    SumTranslation: Cardinal;
    SumComission: Cardinal;
    CountCheckDokPaymentBack: Word;
    SumPensionBack: Cardinal;
    SumTranslationBack: Cardinal;
    SumComissionBack: Cardinal;
    CountCheckDokCashe: Word;
    SumCashe: Cardinal;
    CountCheckDokCasheBack: Word;
    SumCasheBack: Cardinal;
  end;
  PSmenaClose = ^TSmenaClose;

 //  
  TTypeCheck = (Service, ServiceBack, Payment, PaymentBack, Cashe, CasheBack);
  PTypeCheck = ^TTypeCheck;

 //  
  TFiskalStr = record
    IndxDiscStr: Word;
    DefineDiscStr: string[20];
    Count: Cardinal;
    IndxDiscUM: Word;
    Priece: Cardinal;
    IndxDiscCurr: Word;
    IndxStr: Char;
    ProcMargin: Word;
    ProcDiscount: Word;
  end;
  PFiskalStr = ^TFiskalStr;

 //
  TSubTotal = record
    IndxDsicSubTotal: Word;
    DefineSubTotal: string[15];
    IndDiscCurr: Word;
    ProcMargin: Single;
    ProcDiscount: Single;
  end;
  PSubTotal = ^TSubTotal;

  TSubTotalAnswer = record
    TypeCheck: TTypeCheck;
    CountFStr: Word;
    TotalMarginTransl: Currency;
    TotalDiscountComiss: Currency;
    TotalContainer: Currency;
    InCashe: Currency;
  end;
  PSubTotalAnswer = ^TSubTotalAnswer;

 //
  TFontPrn = (Bold, DblH, DblW, UndLine);

 //  
  TTypeDok = (ServiceDok, ServiceBackDok, PaymentDok, PaymentBackDok, CasheDok, CasheBackDok);
  PTypeDok = ^TTypeDok;

  TFDok = record
    TDok: TTypeDok;
    IndxDiscCurr: Word;
    CountFreeLine: Word;
    PosHeader1: Word;
    PosDisc: Word;
    LenDisc: Word;
    PosDefineDisc: Word;
    LenDefineDisc: Word;
    PosDiscStr: Word;
    LenDiscStr: Word;
    PosDefineStr: Word;
    LenDefgineDiscStr: Word;
    PosSum: Word;
    LenSum: Word;
    PosDiscTotal: Word;
    LenDiscTotal: Word;
    PosDefineTotal: Word;
    LenDefineToTal: Word;
    PosSumTotal: Word;
    LenSumTotal: Word;
    PosHeader2: Word;
  end;
  PFDok = ^TFDok;

 //
  TTypeStrFDok = (Sale, ASale); //($47,$67,$53,$73,$44,$64,$52,$72,50,$70,$57,$74);

 //  
  TFDokStr = record
    IndxDisc: Word;
    DefineDisc: string;
    Count: Single;
    IndxDiscEI: Word;
    Price: Currency;
    IndxDiscCurr: Word;
    TypeStr: TTypeStrFDok;
    ProcMargine: Single;
    ProcDiscount: Single;
  end;
  PFDokStr = ^TFDokStr;

 //  
  TSubTotalFDok = record
    PrintDok: Boolean;
    IndxDiscSubTotal: Word;
    DefineSubTotal: string;
    IndDiscCurr: Word;
    ProcMargine: Single;
    ProcDiscount: Single;
  end;
  PSubTotalFDok = ^TSubTotalFDok;

 // 
  TSubTotalDokAnswer = record
    TDok: TTypeDok;
    CountLine: Word;
    TotalSale: Currency;
    TotalMargine: Currency;
    TotalDiscount: Currency;
    TotalConteiner: Currency;
    InCashe: Currency;
  end;
  PSubTotalDokAnswer = ^TSubTotalDokAnswer;

 //  
  TPayment = (NotComplectDok, FullDok);
  TTotalFDok = record
    IndxDiscPayment: Word;
    DefinePayment: string;
    InCashe: Currency;
    IndxDiscCurr: Word;
    Payment: TPayment;
  end;
  PTotalFDok = ^TTotalFDok;


type

  TPrnStates = record
    psError,
      psOff,
      psLogLine,
      psCheckLine,
      psBuffFull,
      psBuffEmpty,
      psBeginDok,
      psEndDok,
      psDokSmall,
      psLineNo,
      psAllError: Boolean;
  end;
  PPrnStates = ^TPrnStates;

  TFiskalStates = record
    fsErrorFMem,
      fsErrorWMem,
      fsBattaryLow,
      fsUnknowCommand,
      fsErrorField,
      fsErrorCommand,
      fsOverTotal,
      fsFMemFull,
      fsFMemSoonFull,
      fsCertificated,
      fsFiskalited,
      fsErrorPassDate,
      fsDokCheckOpened,
      fsCheckOpened,
      fsDokOpened,
      fsAllError: Boolean;
  end;
  PFiskalStates = ^TFiskalStates;

  TStateRegistrator = record
    Printer: TPrnStates;
    Fiskal: TFiskalStates;
  end;


const
  BBit = Chr(2);
  EBit = Chr(3);
  DBit = Chr($1C);
const
  RELEASE_NOCLOSE_PORT = HFILE(INVALID_HANDLE_VALUE - 1);

//------------------------------------------------------------------------
// TCommPortDriver component
//------------------------------------------------------------------------

type
  TE950Driver = class(TComponent)
  private
    // Device Handle ( File Handle )
    FHandle: HFILE;
    // # of the COM port to use, or pnCustom to use custom port name
    FPort: TPortNumber;
    // Custom port name ( usually '\\.\COMn', with n = 1..x )
    FPortName: string;
    // COM Port speed (brXXX)
    FBaudRate: TBaudRate;
    // Baud rate ( actual numeric value )
    FBaudRateValue: DWORD;
    // Data bits size (dbXXX)
    FDataBits: TDataBits;
    // How many stop bits to use (sbXXX)
    FStopBits: TStopBits;
    // Type of parity to use (ptXXX)
    FParity: TParity;
    // Type of hw handshaking (hw flow control) to use (hfXXX)
    FHwFlow: THwFlowControl;
    // Type of sw handshaking (sw flow control) to use (sFXXX)
    FSwFlow: TSwFlowControl;
    // Size of the input buffer
    FInBufSize: DWORD;
    // Size of the output buffer
    FOutBufSize: DWORD;
    // Event to raise on data reception (asynchronous)
    FOnReceiveData: TReceiveDataEvent;
    // Event to raise on packet reception (asynchronous)
    FOnReceivePacket: TReceivePacketEvent;
    // ms of delay between COM port pollings
    FPollingDelay: word;
    // Specifies if the DTR line must be enabled/disabled on connect
    FEnableDTROnOpen: boolean;
    // Output timeout - milliseconds
    FOutputTimeout: word;
    // Timeout for ReadData
    FInputTimeout: DWORD;
    // Set to TRUE to prevent hangs when no device connected or
    // device is OFF
    FCkLineStatus: boolean;
    // This is used for the timer
    FNotifyWnd: HWND;
    // Temporary buffer (RX) - used internally
    FTempInBuffer: pointer;
    // Time of the first byte of current RX packet
//    FFirstByteOfPacketTime     : DWORD;
    // Number of RX polling timer pauses
    FRXPollingPauses: integer;
    FEnumComm: Byte;
    FPassword: string;
    FStateRegistrator: TStateRegistrator;
    FCritSect: TRTLCriticalSection;
    // Sets the COM port handle
    procedure SetHandle(Value: HFILE);
    // Selects the COM port to use
    procedure SetPort(Value: TPortNumber);
    // Sets the port name
    procedure SetPortName(Value: string);
    // Selects the baud rate
    procedure SetBaudRate(Value: TBaudRate);
    // Selects the baud rate ( actual baud rate value )
    procedure SetBaudRateValue(Value: DWORD);
    // Selects the number of data bits
    procedure SetDataBits(Value: TDataBits);
    // Selects the number of stop bits
    procedure SetStopBits(Value: TStopBits);
    // Selects the kind of parity
    procedure SetParity(Value: TParity);
    // Selects the kind of hardware flow control
    procedure SetHwFlowControl(Value: THwFlowControl);
    // Selects the kind of software flow control
    procedure SetSwFlowControl(Value: TSwFlowControl);
    // Sets the RX buffer size
    procedure SetInBufSize(Value: DWORD);
    // Sets the TX buffer size
    procedure SetOutBufSize(Value: DWORD);
    // Sets the delay between polling checks
    procedure SetPollingDelay(Value: word);
    // Applies current settings to open COM port
    function ApplyCOMSettings: boolean;
    // Polling proc
    procedure TimerWndProc(var msg: TMessage);
    // Sends binary data
    //function SendData( DataPtr: pointer; DataSize: DWORD ): DWORD;
    // Sends binary data. Returns number of bytes sent. Timeout overrides
    // the value specifiend in the OutputTimeout property
    function SendDataEx(DataPtr: pchar; DataSize, Timeout: DWORD): DWORD;
    // Sends a byte. Returns true if the byte has been sent
    // Reads binary data. Returns number of bytes read
    function ReadData(DataPtr: pchar; MaxDataSize: DWORD): DWORD;
        // Reads a byte. Returns true if the byte has been read
    function ReadByte(var Value: byte): boolean;
    procedure SetEnumComm;
    //  $3e,  . /.
    function GetTreningState: Boolean;
    procedure SetTreningState(State: Boolean);
    // $5a,   .
    procedure SetCashire(CashireName: string);
    // $5b,   .
    function GetCashire: string;
    // $5D,   .
    procedure SetHeader(Header: string);
    // $5E,    .
    function GetHeader: string;
    function ReadAnswer: string;
    function CheckAnswer(const AAnswer: string; const LastCommand: byte): Boolean;
   { ,    .}
    // $58,    .
    procedure SetDateTime(ADateTime: TDateTime);
    // $59,        .
    function GetDateTime: TDateTime;
  public
    // Constructor
    constructor Create(AOwner: TComponent); override;
    // Destructor
    destructor Destroy; override;
     // Opens the COM port and takes of it. Returns false if something
    // goes wrong.
    function Connect: boolean;
    // Closes the COM port and releases control of it
    procedure Disconnect;
    // Returns true if COM port has been opened
    function Connected: boolean;
    // Returns the current state of CTS, DSR, RING and RLSD (CD) lines.
    // The function fails if the hardware does not support the control-register
    // values (that is, returned set is always empty).
    function GetLineStatus: TLineStatusSet;
    // Returns true if polling has not been paused
    function IsPolling: boolean;
    // Pauses polling
    procedure PausePolling;
    // Re-starts polling (after pause)
    procedure ContinuePolling;
    // Flushes the rx/tx buffers
    function FlushBuffers(inBuf, outBuf: boolean): boolean;
    // Returns number of received bytes in the RX buffer
    function CountRX: integer;
    // Returns the output buffer free space or 65535 if not connected
    function OutFreeSpace: word;
    procedure ToggleDTR(onOff: boolean);
    // Set RTS line high (onOff=TRUE) or low (onOff=FALSE).
    // You must not use HW handshaking.
    procedure ToggleRTS(onOff: boolean);
        // Sends binary data
//    function SendData( DataPtr: pointer; DataSize: DWORD ): DWORD;
      {  }
    //    
    function TestFBlock(var CountBlock,
      CountSuccBlock, CountErrorBlock,
      CountFreeBlock: Integer): Boolean;
    //      
    function ReportTestFMem(var CountBlock,
      CountSuccBlock, CountErrorBlock,
      CountFreeBlock: Integer): Boolean;
    // $22,  .
    procedure ReportDiag;
    // $2c,      .
    function SetBlockWorkMem(const SetOf: Integer;
      const InfoBlock: string): Boolean;
    // $2d,      .
    function GetBlockWorkMem(const SetOf: Integer;
      const LenBlock: Byte; var InfoBlock: string): Boolean;
    // $2e,    .
    procedure PrintTestPrinter;
    // $2f,   .
    procedure ResetRegistrator;
    // $2a,    .
    function GetStatusRegistrator(Info: PStatusRegistr): Boolean;
    {    .}
     {  .}
    // $30,   .
    function GetDmpSertificate(SendToPrint: Boolean; SertificateDmp: PSertificate): Boolean;
    // $31,   .
    function LoadSertificate(Sertificate: PSertificate): Boolean;
    // $32, .
    procedure Certification;
  {  .}
    // $33,   .
    function GetDmpFiskal(SendToPrint: Boolean; FiskalDmp: PFiskal): Boolean;
    // $34,   .
    function LoadFiskal(FiskalDmp: PFiskal): Boolean;
    // $35, .
    procedure Fiskalization;
    // $37,   
    function GetFMemSize(FMemSize: PFMemSize): Boolean;
    // $39,  .
    function CloseSmena(Fix: Boolean; TotalSmena: PSmenaClose): Boolean;
    // $3a,    .
    procedure ReportOfDates(const BeginDate, EndDate: TDate; ItogOnly: Boolean);
    // $3b,      .
    procedure ReportOfSmena(const BeginSmena, EndSmena: TDate; ItogOnly: Boolean);
    //  $3f,   .
    function ChangePassword(const PasswordOld, PasswordNew: string): Boolean;
 {   .}
    // $40,   .
    function OpenCheck(TypeCheck: TTypeCheck; const IndxCurr: Word): Boolean;
    // $41,   .
    procedure PrintFiskalTxt(const IndxDisc: Word; const FiskalTxt: string);
    // $42,    .
    procedure PrintFiskalStr(FislakStr: PFiskalStr);
    // $43,   .
    function SubTotal(PrintSubTotal: Boolean;
      SubTotal: PSubTotal; SubTotalAnswer: PSubTotalAnswer): Boolean;
    // $44,   .
    function CheckTotal(const InSum: Cardinal; const IndxCurr: Word;
      IndxOpl: Byte; DefOpl: string; Tender: Boolean; var Sum: Cardinal): Boolean;
    // $45,   .
    function CloseCheck(var LastCheck: Cardinal): Boolean;
  {  .}
    // $48,   .
    function OpenNFCheck(const IndxNFDok: Word;
      DefineDiscNFDok: string): Boolean;
    // $49,   .
    procedure PrintNFText(const Str: string);
    // $4a,   .
    procedure CloseNFCheck;
    // $4b,   .
    procedure CutNFCheck;
  {  .}
    // $50,   .
    procedure FeedLineCheck(const CountLine: Word);
    // $51,   .
    function FeedLineLog(CountLine: Word): Boolean;
    // $52,     .
    function FeedLineBoth(CountLine: Word): Boolean;
    // $53,   .
    function FeedLineDok(CountLine: Word): Boolean;
    // $54,   .
    procedure EjectDok;
    // $55,  Slip- .
    function ConditionSLIP(
      CountFreeLine, Pos1, Pos2: Word; var NumDok: Cardinal): Boolean;
    // $57,   .
    function FontPrn(Font: TFontPrn): Boolean;
  {  Slip-.}
    // $60,    .
    function OpenFDok(TDok: PTypeDok): Boolean;
    // $61,    .
    function PrintDefineFDok(IndxDisc: Word;
      DefineFDok: string): Boolean;
    // $62,    .
    function PrintStrFDok(DokStr: PFDokStr): Boolean;
    // $63,    
    function SubTotalFDok(SubTotal: PSubTotalFDok;
      SubTotalAnswer: PSubTotalAnswer): Boolean;
    // $64,    .
    function TotalFDok(TotalFDok: PTotalFDok; var Sum: Currency): Boolean;
    // $65,    .
    function CloseFDok: Cardinal;
  {   .}
    // $68,    .
    procedure OpenNFDok;
    // $69,     .
    procedure PrintNFDokStr(const Str: string);
    // $6a,    .
    procedure CloseNFDok;
{  .}

 // $70,    COM 1.
 // $71,   COM 1.
 // $72,     COM 1.
 // $73,     COM 1.
 // $74,    COM 2.
 // $75,   COM 2.
 // $76,     COM 2.
 // $77,     COM 2.

{  .}

 // $7b,    .
 // $7c,    .

   { .}
    // $6e,  .
    function GetDefineDisc(const NumTbl, NumDisc: Word): string;
    // $6f,     .
    procedure SetDefineDisc(NumTbl, NumDisc: Word; const DefineDisc: string);
    // Make the Handle of the COM port public (for TAPI...) [read/write]
    property Handle: HFILE read FHandle write SetHandle;
    property TreiningState: Boolean read GetTreningState write SetTreningState;
    property CashireName: string read GetCashire write SetCashire;
    property HeaderCheck: string read GetHeader write SetHeader;
    property Password: string read FPassword write FPassword;
    property StateRegistrator: TStateRegistrator read FStateRegistrator;
    property DateTimeFR: TDateTime read GetDateTime write SetDateTime;
  published
    // # of the COM Port to use ( or pnCustom for port by name )
    property Port: TPortNumber read FPort write SetPort default pnCOM2;
    // Name of COM port
    property PortName: string read FPortName write SetPortName;
    // Speed ( Baud Rate )
    property BaudRate: TBaudRate read FBaudRate write SetBaudRate default br9600;
    // Speed ( Actual Baud Rate value )
    property BaudRateValue: DWORD read FBaudRateValue write SetBaudRateValue default 9600;
    // Data bits to use (5..8, for the 8250 the use of 5 data bits with 2 stop
    // bits is an invalid combination, as is 6, 7, or 8 data bits with 1.5 stop
    // bits)
    property DataBits: TDataBits read FDataBits write SetDataBits default db8BITS;
    // Stop bits to use (1, 1.5, 2)
    property StopBits: TStopBits read FStopBits write SetStopBits default sb1BITS;
    // Kind of Parity to use (none,odd,even,mark,space)
    property Parity: TParity read FParity write SetParity default ptNONE;
    // Kind of Hardware Flow Control to use:
    //   hfNONE          none
    //   hfNONERTSON     no flow control but keep RTS line on
    //   hfRTSCTS        Request-To-Send/Clear-To-Send
    property HwFlow: THwFlowControl read FHwFlow write SetHwFlowControl default hfNONERTSON;
    // Kind of Software Flow Control to use:
    //   sfNONE          none
    //   sfXONXOFF       XON/XOFF
    property SwFlow: TSwFlowControl read FSwFlow write SetSwFlowControl default sfNONE;
    // Input Buffer size ( suggested - driver might ignore this setting ! )
    property InBufSize: DWORD read FInBufSize write SetInBufSize default 2048;
    // Output Buffer size ( suggested - driver usually ignores this setting ! )
    property OutBufSize: DWORD read FOutBufSize write SetOutBufSize default 2048;
    // ms of delay between COM port pollings
    property PollingDelay: word read FPollingDelay write SetPollingDelay default 50;
    // Set to TRUE to enable DTR line on connect and to leave it on until disconnect.
    // Set to FALSE to disable DTR line on connect.
    property EnableDTROnOpen: boolean read FEnableDTROnOpen write FEnableDTROnOpen default true;
    // Output timeout (milliseconds)
    property OutputTimeout: word read FOutputTimeOut write FOutputTimeout default 500;
    // Input timeout (milliseconds)
    property InputTimeout: DWORD read FInputTimeOut write FInputTimeout default 200;
    // Set to TRUE to prevent hangs when no device connected or device is OFF
    property CheckLineStatus: boolean read FCkLineStatus write FCkLineStatus default false;
    // Event to raise when there is data available (input buffer has data)
    // (called only if PacketSize <= 0)
    property OnReceiveData: TReceiveDataEvent read FOnReceiveData write FOnReceiveData;
    // Event to raise when there is data packet available (called only if PacketSize > 0)
    property OnReceivePacket: TReceivePacketEvent read FOnReceivePacket write FOnReceivePacket;
  end;

function BaudRateOf(bRate: TBaudRate): DWORD;
function DelayForRX(bRate: TBaudRate; DataSize: DWORD): DWORD;

implementation

const
  Win32BaudRates: array[br110..br256000] of DWORD =
    (CBR_110, CBR_300, CBR_600, CBR_1200, CBR_2400, CBR_4800, CBR_9600,
    CBR_14400, CBR_19200, CBR_38400, CBR_56000, CBR_57600, CBR_115200,
    CBR_128000, CBR_256000);

const
  dcb_Binary = $00000001;
  dcb_ParityCheck = $00000002;
  dcb_OutxCtsFlow = $00000004;
  dcb_OutxDsrFlow = $00000008;
  dcb_DtrControlMask = $00000030;
  dcb_DtrControlDisable = $00000000;
  dcb_DtrControlEnable = $00000010;
  dcb_DtrControlHandshake = $00000020;
  dcb_DsrSensivity = $00000040;
  dcb_TXContinueOnXoff = $00000080;
  dcb_OutX = $00000100;
  dcb_InX = $00000200;
  dcb_ErrorChar = $00000400;
  dcb_NullStrip = $00000800;
  dcb_RtsControlMask = $00003000;
  dcb_RtsControlDisable = $00000000;
  dcb_RtsControlEnable = $00001000;
  dcb_RtsControlHandshake = $00002000;
  dcb_RtsControlToggle = $00003000;
  dcb_AbortOnError = $00004000;
  dcb_Reserveds = $FFFF8000;

function GetWinPlatform: string;
var ov: TOSVERSIONINFO;
begin
  ov.dwOSVersionInfoSize := sizeof(ov);
  if GetVersionEx(ov) then
  begin
    case ov.dwPlatformId of
      VER_PLATFORM_WIN32s: // Win32s on Windows 3.1
        Result := 'W32S';
      VER_PLATFORM_WIN32_WINDOWS: // Win32 on Windows 95/98
        Result := 'W95';
      VER_PLATFORM_WIN32_NT: //	Windows NT
        Result := 'WNT';
    end;
  end
  else
    Result := '??';
end;

function GetWinVersion: DWORD;
var ov: TOSVERSIONINFO;
begin
  ov.dwOSVersionInfoSize := sizeof(ov);
  if GetVersionEx(ov) then
    Result := MAKELONG(ov.dwMinorVersion, ov.dwMajorVersion)
  else
    Result := $00000000;
end;

function BaudRateOf(bRate: TBaudRate): DWORD;
begin
  if bRate = brCustom then
    Result := 0
  else
    Result := Win32BaudRates[bRate];
end;

function DelayForRX(bRate: TBaudRate; DataSize: DWORD): DWORD;
begin
  Result := round(DataSize / (BaudRateOf(bRate) / 10) * 1000);
end;

function AddCheckSum(const AQueryStr: string): string;
var
  i, AQuerySum: Integer;
begin
  AQuerySum := 0;
  for i := 1 to Length(AQueryStr) do
    AQuerySum := AQuerySum + Ord(AQueryStr[i]);
  Result := AQueryStr + IntToHex(AQuerySum, 4);
end;

function WinToDos(const WinStr: string): string;
begin
  SetLength(Result, Length(WinStr));
  if Length(Result) > 0 then
    CharToOemBuff(PChar(WinStr), PChar(Result), Length(Result));
end;

function DosToWin(const DosStr: string): string;
begin
  SetLength(Result, Length(DosStr));
  if Length(Result) > 0 then
    OemToCharBuff(PChar(DosStr), PChar(Result), Length(Result));
end;


constructor TE950Driver.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  InitializeCriticalSection(FCritSect);
  // Initialize to default values -----------------------
  // Not connected
  FHandle := INVALID_HANDLE_VALUE;
  // COM 2
  FPort := pnCOM2;
  FPortName := '\\.\COM2';
  // 9600 bauds
  FBaudRate := br9600;
  FBaudRateValue := BaudRateOf(br9600);
  // 8 data bits
  FDataBits := db8BITS;
  // 1 stop bit
  FStopBits := sb1BITS;
  // no parity
  FParity := ptNONE;
  // No hardware flow control but RTS on
  FHwFlow := hfNONERTSON;
  // No software flow control
  FSwFlow := sfNONE;
  // Input buffer of 2048 bytes
  FInBufSize := 2048;
  // Output buffer of 2048 bytes
  FOutBufSize := 2048;
  // Poll COM port every 50ms
  FPollingDelay := 50;
  // Output timeout of 500ms
  FOutputTimeout := 500;
  // Timeout for ReadData(), 500ms
  FInputTimeout := 500;
  // DTR high on connect
  FEnableDTROnOpen := true;
  // Time not valid ( used by the packing routines )
  //FFirstByteOfPacketTime     := DWORD(-1);
  // Don't check of off-line devices
  FCkLineStatus := false;
  // Init number of RX polling timer pauses - not paused
  FRXPollingPauses := 0;
  // Temporary buffer for received data
  FTempInBuffer := AllocMem(FInBufSize);
  // Allocate a window handle to catch timer's notification messages
  if not (csDesigning in ComponentState) then
    FNotifyWnd := AllocateHWnd(TimerWndProc);
end;

destructor TE950Driver.Destroy;
begin
  // Be sure to release the COM port
  Disconnect;
  // Free the temporary buffer
  FreeMem(FTempInBuffer, FInBufSize);
  // Destroy the timer's window
  if not (csDesigning in ComponentState) then
    DeallocateHWnd(FNotifyWnd);
  DeleteCriticalSection(FCritSect);
  // Call inherited destructor
  inherited Destroy;
end;

// The COM port handle made public and writeable.
// This lets you connect to external opened com port.
// Setting ComPortHandle to INVALID_PORT_HANDLE acts as Disconnect.

procedure TE950Driver.SetHandle(Value: HFILE);
begin
  // If same COM port then do nothing
  if FHandle = Value then
    exit;
  // If value is RELEASE_NOCLOSE_PORT then stop controlling the COM port
  // without closing in
  if Value = RELEASE_NOCLOSE_PORT then
  begin
    // Stop the timer
    if Connected then
      KillTimer(FNotifyWnd, 1);
    // No more connected
    FHandle := INVALID_HANDLE_VALUE;
  end
  else
  begin
    // Disconnect
    Disconnect;
    // If Value is INVALID_HANDLE_VALUE then exit now
    if Value = INVALID_HANDLE_VALUE then
      exit;
    // Set COM port handle
    FHandle := Value;
    // Start the timer ( used for polling )
    SetTimer(FNotifyWnd, 1, FPollingDelay, nil);
  end;
end;

// Selects the COM port to use

procedure TE950Driver.SetPort(Value: TPortNumber);
begin
  // Be sure we are not using any COM port
  if Connected then
    exit;
  // Change COM port
  FPort := Value;
  // Update the port name
  if FPort <> pnCustom then
    FPortName := Format('\\.\COM%d', [ord(FPort)]);
end;

// Sets the port name

procedure TE950Driver.SetPortName(Value: string);
begin
  // Be sure we are not using any COM port
  if Connected then
    exit;
  // Change COM port
  FPort := pnCustom;
  // Update the port name
  FPortName := Value;
end;

// Selects the baud rate

procedure TE950Driver.SetBaudRate(Value: TBaudRate);
begin
  // Set new COM speed
  FBaudRate := Value;
  if FBaudRate <> brCustom then
    FBaudRateValue := BaudRateOf(FBaudRate);
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Selects the baud rate ( actual baud rate value )

procedure TE950Driver.SetBaudRateValue(Value: DWORD);
begin
  // Set new COM speed
  FBaudRate := brCustom;
  FBaudRateValue := Value;
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Selects the number of data bits

procedure TE950Driver.SetDataBits(Value: TDataBits);
begin
  // Set new data bits
  FDataBits := Value;
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Selects the number of stop bits

procedure TE950Driver.SetStopBits(Value: TStopBits);
begin
  // Set new stop bits
  FStopBits := Value;
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Selects the kind of parity

procedure TE950Driver.SetParity(Value: TParity);
begin
  // Set new parity
  FParity := Value;
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Selects the kind of hardware flow control

procedure TE950Driver.SetHwFlowControl(Value: THwFlowControl);
begin
  // Set new hardware flow control
  FHwFlow := Value;
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Selects the kind of software flow control

procedure TE950Driver.SetSwFlowControl(Value: TSwFlowControl);
begin
  // Set new software flow control
  FSwFlow := Value;
  // Apply changes
  if Connected then
    ApplyCOMSettings;
end;

// Sets the RX buffer size

procedure TE950Driver.SetInBufSize(Value: DWORD);
begin
  // Do nothing if connected
  if Connected then
    exit;
  // Free the temporary input buffer
  FreeMem(FTempInBuffer, FInBufSize);
  // Set new input buffer size
  if Value > 8192 then
    Value := 8192
  else if Value < 128 then
    Value := 128;
  FInBufSize := Value;
  // Allocate the temporary input buffer
  FTempInBuffer := AllocMem(FInBufSize);
  // Adjust the RX packet size
end;

// Sets the TX buffer size

procedure TE950Driver.SetOutBufSize(Value: DWORD);
begin
  // Do nothing if connected
  if Connected then
    exit;
  // Set new output buffer size
  if Value > 8192 then
    Value := 8192
  else if Value < 128 then
    Value := 128;
  FOutBufSize := Value;
end;

// Sets the size of incoming packets

// Sets the delay between polling checks

procedure TE950Driver.SetPollingDelay(Value: word);
begin
  // Make it greater than 4 ms
  if Value < 5 then
    Value := 5;
  // If new delay is not equal to previous value...
  if Value <> FPollingDelay then
  begin
    // Stop the timer
    if Connected then
      KillTimer(FNotifyWnd, 1);
    // Store new delay value
    FPollingDelay := Value;
    // Restart the timer
    if Connected then
      SetTimer(FNotifyWnd, 1, FPollingDelay, nil);
  end;
end;

// Apply COM settings

function TE950Driver.ApplyCOMSettings: boolean;
var dcb: TDCB;
begin
  // Do nothing if not connected
  Result := false;
  if not Connected then
    exit;

  // ** Setup DCB (Device Control Block) fields ******************************

  // Clear all
  fillchar(dcb, sizeof(dcb), 0);
  // DCB structure size
  dcb.DCBLength := sizeof(dcb);
  // Baud rate
  dcb.BaudRate := FBaudRateValue;
  // Set fBinary: Win32 does not support non binary mode transfers
  // (also disable EOF check)
  dcb.Flags := dcb_Binary;
  // Enables the DTR line when the device is opened and leaves it on
  if EnableDTROnOpen then
    dcb.Flags := dcb.Flags or dcb_DtrControlEnable;
  // Kind of hw flow control to use
  case FHwFlow of
    // No hw flow control
    hfNONE: ;
    // No hw flow control but set RTS high and leave it high
    hfNONERTSON:
      dcb.Flags := dcb.Flags or dcb_RtsControlEnable;
    // RTS/CTS (request-to-send/clear-to-send) flow control
    hfRTSCTS:
      dcb.Flags := dcb.Flags or dcb_OutxCtsFlow or dcb_RtsControlHandshake;
  end;
  // Kind of sw flow control to use
  case FSwFlow of
    // No sw flow control
    sfNONE: ;
    // XON/XOFF sw flow control
    sfXONXOFF:
      dcb.Flags := dcb.Flags or dcb_OutX or dcb_InX;
  end;
  // Set XONLim: specifies the minimum number of bytes allowed in the input
  // buffer before the XON character is sent (or CTS is set).
  if (GetWinPlatform = 'WNT') and (GetWinVersion >= $00040000) then
  begin
    // WinNT 4.0 + Service Pack 3 needs XONLim to be less than or
    // equal to 4096 bytes. Win95/98 doesn't have such limit.
    if FInBufSize div 4 > 4096 then
      dcb.XONLim := 4096
    else
      dcb.XONLim := FInBufSize div 4;
  end
  else
    dcb.XONLim := FInBufSize div 4;
  // Specifies the maximum number of bytes allowed in the input buffer before
  // the XOFF character is sent (or CTS is set low). The maximum number of bytes
  // allowed is calculated by subtracting this value from the size, in bytes, of
  // the input buffer.
  dcb.XOFFLim := dcb.XONLim;
  // How many data bits to use
  dcb.ByteSize := 5 + ord(FDataBits);
  // Kind of parity to use
  dcb.Parity := ord(FParity);
  // How many stop bits to use
  dcb.StopBits := ord(FStopbits);
  // XON ASCII char - DC1, Ctrl-Q, ASCII 17
  dcb.XONChar := #17;
  // XOFF ASCII char - DC3, Ctrl-S, ASCII 19
  dcb.XOFFChar := #19;

  // Apply new settings
  Result := SetCommState(FHandle, dcb);
  if not Result then
    exit;
  // Flush buffers
  Result := FlushBuffers(true, true);
  if not Result then
    exit;
  // Setup buffers size
  Result := SetupComm(FHandle, FInBufSize, FOutBufSize);
end;

function TE950Driver.Connect: boolean;
var tms: TCOMMTIMEOUTS;
begin
  // Do nothing if already connected
  Result := Connected;
  if Result then
    exit;
  // Open the COM port
  FHandle := CreateFile(pchar(FPortName),
    GENERIC_READ or GENERIC_WRITE,
    0, // Not shared
    nil, // No security attributes
    OPEN_EXISTING,
    FILE_ATTRIBUTE_NORMAL or FILE_FLAG_OVERLAPPED,
    0 // No template
    );
  Result := Connected;
  if not Result then
    raise Exception.CreateFmt('   : %s', [SysErrorMessage(GetLastError)]);

  // Apply settings
  Result := ApplyCOMSettings;
  if not Result then
  begin
    Disconnect;
    //exit;
    raise Exception.CreateFmt('   : %s', [SysErrorMessage(GetLastError)]);
  end;
  // Set ReadIntervalTimeout: Specifies the maximum time, in milliseconds,
  // allowed to elapse between the arrival of two characters on the
  // communications line.
  // We disable timeouts because we are polling the com port!
  tms.ReadIntervalTimeout := 1;
  // Set ReadTotalTimeoutMultiplier: Specifies the multiplier, in milliseconds,
  // used to calculate the total time-out period for read operations.
  tms.ReadTotalTimeoutMultiplier := 0;
  // Set ReadTotalTimeoutConstant: Specifies the constant, in milliseconds,
  // used to calculate the total time-out period for read operations.
  tms.ReadTotalTimeoutConstant := 1;
  // Set WriteTotalTimeoutMultiplier: Specifies the multiplier, in milliseconds,
  // used to calculate the total time-out period for write operations.
  tms.WriteTotalTimeoutMultiplier := 0;
  // Set WriteTotalTimeoutConstant: Specifies the constant, in milliseconds,
  // used to calculate the total time-out period for write operations.
  tms.WriteTotalTimeoutConstant := 1;
  // Apply timeouts
  SetCommTimeOuts(FHandle, tms);
  // Start the timer (used for polling)
  SetTimer(FNotifyWnd, 1, FPollingDelay, nil);
end;

procedure TE950Driver.Disconnect;
begin
  if Connected then
  begin
    // Stop the timer (used for polling)
    KillTimer(FNotifyWnd, 1);
    // Release the COM port
    CloseHandle(FHandle);
    // No more connected
    FHandle := INVALID_HANDLE_VALUE;
  end;
end;

// Returns true if connected

function TE950Driver.Connected: boolean;
begin
  Result := FHandle <> INVALID_HANDLE_VALUE;
end;

// Returns CTS, DSR, RING and RLSD (CD) signals status

function TE950Driver.GetLineStatus: TLineStatusSet;
var dwS: DWORD;
begin
  Result := [];
  if not Connected then
    exit;
  // Retrieves modem control-register values.
  // The function fails if the hardware does not support the control-register
  // values.
  if not GetCommModemStatus(FHandle, dwS) then
    exit;
  if dwS and MS_CTS_ON <> 0 then Result := Result + [lsCTS];
  if dwS and MS_DSR_ON <> 0 then Result := Result + [lsDSR];
  if dwS and MS_RING_ON <> 0 then Result := Result + [lsRING];
  if dwS and MS_RLSD_ON <> 0 then Result := Result + [lsCD];
end;

// Returns true if polling has not been paused

function TE950Driver.IsPolling: boolean;
begin
  Result := FRXPollingPauses <= 0;
end;

// Pauses polling

procedure TE950Driver.PausePolling;
begin
  // Inc. RX polling pauses counter
  inc(FRXPollingPauses);
end;

// Re-starts polling (after pause)

procedure TE950Driver.ContinuePolling;
begin
  // Dec. RX polling pauses counter
  dec(FRXPollingPauses);
end;

// Flush rx/tx buffers

function TE950Driver.FlushBuffers(inBuf, outBuf: boolean): boolean;
var dwAction: DWORD;
begin
  // Do nothing if not connected
  Result := false;
  if not Connected then
    exit;
  // Flush the RX data buffer
  dwAction := 0;
  if outBuf then
    dwAction := dwAction or PURGE_TXABORT or PURGE_TXCLEAR;
  // Flush the TX data buffer
  if inBuf then
    dwAction := dwAction or PURGE_RXABORT or PURGE_RXCLEAR;
  Result := PurgeComm(FHandle, dwAction);
  // Used by the RX packet mechanism
//  if Result then
//    FFirstByteOfPacketTime := DWORD(-1);
end;

// Returns number of received bytes in the RX buffer

function TE950Driver.CountRX: integer;
var stat: TCOMSTAT;
  errs: DWORD;
begin
  // Do nothing if port has not been opened
  Result := 65535;
  if not Connected then
    exit;
  // Get count
  ClearCommError(FHandle, errs, @stat);
  Result := stat.cbInQue;
end;

// Returns the output buffer free space or 65535 if not connected

function TE950Driver.OutFreeSpace: word;
var stat: TCOMSTAT;
  errs: DWORD;
begin
  if not Connected then
    Result := 65535
  else
  begin
    ClearCommError(FHandle, errs, @stat);
    Result := FOutBufSize - stat.cbOutQue;
  end;
end;

// Sends binary data. Returns number of bytes sent. Timeout overrides
// the value specifiend in the OutputTimeout property

function TE950Driver.SendDataEx(DataPtr: pchar; DataSize, Timeout: DWORD): DWORD;
var nToSend, nSent, t1: DWORD;
begin
  // Do nothing if port has not been opened
  Result := 0;
  if not Connected then
    exit;
  // Current time
  t1 := GetTickCount;
  // Loop until all data sent or timeout occurred
  while DataSize > 0 do
  begin
    // Get TX buffer free space
    nToSend := OutFreeSpace;
    // If output buffer has some free space...
    if nToSend > 0 then
    begin
      // Check signals
      if FCkLineStatus and (GetLineStatus = []) then
        exit;
      // Don't send more bytes than we actually have to send
      if nToSend > DataSize then
        nToSend := DataSize;
      // Send
      WriteFile(FHandle, DataPtr^, nToSend, nSent, nil);
      nSent := abs(nSent);
      if nSent > 0 then
      begin
        // Update number of bytes sent
        Result := Result + nSent;
        // Decrease the count of bytes to send
        DataSize := DataSize - nSent;
        // Inc. data pointer
        DataPtr := DataPtr + nSent;
        // Get current time
        t1 := GetTickCount;
        // Continue. This skips the time check below (don't stop
        // trasmitting if the Timeout is set too low)
        continue;
      end;
    end;
    // Buffer is full. If we are waiting too long then exit
    if DWORD(GetTickCount - t1) > Timeout then
      exit;
  end;
end;

// Send data (breaks the data in small packets if it doesn't fit in the output
// buffer)
//function TE950Driver.SendData( DataPtr: pointer; DataSize: DWORD ): DWORD;
//begin
//  Result := SendDataEx( DataPtr, DataSize, FOutputTimeout );
//end;

// Sends a byte. Returns true if the byte has been sent

// Reads binary data. Returns number of bytes read

function TE950Driver.ReadData(DataPtr: pchar; MaxDataSize: DWORD): DWORD;
var nToRead, nRead, t1: DWORD;
begin
  // Do nothing if port has not been opened
  Result := 0;
  if not Connected then
    exit;
  // Pause polling
  PausePolling;
  // Current time
  t1 := GetTickCount;
  // Loop until all requested data read or timeout occurred
  while MaxDataSize > 0 do
  begin
    // Get data bytes count in RX buffer
    nToRead := CountRX;
    // If input buffer has some data...
    if nToRead > 0 then
    begin
      // Don't read more bytes than we actually have to read
      if nToRead > MaxDataSize then
        nToRead := MaxDataSize;
      // Read
      ReadFile(FHandle, DataPtr^, nToRead, nRead, nil);
      // Update number of bytes read
      Result := Result + nRead;
      // Decrease the count of bytes to read
      MaxDataSize := MaxDataSize - nRead;
      // Inc. data pointer
      DataPtr := DataPtr + nRead;
      // Get current time
      t1 := GetTickCount;
      // Continue. This skips the time check below (don't stop
      // reading if the FInputTimeout is set too low)
      continue;
    end;
    // Buffer is empty. If we are waiting too long then exit
    if (GetTickCount - t1) > FInputTimeout then
      break;
  end;
  // Continue polling
  ContinuePolling;
end;

// Reads a byte. Returns true if the byte has been read

function TE950Driver.ReadByte(var Value: byte): boolean;
begin
  Result := ReadData(@Value, 1) = 1;
end;

// Set DTR line high (onOff=TRUE) or low (onOff=FALSE).
// You must not use HW handshaking.

procedure TE950Driver.ToggleDTR(onOff: boolean);
const funcs: array[boolean] of integer = (CLRDTR, SETDTR);
begin
  if Connected then
    EscapeCommFunction(FHandle, funcs[onOff]);
end;

// Set RTS line high (onOff=TRUE) or low (onOff=FALSE).
// You must not use HW handshaking.

procedure TE950Driver.ToggleRTS(onOff: boolean);
const funcs: array[boolean] of integer = (CLRRTS, SETRTS);
begin
  if Connected then
    EscapeCommFunction(FHandle, funcs[onOff]);
end;

// COM port polling proc

procedure TE950Driver.TimerWndProc(var msg: TMessage);
var nRead, nToRead, dummy: DWORD;
  comStat: TCOMSTAT;
begin
  if (msg.Msg = WM_TIMER) and Connected then
  begin
    // Do nothing if RX polling has been paused
    if FRXPollingPauses > 0 then
      exit;
    // If PacketSize is > 0 then raise the OnReceiveData event only if the RX
    // buffer has at least PacketSize bytes in it.
    ClearCommError(FHandle, dummy, @comStat);
    // Standard data handling
    nRead := 0;
    nToRead := comStat.cbInQue;
    if (nToRead > 0) and ReadFile(FHandle, FTempInBuffer^, nToRead, nRead, nil) then
      if (nRead <> 0) and Assigned(FOnReceiveData) then
        FOnReceiveData(Self, FTempInBuffer, nRead);
  end
  // Let Windows handle other messages
  else
    Msg.Result := DefWindowProc(FNotifyWnd, Msg.Msg, Msg.wParam, Msg.lParam);
end;


function TE950Driver.CheckAnswer(const AAnswer: string; const LastCommand: byte): Boolean;
  function CheckSumAnswer(const Str: string): Boolean;
  var
    i, AnswerSum: Integer;
    TestSum: Integer;
  begin
    AnswerSum := 0;
    for i := 1 to Length(AAnswer) - 5 do
      AnswerSum := AnswerSum + Ord(AAnswer[i]);
    TestSum := StrToIntDef('$' + Copy(AAnswer, (Length(AAnswer) - 3), 4), 0);
    Result := TestSum = AnswerSum;
  end;

  function CheckPassword(const Str: string): Boolean;
  begin
    Result := DosToWin(Str) = Format('%-*s', [8, FPassword]);
  end;

  function CheckEnumComm(const Str: Char): Boolean;
  begin
    Result := Ord(Str) = FEnumComm;
  end;

  function CheckCommand(const Str: Char; const LCommand: Byte): Boolean;
  begin
    Result := Ord(Str) = LCommand;
  end;

  procedure SetFState(const AFState: string);
  var
    iState: Word;
  begin
    iState := StrToIntDef('$' + AFState, -1);
    with FStateRegistrator.Fiskal do
    begin
      fsErrorFMem := iState and 1 > 0;
      fsErrorWMem := iState and 2 > 0;
      fsBattaryLow := iState and 4 > 0;
      fsUnknowCommand := iState and 8 > 0;
      fsErrorField := iState and 16 > 0;
      fsErrorCommand := iState and 32 > 0;
      fsOverTotal := iState and 64 > 0;
      fsFMemFull := iState and 128 > 0;
      fsFMemSoonFull := iState and 256 > 0;
      fsCertificated := iState and 512 > 0;
      fsFiskalited := iState and 1024 > 0;
      fsErrorPassDate := iState and 2048 > 0;
      fsDokCheckOpened := iState and 4096 > 0;
      fsCheckOpened := iState and 8192 > 0;
      fsDokOpened := iState and 16384 > 0;
      fsAllError := iState and 32768 > 0;
    end;
  end;

  procedure SetPState(const APState: string);
  var
    iState: Word;
  begin
    iState := StrToIntDef('$' + APState, -1);
    with FStateRegistrator.Printer do
    begin
      psError := iState and 4 > 0;
      psOff := iState and 8 > 0;
      psLogLine := iState and 16 > 0;
      psCheckLine := iState and 32 > 0;
      psBuffFull := iState and 64 > 0;
      psBuffEmpty := iState and 128 > 0;
      psBeginDok := iState and 512 > 0;
      psEndDok := iState and 2048 > 0;
      psDokSmall := iState and 8192 > 0;
      psLineNo := iState and 16384 > 0;
      psAllError := iState and 32768 > 0;
    end;
  end;
var
  ErrStr: string;
begin
  Result := False;
  if CheckSumAnswer(AAnswer)
    then
    Result := True
  else
    ErrStr := '  ';
  if CheckPassword(Copy(AAnswer, 4, 8))
    then
    Result := Result = True
  else
    if ErrStr = ''
      then
      ErrStr := ' '
    else
      ErrStr := ErrStr + #10#13 + ' ';

  if CheckEnumComm(AAnswer[2])
    then
    Result := Result = True
  else
    if ErrStr = ''
      then
      ErrStr := '   '
    else
      ErrStr := ErrStr + #10#13 + '   ';
  if CheckCommand(AAnswer[3], LastCommand)
    then
    Result := Result = True
  else
    if ErrStr = ''
      then
      ErrStr := '    '
    else
      ErrStr := ErrStr + #10#13 + '    ';

  SetPState(Copy(AAnswer, 13, 4));
  if FStateRegistrator.Fiskal.fsAllError
    then
  begin
    if FStateRegistrator.Fiskal.fsErrorFMem
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFSumFMem
      else
        ErrStr := ErrStr + #10#13 + SErrorFSumFMem;

    if FStateRegistrator.Fiskal.fsErrorWMem
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFSumWorkMeme
      else
        ErrStr := ErrStr + #10#13 + SErrorFSumWorkMeme;

    if FStateRegistrator.Fiskal.fsBattaryLow
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFLowBat
      else
        ErrStr := ErrStr + #10#13 + SErrorFLowBat;

    if FStateRegistrator.Fiskal.fsUnknowCommand
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFUnknown
      else
        ErrStr := ErrStr + #10#13 + SErrorFUnknown;

    if FStateRegistrator.Fiskal.fsErrorField
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFField
      else
        ErrStr := ErrStr + #10#13 + SErrorFField;

    if FStateRegistrator.Fiskal.fsErrorCommand
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFStat
      else
        ErrStr := ErrStr + #10#13 + SErrorFStat;

    if FStateRegistrator.Fiskal.fsOverTotal
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFOver
      else
        ErrStr := ErrStr + #10#13 + SErrorFOver;

    if FStateRegistrator.Fiskal.fsFMemFull
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFFullMem
      else
        ErrStr := ErrStr + #10#13 + SErrorFFullMem;

    if FStateRegistrator.Fiskal.fsFMemSoonFull
      then
      if ErrStr = ''
        then
        ErrStr := SErrorFFullMem1
      else
        ErrStr := ErrStr + #10#13 + SErrorFSumFMem;

  end;
  SetFState(Copy(AAnswer, 18, 4));
  if FStateRegistrator.Printer.psAllError
    then
  begin
    if FStateRegistrator.Printer.psError
      then
      if ErrStr = ''
        then
        ErrStr := SErrorP
      else
        ErrStr := ErrStr + #10#13 + SErrorP;
    if FStateRegistrator.Printer.psOff
      then
      if ErrStr = ''
        then
        ErrStr := SErrorPOff
      else
        ErrStr := ErrStr + #10#13 + SErrorPOff;

    if FStateRegistrator.Printer.psLogLine
      then
      if ErrStr = ''
        then
        ErrStr := SErrorPEndGL
      else
        ErrStr := ErrStr + #10#13 + SErrorPEndGL;

    if FStateRegistrator.Printer.psCheckLine
      then
      if ErrStr = ''
        then
        ErrStr := SErrorPEndCL
      else
        ErrStr := ErrStr + #10#13 + SErrorPEndCL;

    if FStateRegistrator.Printer.psDokSmall
      then
      if ErrStr = ''
        then
        ErrStr := SErrorPDokSmall
      else
        ErrStr := ErrStr + #10#13 + SErrorPDokSmall;

    if FStateRegistrator.Printer.psLineNo
      then
      if ErrStr = ''
        then
        ErrStr := SErrorPNoCG
      else
        ErrStr := ErrStr + #10#13 + SErrorPNoCG;
  end;

  if ErrStr <> ''
    then
    raise Exception.Create(ErrStr);
end;

//    

procedure TE950Driver.SetEnumComm;
begin
  if (FEnumComm < $7F) and (FEnumComm > $19)
    then Inc(FEnumComm)
  else FEnumComm := $20;
end;

function TE950Driver.ReadAnswer: string;
var
  InByte: Byte;
  blnR: Boolean;
begin
  InByte := 0;
  Result := Chr($15);
  repeat
    blnR := ReadByte(InByte);
  until (InByte in [$2, $12, $14, $15]) or (not blnR);
  if (InByte <> $15) and BlnR
    then
  begin
    if InByte = $02
      then
      Result := Chr(InByte)
    else
      Result := '';
    repeat
      ReadByte(InByte);
      if not (InByte in [$12, $14, $15])
        then
        Result := Result + Chr(InByte);
    until InByte = $03;
    Result := Result + Chr(InByte);
    ReadByte(InByte);
    Result := Result + Chr(InByte);
    ReadByte(InByte);
    Result := Result + Chr(InByte);
    ReadByte(InByte);
    Result := Result + Chr(InByte);
    ReadByte(InByte);
    Result := Result + Chr(InByte);
  end
  else if InByte = $0 then Result := '';
end;
  {  }

 //$20    

function TE950Driver.TestFBlock(var CountBlock,
  CountSuccBlock, CountErrorBlock,
  CountFreeBlock: Integer): Boolean;
var
  SendStr: string;
begin
  Result := False;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($20) +
        Format('%-*s', [8, WinToDos(FPassword)]) + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        if CheckAnswer(SendStr, $20)
          then
        begin
          CountBlock := StrToIntDef(Copy(SendStr, 23, 5), -1);
          CountSuccBlock := StrToIntDef(Copy(SendStr, 29, 5), -1);
          CountErrorBlock := StrToIntDef(Copy(SendStr, 35, 5), -1);
          CountFreeBlock := StrToIntDef(Copy(SendStr, 41, 5), -1);
          Result := True;
        end;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    //$21      

function TE950Driver.ReportTestFMem(var CountBlock,
  CountSuccBlock, CountErrorBlock,
  CountFreeBlock: Integer): Boolean;
var
  SendStr: string;
begin
  Result := False;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($21) +
        Format('%-*s', [8, WinToDos(FPassword)]) + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        if CheckAnswer(SendStr, $21)
          then
        begin
          CountBlock := StrToIntDef(Copy(SendStr, 23, 5), -1);
          CountSuccBlock := StrToIntDef(Copy(SendStr, 29, 5), -1);
          CountErrorBlock := StrToIntDef(Copy(SendStr, 35, 5), -1);
          CountFreeBlock := StrToIntDef(Copy(SendStr, 41, 5), -1);
          Result := True;
        end;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $22,  .

procedure TE950Driver.ReportDiag;
var
  SendStr: string;
begin
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($22) +
        Format('%-*s', [8, WinToDos(FPassword)]) + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      CheckAnswer(SendStr, $22);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $2c,      .

function TE950Driver.SetBlockWorkMem(const SetOf: Integer;
  const InfoBlock: string): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $2d,      .

function TE950Driver.GetBlockWorkMem(const SetOf: Integer;
  const LenBlock: Byte; var InfoBlock: string): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $2e,    .

procedure TE950Driver.PrintTestPrinter;
var
  SendStr: string;
begin
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($2E) +
        Format('%-*s', [8, WinToDos(FPassword)]) + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        CheckAnswer(SendStr, $2E);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $2f,   .

procedure TE950Driver.ResetRegistrator;
var
  SendStr: string;
begin
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($2F) +
        Format('%-*s', [8, WinToDos(FPassword)]) + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        CheckAnswer(SendStr, $2F);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $2a,    .

function TE950Driver.GetStatusRegistrator(Info: PStatusRegistr): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    {    .}
     {  .}
    // $30,   .

function TE950Driver.GetDmpSertificate(SendToPrint: Boolean; SertificateDmp: PSertificate): Boolean;
var
  SendStr, Str: string;
  Month, Day, Year: word;
begin
  Result := False;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($30) +
        Format('%-*s', [8, WinToDos(FPassword)]) + DBit;
      if SendToPrint
        then
        SendStr := SendStr + Chr($50)
      else
        SendStr := SendStr + Chr($20);
      SendStr := SendStr + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        Result := CheckAnswer(SendStr, $30);
      if Result
        then
      begin
        Str := Copy(SendStr, 23, Pos(#$03, SendStr) - 23);
        SertificateDmp^.SN := Copy(Str, 1, Pos(DBit, SendStr) - 1);
        Delete(Str, 1, Pos(DBit, Str));
        SertificateDmp^.PosPoint := Byte(StrToInt(Copy(Str, 1, Pos(DBit, Str) - 1)));
        Delete(Str, 1, Pos(DBit, Str));
        SertificateDmp^.Devaluations := Byte(StrToInt(Copy(Str, 1, Pos(DBit, Str) - 1)));
        Delete(Str, 1, Pos(DBit, Str));
        SertificateDmp^.CurrDef := Byte(StrToInt(Copy(Str, 1, Pos(DBit, Str) - 1)));
        Delete(Str, 1, Pos(DBit, Str));
        SertificateDmp^.TypePrn := Byte(StrToInt(Copy(Str, 1, Pos(DBit, Str) - 1)));
        Delete(Str, 1, Pos(DBit, Str));
        Year := StrToInt(Copy(Str, 1, 2));
        Month := StrToInt(Copy(Str, 3, 2));
        Day := StrToInt(Copy(Str, 5, 2));
        SertificateDmp^.DateSer := EncodeDate(Year, Month, Day);
      end;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $31,   .

function TE950Driver.LoadSertificate(Sertificate: PSertificate): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $32, .

procedure TE950Driver.Certification;
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

  {  .}
    // $33,   .

function TE950Driver.GetDmpFiskal(SendToPrint: Boolean; FiskalDmp: PFiskal): Boolean;
var
  SendStr, Str: string;
  Month, Day, Year: word;
begin
  Result := False;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($33) +
        Format('%-*s', [8, WinToDos(FPassword)]) + DBit;
      if SendToPrint
        then
        SendStr := SendStr + Chr($50)
      else
        SendStr := SendStr + Chr($20);
      SendStr := SendStr + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        if CheckAnswer(SendStr, $33)
          then
        begin
          Str := Copy(SendStr, 23, Pos(#$03, SendStr) - 23);
          FiskalDmp^.RN := Copy(Str, 1, Pos(DBit, Str) - 1);
          Delete(Str, 1, Pos(DBit, Str));
          FiskalDmp^.UserFR := Copy(Str, 1, Pos(DBit, Str) - 1);
          Delete(Str, 1, Pos(DBit, Str));
          FiskalDmp^.PasswordFiskal := Copy(Str, 1, Pos(DBit, Str) - 1);
          Delete(Str, 1, Pos(DBit, Str));
          FiskalDmp^.PasswordUser := Copy(Str, 1, Pos(DBit, Str) - 1);
          Delete(Str, 1, Pos(DBit, Str));
          Year := StrToInt(Copy(Str, 1, 2));
          Month := StrToInt(Copy(Str, 3, 2));
          Day := StrToInt(Copy(Str, 5, 2));
          FiskalDmp^.DateReg := EncodeDate(Year, Month, Day);
          Result := True;
        end;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $34,   .

function TE950Driver.LoadFiskal(FiskalDmp: PFiskal): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $35, .

procedure TE950Driver.Fiskalization;
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $37,   

function TE950Driver.GetFMemSize(FMemSize: PFMemSize): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $39,  .

function TE950Driver.CloseSmena(Fix: Boolean; TotalSmena: PSmenaClose): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $3a,    .

procedure TE950Driver.ReportOfDates(const BeginDate, EndDate: TDate; ItogOnly: Boolean);
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $3b,      .

procedure TE950Driver.ReportOfSmena(const BeginSmena, EndSmena: TDate; ItogOnly: Boolean);
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    //  $3f,   .

function TE950Driver.ChangePassword(const PasswordOld, PasswordNew: string): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

 {   .}
    // $40,   .

function TE950Driver.OpenCheck(TypeCheck: TTypeCheck; const IndxCurr: Word): Boolean;
var
  SendStr: string;
begin
  Result := False;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($40) +
        Format('%-*s', [8, WinToDos(FPassword)]) + DBit;
      case TypeCheck of
        Service: SendStr := SendStr + Chr($53);
        ServiceBack: SendStr := SendStr + Chr($52);
        Payment: SendStr := SendStr + Chr($42);
        PaymentBack: SendStr := SendStr + Chr($43);
        Cashe: SendStr := SendStr + Chr($50);
        CasheBack: SendStr := SendStr + Chr($54);
      end;
      SendStr := SendStr + DBit + '1';
      SendStr := SendStr + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        Result := CheckAnswer(SendStr, $40);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $41,   .

procedure TE950Driver.PrintFiskalTxt(const IndxDisc: Word; const FiskalTxt: string);
var
  SendStr: string;
begin
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($41) +
        Format('%-*s', [8, WinToDos(FPassword)]) + DBit;
      SendStr := SendStr + IntToStr(IndxDisc) + DBit;
      SendStr := SendStr + Format('%-*s', [20, WinToDos(FiskalTxt)]) + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        CheckAnswer(SendStr, $41);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $42,    .

procedure TE950Driver.PrintFiskalStr(FislakStr: PFiskalStr);
var
  SendStr: string;
begin
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($42) +
        Format('%-*s', [8, WinToDos(FPassword)]) + DBit;
      SendStr := SendStr + IntToStr(FislakStr^.IndxDiscStr) + DBit;
      SendStr := SendStr + WinToDos(FislakStr^.DefineDiscStr + DBit);
      SendStr := SendStr + IntToStr(FislakStr^.Count) + DBit;
      SendStr := SendStr + IntToStr(FislakStr^.IndxDiscUM) + DBit;
      SendStr := SendStr + IntToStr(FislakStr^.Priece) + DBit;
      SendStr := SendStr + IntToStr(FislakStr^.IndxDiscCurr) + DBit;
      SendStr := SendStr + FislakStr^.IndxStr + DBit;
      SendStr := SendStr + IntToStr(FislakStr^.ProcMargin) + DBit;
      SendStr := SendStr + IntToStr(FislakStr^.ProcDiscount);
      SendStr := SendStr + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        CheckAnswer(SendStr, $42);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $43,   .

function TE950Driver.SubTotal(PrintSubTotal: Boolean;
  SubTotal: PSubTotal; SubTotalAnswer: PSubTotalAnswer): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $44,   .

function TE950Driver.CheckTotal(const InSum: Cardinal; const IndxCurr: Word;
  IndxOpl: Byte; DefOpl: string; Tender: Boolean; var Sum: Cardinal): Boolean;
var
  SendStr: string;
begin
  Result := False;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($44) +
        Format('%-*s', [8, WinToDos(FPassword)]) + DBit;
      SendStr := SendStr + IntToStr(IndxOpl) + DBit;
      SendStr := SendStr + WinToDos(DefOpl) + DBit;
      SendStr := SendStr + IntToStr(InSum) + DBit;
      SendStr := SendStr + IntToStr(IndxCurr) + DBit;
      if Tender
        then
        SendStr := SendStr + Chr($54)
      else
        SendStr := SendStr + Chr($43);
      SendStr := SendStr + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        Result := CheckAnswer(SendStr, $44);
      if Result
        then
      begin
        Delete(SendStr, 1, 22);
        Delete(SendStr, Pos(Chr($03), SendStr), 5);
        Sum := StrToIntDef(SendStr, -1);
      end;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $45,   .

function TE950Driver.CloseCheck(var LastCheck: Cardinal): Boolean;
var
  SendStr: string;
begin
  Result := False;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($45) +
        Format('%-*s', [8, WinToDos(FPassword)]);
      SendStr := SendStr + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        Result := CheckAnswer(SendStr, $45);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

  {  .}
    // $48,   .

function TE950Driver.OpenNFCheck(const IndxNFDok: Word;
  DefineDiscNFDok: string): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $49,   .

procedure TE950Driver.PrintNFText(const Str: string);
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $4a,   .

procedure TE950Driver.CloseNFCheck;
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $4b,   .

procedure TE950Driver.CutNFCheck;
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

  {  .}
    // $50,   .

procedure TE950Driver.FeedLineCheck(const CountLine: Word);
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $51,   .

function TE950Driver.FeedLineLog(CountLine: Word): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $52,     .

function TE950Driver.FeedLineBoth(CountLine: Word): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $53,   .

function TE950Driver.FeedLineDok(CountLine: Word): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $54,   .

procedure TE950Driver.EjectDok;
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $55,  Slip- .

function TE950Driver.ConditionSLIP(
  CountFreeLine, Pos1, Pos2: Word; var NumDok: Cardinal): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $57,   .

function TE950Driver.FontPrn(Font: TFontPrn): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

  { ,    .}
    // $58,    .

procedure TE950Driver.SetDateTime(ADateTime: TDateTime);
var
  SendStr: string;
  TimeStr, DateStr: string;
begin
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      DateTimeToString(TimeStr, 'hhmmss', ADateTime);
      DateTimeToString(DateStr, 'yymmdd', ADateTime);
      SendStr := BBit + Chr(FEnumComm) + Chr($58) +
        Format('%-*s', [8, WinToDos(FPassword)]) +
      DBit + DateStr + DBit + TimeStr + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
     {if not} CheckAnswer(SendStr, $58);
//      then
//      raise Exception.Create('     ');
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $59,        .

function TE950Driver.GetDateTime: TDateTime;
var
  SendStr: string;
  Month, Day, Year: Word;
  Hour, Min, Sec: Word;
begin
  Result := 0;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($59) +
        Format('%-*s', [8, WinToDos(FPassword)]) + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        if CheckAnswer(SendStr, $59)
          then
        begin
          Hour := StrToInt(Copy(SendStr, 30, 2));
          Min := StrToInt(Copy(SendStr, 32, 2));
          Sec := StrToInt(Copy(SendStr, 34, 2));
          Year := StrToInt(Copy(SendStr, 23, 2));
          if Year < 99
            then
            Year := 2000 + Year
          else
            Year := 1900 + Year;
          Month := StrToInt(Copy(SendStr, 25, 2));
          Day := StrToInt(Copy(SendStr, 27, 2));
          Result := EncodeDate(Year, Month, Day) + EncodeTime(Hour, Min, Sec, 0);
        end;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

  {  Slip-.}
    // $60,    .

function TE950Driver.OpenFDok(TDok: PTypeDok): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $61,    .

function TE950Driver.PrintDefineFDok(IndxDisc: Word;
  DefineFDok: string): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $62,    .

function TE950Driver.PrintStrFDok(DokStr: PFDokStr): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $63,    

function TE950Driver.SubTotalFDok(SubTotal: PSubTotalFDok;
  SubTotalAnswer: PSubTotalAnswer): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $64,    .

function TE950Driver.TotalFDok(TotalFDok: PTotalFDok; var Sum: Currency): Boolean;
begin
  Result := False;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $65,    .

function TE950Driver.CloseFDok: Cardinal;
begin
  Result := 0;
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

  {   .}
    // $68,    .

procedure TE950Driver.OpenNFDok;
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $69,     .

procedure TE950Driver.PrintNFDokStr(const Str: string);
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $6a,    .

procedure TE950Driver.CloseNFDok;
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

{  .}

 // $70,    COM 1.
 // $71,   COM 1.
 // $72,     COM 1.
 // $73,     COM 1.
 // $74,    COM 2.
 // $75,   COM 2.
 // $76,     COM 2.
 // $77,     COM 2.

{  .}

 // $7b,    .
 // $7c,    .

   { .}
    // $6e,  .

function TE950Driver.GetDefineDisc(const NumTbl, NumDisc: Word): string;
var
  SendStr: string;
begin
  Result := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($6E) +
        Format('%-*s', [8, WinToDos(FPassword)]) + DBit +
      IntToStr(NumTbl) + DBit + IntToStr(NumDisc) + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        if CheckAnswer(SendStr, $6E)
          then
        begin
          Delete(SendStr, 1, 22);
          Result := Copy(SendStr, 1, Pos(EBit, SendStr) - 1);
          Result := DosToWin(Result);
        end;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $6f,     .

procedure TE950Driver.SetDefineDisc(NumTbl, NumDisc: Word; const DefineDisc: string);
var
  SendStr: string;
begin
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($6F) +
        Format('%-*s', [8, WinToDos(FPassword)]) + DBit +
      IntToStr(NumTbl) + DBit + IntToStr(NumDisc) +
        DBit + WinToDos(DefineDisc) + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
     {if} CheckAnswer(SendStr, $6F);
      //then
      //raise Exception.Create('      ');
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;


function TE950Driver.GetTreningState: Boolean;
var
  SendStr: string;
begin
  Result := False;
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($3E) +
        Format('%-*s', [8, WinToDos(FPassword)]) + DBit + Chr($51) + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        if CheckAnswer(SendStr, $3E)
          then
          Result := SendStr[23] = Chr($53);
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

procedure TE950Driver.SetTreningState(State: Boolean);
var
  SendStr: string;
begin
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($3E) +
        Format('%-*s', [8, WinToDos(FPassword)]) + DBit;
      if State
        then
        SendStr := SendStr + Chr($53)
      else
        SendStr := SendStr + Chr($52);
      SendStr := SendStr + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
    {if not} CheckAnswer(SendStr, $3E);
//     then
//      raise Exception.Create('    ');
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $5a,   .

procedure TE950Driver.SetCashire(CashireName: string);
var
  SendStr: string;
begin
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($5A) +
        Format('%-*s', [8, WinToDos(FPassword)]) + DBit +
      WinToDos(CashireName) + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
     {if not}
        CheckAnswer(SendStr, $5A);
    // then raise Exception.Create('    ');
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $5b,   .

function TE950Driver.GetCashire: string;
var
  SendStr: string;
begin
  Result := '';
  try
    EnterCriticalSection(FCritSect);
    try
      SetEnumComm;
      SendStr := BBit + Chr(FEnumComm) + Chr($5B) +
        Format('%-*s', [8, WinToDos(FPassword)]) + EBit;
      SendStr := AddCheckSum(SendStr);
      SendDataEx(PChar(SendStr), Length(SendStr), FOutputTimeout);
      SendStr := ReadAnswer();
      if SendStr = ''
        then
        raise Exception.Create(SErrorNotAnswer);
      if (SendStr[1] <> Chr($15))
        then
        if CheckAnswer(SendStr, $5B)
          then
        begin
          Delete(SendStr, 1, 22);
          Result := Copy(SendStr, 1, Pos(#$03, SendStr) - 1);
          Result := DosToWin(Result);
        end;
    finally
      LeaveCriticalSection(FCritSect);
    end;
  except
    raise;
  end;
end;

    // $5D,   .

procedure TE950Driver.SetHeader(Header: string);
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

    // $5E,    .

function TE950Driver.GetHeader: string;
begin
  EnterCriticalSection(FCritSect);
  try
  finally
    LeaveCriticalSection(FCritSect);
  end;
end;

end.

