{
  Programm : LED.PAS
  Sprache  : Delphi 1.0
  Zweck    : LED VC
  Datum    : 9. Mrz 1996
  Update   : 11. Juli 1998
}

unit
  LED;

interface

uses
  Windows, Classes, Graphics, Controls, ExtCtrls;
{------------------------------------------------------------------------------}

type
  TLEDType = (ltRound, ltRect, ltLeft, ltRight, ltUp, ltDown);
  TBevelWidth = 0..5;
  TInterval = 0..MaxInt;
  TOnChange = procedure(Sender: TObject; IsOn: Boolean) of object;

  TLED = class(TCustomControl)
  private
    FTimer: TTimer;
    FBevelOuter: TPanelBevel;
    FBevelInner: TPanelBevel;
    FLEDBkGndCol: TColor;
    FLEDOnCol: TColor;
    FLEDOffCol: TColor;
    FLEDBorderCol: TColor;
    FLEDType: TLEDType;
    FBevelWidth: TBevelWidth;
    FLEDOn: Boolean;
    FToggleOnClick: Boolean;
    FTransparent: Boolean;
    FLEDInterval: TInterval;
    FOnChange: TOnChange;
    Border: Byte;
  protected
    procedure DrawBevel(Canvas: TCanvas);
    procedure SetColor(Index: Integer; Value: TColor);
    procedure SetBevelOuter(Value: TPanelBevel);
    procedure SetBevelInner(Value: TPanelBevel);
    procedure SetBevelWidth(Value: TBevelWidth);
    procedure SetBool(Index: Integer; Value: Boolean);
    procedure SetLEDType(Value: TLEDType);
    procedure SetInterval(Value: TInterval);
    procedure OnTimer(Sender: TObject);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
  published
    property Align;
    property Anchors;
    property Hint;
    property ShowHint;
    property OnClick;
    property BevelOuter: TPanelBevel read FBevelOuter write SetBevelOuter;
    property BevelInner: TPanelBevel read FBevelInner write SetBevelInner;
    property BevelWidth: TBevelWidth read FBevelWidth write SetBevelWidth;
    property LEDBkGndCol: TColor index 1 read FLEDBkGndCol write SetColor;
    property LEDOnCol: TColor index 2 read FLEDOnCol write SetColor;
    property LEDOffCol: TColor index 3 read FLEDOffCol write SetColor;
    property LEDBorderCol: TColor index 4 read FLEDBorderCol write SetColor;
    property LEDOn: Boolean index 1 read FLEDOn write SetBool;
    property ToggleOnClick: Boolean index 2 read FToggleOnClick write SetBool;
    property Transparent: Boolean index 3 read FTransparent write SetBool;
    property LEDType: TLEDType read FLEDType write SetLEDType;
    property LEDInterval: TInterval read FLEDInterval write SetInterval;
    property OnChange: TOnChange read FOnChange write FOnChange;
  end;
{------------------------------------------------------------------------------}

procedure Register;

implementation
{------------------------------------------------------------------------------}

constructor TLED.Create(AOwner: TComponent);
begin
  ControlState := ControlState + [csCreating];
  inherited Create(AOwner);
  FBevelOuter := bvNone;
  FBevelInner := bvNone;
  FBevelWidth := 1;
  FLEDOn := False;
  FLEDBkGndCol := clBtnFace;
  FLEDBorderCol := clGray;
  FLEDOnCol := clRed;
  FLEDOffCol := clMaroon;
  FLEDType := ltRound;
  FTransparent := True;
  FLEDInterval := 0;
  FToggleOnClick := False;
  FTimer := nil;
  Width := 15;
  Height := 15;
  if FBevelOuter <> bvNone then Border := FBevelWidth else Border := 0;
  if FBevelInner <> bvNone then Inc(Border, FBevelWidth);
  ControlState := ControlState - [csCreating];
end;
{------------------------------------------------------------------------------}

destructor TLed.Destroy;
begin
  if Assigned(FTimer) then FTimer.Free;
  inherited Destroy;
end;
{------------------------------------------------------------------------------}

procedure TLED.DrawBevel(Canvas: TCanvas);
var
  TopColor: TColor;
  BottomColor: TColor;
  Rect: TRect;

  procedure SetColors(Bevel: TPanelBevel);
  begin
    TopColor := clBtnHighlight;
    if Bevel = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if Bevel = bvLowered then BottomColor := clBtnHighlight;
  end;

begin
  Rect := GetClientRect;
  if FBevelOuter <> bvNone then
  begin
    SetColors(FBevelOuter);
    Frame3D(Canvas, Rect, TopColor, BottomColor, FBevelWidth);
  end;

  if FBevelInner <> bvNone then
  begin
    SetColors(FBevelInner);
    Frame3D(Canvas, Rect, TopColor, BottomColor, FBevelWidth);
  end;
end;
{------------------------------------------------------------------------------}

procedure TLED.Paint;
var
  R: TRect;
  P: array[0..3] of TPoint;
  Bmp: TBitMap;
begin
  if (csCreating in ControlState) or (csCreating in Parent.ControlState) then Exit;

  R := GetClientRect;
  Bmp := TBitMap.Create;
  try
    Bmp.Width := Width;
    Bmp.Height := Height;
    Bmp.Canvas.CopyRect(R, Canvas, R);
    if not FTransparent then
    begin
      DrawBevel(Bmp.Canvas);
      Bmp.Canvas.Pen.Color := FLEDBkGndCol;
      Bmp.Canvas.Brush.Color := FLEDBkGndCol;
      InflateRect(R, -Border, -Border);
      Bmp.Canvas.FillRect(R);
    end;
    if not FTransparent then InflateRect(R, -Border, -Border);

    if FLEDOn then Bmp.Canvas.Brush.Color := FLEDOnCol else Bmp.Canvas.Brush.Color := FLEDOffCol;

    Bmp.Canvas.Pen.Color := FLEDBorderCol;
    Bmp.Canvas.Pen.Width := 1;
    case FLEDType of
      ltRound: begin
          Bmp.Canvas.Ellipse(R.Left, R.Top, R.Right, R.Bottom);
          P[0].x := R.Left + (R.Right - R.Left) div 4;
          P[0].y := R.Top + (R.Bottom - R.Top) div 4;
          P[1].x := P[0].x + (R.Right - R.Left) div 3;
          P[1].y := P[0].y + (R.Bottom - R.Top) div 3;
          P[2].x := P[1].x;
          P[2].y := P[0].y;
          P[3].x := P[0].x;
          P[3].y := P[1].y;
          if FLEDOn then
          begin
            Bmp.Canvas.Pen.Color := clWhite;
            Bmp.Canvas.Brush.Color := clWhite;
          end
          else
          begin
            Bmp.Canvas.Pen.Color := clGray;
            Bmp.Canvas.Brush.Color := clGray;
          end;
          Bmp.Canvas.Chord(P[0].x, P[0].y, P[1].x, P[1].y, P[2].x, P[2].y, P[3].x, P[3].y);
        end;
      ltRect: begin
          case BevelInner of
            bvRaised: Frame3D(Bmp.Canvas, R, clBlack, clWhite, 1);
            bvLowered: Frame3D(Bmp.Canvas, R, clWhite, clBlack, 1);
            bvNone: if FBevelOuter = bvLowered then Frame3D(Bmp.Canvas, R, clWhite, clBlack, 1)
              else Frame3D(Canvas, R, clBlack, clWhite, 1);
          end;
          Bmp.Canvas.FillRect(R);
        end;
      ltLeft: begin
          InflateRect(R, -Border, -Border);
          P[0].x := R.Left;
          P[0].y := R.Top + ((R.Bottom - R.Top) div 2);
          P[1].x := R.Right - 1;
          P[1].y := R.Top;
          P[2].x := R.Right - 1;
          P[2].y := R.Bottom;
          P[3].x := P[0].x;
          P[3].y := P[0].y;
          Bmp.Canvas.Polygon(P);
        end;
      ltRight: begin
          InflateRect(R, -Border, -Border);
          P[0].x := R.Right;
          P[0].y := R.Top + ((R.Bottom - R.Top) div 2);
          P[1].x := R.Left;
          P[1].y := R.Top;
          P[2].x := R.Left;
          P[2].y := R.Bottom;
          P[3].x := P[0].x;
          P[3].y := P[0].y;
          Bmp.Canvas.Polygon(P);
        end;
      ltUp: begin
          InflateRect(R, -Border, -Border);
          P[0].x := R.Left + ((R.Right - R.Left) div 2);
          P[0].y := R.Top;
          P[1].x := R.Left;
          P[1].y := R.Bottom - 1;
          P[2].x := R.Right;
          P[2].y := R.Bottom - 1;
          P[3].x := P[0].x;
          P[3].y := P[0].y;
          Bmp.Canvas.Polygon(P);
        end;
      ltDown: begin
          InflateRect(R, -Border, -Border);
          P[0].x := R.Left + ((R.Right - R.Left) div 2);
          P[0].y := R.Bottom;
          P[1].x := R.Left;
          P[1].y := R.Top;
          P[2].x := R.Right;
          P[2].y := R.Top;
          P[3].x := P[0].x;
          P[3].y := P[0].y;
          Bmp.Canvas.Polygon(P);
        end;
    end;
    R := GetClientRect;
    Canvas.CopyRect(R, Bmp.Canvas, R);
  finally
    Bmp.Free;
  end;
end;
{------------------------------------------------------------------------------}

procedure TLED.SetBevelOuter(Value: TPanelBevel);
begin
  if FBevelOuter <> Value then
  begin
    FBevelOuter := Value;
    if FBevelOuter <> bvNone then Border := FBevelWidth else Border := 0;
    if FBevelInner <> bvNone then Inc(Border, FBevelWidth);
    Invalidate;
  end;
end;
{------------------------------------------------------------------------------}

procedure TLED.SetBevelInner(Value: TPanelBevel);
begin
  if FBevelInner <> Value then
  begin
    FBevelInner := Value;
    if FBevelOuter <> bvNone then Border := FBevelWidth else Border := 0;
    if FBevelInner <> bvNone then Inc(Border, FBevelWidth);
    Invalidate;
  end;
end;
{------------------------------------------------------------------------------}

procedure TLED.SetBevelWidth(Value: TBevelWidth);
begin
  if FBevelWidth <> Value then
  begin
    FBevelWidth := Value;
    if FBevelOuter <> bvNone then Border := FBevelWidth else Border := 0;
    if FBevelInner <> bvNone then Inc(Border, FBevelWidth);
    Invalidate;
  end;
end;
{------------------------------------------------------------------------------}

procedure TLED.SetBool(Index: Integer; Value: Boolean);
begin
  case Index of
    1: if FLEDOn <> Value then
      begin
        FLEDOn := Value;
        Paint;
      end;
    2: if Value <> FToggleOnClick then FToggleOnClick := Value;
    3: if Value <> FTransparent then
      begin
        FTransparent := Value;
        Invalidate;
      end;
  end;
end;
{------------------------------------------------------------------------------}

procedure TLED.SetColor(Index: Integer; Value: TColor);
begin
  case Index of
    1: FLEDBkGndCol := Value;
    2: FLEDOnCol := Value;
    3: FLEDOffCol := Value;
    4: FLEDBorderCol := Value;
  end;
  Invalidate;
end;
{------------------------------------------------------------------------------}

procedure TLED.SetLEDType(Value: TLEDType);
begin
  if FLEDType <> Value then
  begin
    FLEDType := Value;
    Invalidate;
  end;
end;
{------------------------------------------------------------------------------}

procedure TLED.SetInterval(Value: TInterval);
begin
  if FLEDInterval <> Value then
  begin
    FLEDInterval := Value;
    if (FLEDInterval = 0) and Assigned(FTimer) then
    begin
      FTimer.Free;
      FTimer := nil;
    end
    else if (FLEDInterval <> 0) and Assigned(FTimer) then FTimer.Interval := FLEDInterval
    else if (FLEDInterval <> 0) and not Assigned(FTimer) then
    begin
      FTimer := TTimer.Create(nil);
      FTimer.Interval := FLEDInterval;
      FTimer.OnTimer := OnTimer;
    end;
  end;
end;
{------------------------------------------------------------------------------}

procedure TLED.OnTimer(Sender: TObject);
begin
  FLEDOn := FLEDOn xor True;
  Paint;
  if Assigned(FOnChange) then FOnChange(Self, FLEDOn);
end;
{------------------------------------------------------------------------------}

procedure TLED.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and FToggleOnClick then
  begin
    FLEDOn := FLEDOn xor True;
    Paint;
    if Assigned(FOnChange) then FOnChange(Self, FLEDOn);
  end;
end;
{------------------------------------------------------------------------------}

procedure Register;
begin
  RegisterComponents('Udo', [TLED]);
end;
{------------------------------------------------------------------------------}

initialization
end.

